/*******************************************************************************
*
*  DSPISRs.dsp
*
*  (c) 2002-2004 ethan bordeaux
*  syndevkit@dspmusic.org
*
*
*******************************************************************************/

.MODULE/RAM                                 modDSPISRs;

#include ".\chicsys\hdr\chichdr.h"
#include ".\comsys\hdr\comhdr.h"
#include ".\gen_fx\hdr\GenFX.h"

.ENTRY      GetSamples;
.ENTRY      TX0ISR;
.ENTRY      GetRandomVal;

/****************************************************************************
 *  Serial Port 0 - Receive Interrupt service routines
 ****************************************************************************/

GetSamples:
   rti;
     
/****************************************************************************
 *  Serial Port 0 Transmit -- interrupt service routine 
 ****************************************************************************/

TX0ISR:
	ena SEC_REG, ena AR_SAT;
	
    AR = DM(b_CodecInitialized);
    AR = pass AR;
    if EQ rti;
	
    MY0 = I7;                               /* save I7, not in SEC_REG */
    MY1 = L7;                               /* save L7, not in SEC_REG */

    /* read L/R inputs into a_LAudioInBuff, a_RAudioInBuff */
    I7 = DM(p_LInAudioInBuff);
    L7 = %a_LAudioInBuff;
    AR = DM(a_rxBuf + MASTER_LEFT);
    DM(I7, M7) = AR;
    DM(p_LInAudioInBuff) = I7;
    
    I7 = DM(p_RInAudioInBuff);
    !L7 = %a_RAudioInBuff;                  /* R and L buffers same len */
    AR = DM(a_rxBuf + MASTER_RIGHT);
    DM(I7, M7) = AR;
    DM(p_RInAudioInBuff) = I7;
    
    /* enable left and right channel master output */
    MR0 = 0x9800;				
    dm(a_txBuf + TAG_PHASE) = MR0;
    
    /* read output from buffer earmarked for transmission */
    I7 = DM(p_LBuffToTX);
    L7 = OUTBUFFLEN;
    AR = DM(I7, M7);
    DM(p_LBuffToTX) = I7;                   /* p_LBuff++ */
    
    /* if b_AllMute==1, no output */
    SR0 = DM(b_AllMute);
    AF = pass SR0;
    if NE AR = pass 0;
        
    DM(a_txBuf + MASTER_LEFT) = AR;         /* send left channel data */
    
    I7 = DM(p_RBuffToTX);
    !L7 = OUTBUFFLEN;                       /* L7 already set for left buff */
    AR = DM(I7, M7);
    DM(p_RBuffToTX) = I7;                   /* p_RBuff++ */
    
    /* if b_AllMute==1, no output */
    SR0 = DM(b_AllMute);
    AF = pass SR0;
    if NE AR = pass 0;
    AR = pass AR;
    
    DM(a_txBuf + MASTER_RIGHT) = AR;          /* send right channel data */    
    
    /* check if end of transmission buffer was reached */
    AY0 = ^a_ROutBuff0;
    AX0 = DM(v_TXBuff0or1);
    AR = pass AX0;
    if EQ jump CheckBuffWraparound;
    
    AY0 = ^a_ROutBuff1;                     /* oops, it's Buff1 */    
    
CheckBuffWraparound:
    AX1 = I7;
    AR = AX1 - AY0;
    if NE jump EndSendMusic;
    
/*
 *  set FillNewBuffer bit, swap buffer pointers, and toggle OutBuff0or1
 *  indicator.
 */
    
SwapBuffers:
    AR = tglbit 0 of AX0;                   /* AX0=v_TXBuff0or1 */
    DM(v_TXBuff0or1) = AR;
    DM(b_FillNewBuffer) = M1;               /* time to fill data buff */
    
    AY0 = ^a_LOutBuff0;
    AY1 = ^a_ROutBuff0;
    NONE = pass AR;
    if EQ jump SetBufferPtr;
    
    AY0 = ^a_LOutBuff1;
    AY1 = ^a_ROutBuff1;
    
SetBufferPtr:
    DM(p_LBuffToTX) = AY0;
    DM(p_RBuffToTX) = AY1;
    
EndSendMusic:
    I7 = MY0;
    L7 = MY1;
    
	rti;

/******************************************************************************
*
*  IRQE ISR.  provide seed for random number generator
*
******************************************************************************/

GetRandomVal:
    ena SEC_REG;

    AR = DM(TCOUNT);
    DM(v_LSWSeed) = AR;
        
    DM(b_SeedCaptured) = M1;

    call IRQEProc;                      /* additional IRQE processing */

    rti;

.ENDMOD;
