/*******************************************************************************
*
*  DSPInit.dsp
*
*  (c) 2002-2004 ethan bordeaux
*  syndevkit@dspmusic.org
*
*
*******************************************************************************/

.module                                     modDSPInit;

#include ".\chicsys\hdr\chichdr.h"
#include ".\comsys\hdr\comhdr.h"
#include ".\gen_fx\hdr\GenFX.h"

.ENTRY      InitSport0;
.ENTRY      InitSystemRegs;
.ENTRY      InitMemory;
.ENTRY      CBRC;

InitMemory:
    /* always assume linear addressing, code must always reset L-reg to zero */
    L2 = 0;
    L3 = 0;
    L4 = 0;
    L5 = 0;
	
	/* these M-reg values can never be changed */
    M0 = 0;
    M1 = 1;
    M6 = 0;
    M7 = 1;
	
	/* at powerup, codec not initialized, mute output */
    DM(b_CodecInitialized) = M0;
    DM(b_AllMute) = M1;
	
    PMOVLAY = 0;
    DMOVLAY = 0;
 
    i0 = ^a_rxBuf;           		/* SPORT0 rcv autobuffer */
    l0 = %a_rxBuf;
	
    i1 = ^a_txBuf;           		/* SPORT0 xmt autobuffer */
    l1 = %a_txBuf;
  
    i4 = ^a_rxBuf;
    cntr = 16;
    do CRB until ce;
CRB:  DM(I4, M7) = 0;

	DM(b_EndSong) = M0;
	
	/* set up ptrs into input audio buff and clear input buffer */
	AR = ^a_LAudioInBuff+1;
	DM(p_LOutAudioInBuff) = AR;
	AR = ^a_LAudioInBuff+0;
	DM(p_LInAudioInBuff) = AR;
	
	I2 = ^a_LAudioInBuff;
	CNTR = %a_LAudioInBuff;
	do CLIB until CE;
CLIB: DM(I2, M1) = 0;

	AR = ^a_RAudioInBuff+1;
	DM(p_ROutAudioInBuff) = AR;
	AR = ^a_RAudioInBuff+0;
	DM(p_RInAudioInBuff) = AR;
	
	I2 = ^a_RAudioInBuff;
	CNTR = %a_RAudioInBuff;
	do CRIB until CE;
CRIB: DM(I2, M1) = 0;
   
    rts;
   
InitSport0:
    AX0 = RX_AUTOBUF |
          TX_AUTOBUF |
          RX_MREG_M1 |
          RX_IREG_I0 |
          TX_MREG_M1 |
          TX_IREG_I1 |
          DIS_MAC_BIASRND |
          ENA_CLKOUT;

    DM(SPORT0_AUTOBUF) = AX0;

    /* set RFSDIV for 256 cycles, where RFSDIV = SCLK Hz/RFS Hz - 1 */
    AX0 = 0x00FF;
    DM(SPORT0_RFSDIV) = AX0;
   
   /* externally generated clock, no divider */
    AX0 = 0;
    DM(SPORT0_SCLKDIV) = AX0;

    AX0 = SERIAL_16BITS |
          RIGHT_JUSTIFY_ZERO_FILL |
          RFS_ACTIVE_HIGH |
          TFS_ACTIVE_HIGH |
          RFS_INTERNAL |
          TDM_32_CHANNELS |
          FRAME_DELAY1 |
          ENABLE_MULTICHANNEL;
          
    DM(SPORT0_CONTROL) = AX0;

	/* sport0 receive and transmit multichannel word enable registers */
	AX0 = TX_ENA_CHANNEL1_16;
	DM(SPORT0_TX_CHANNELS0) = AX0;
	
	AX0 = TX_ENA_CHANNEL17_32;
	DM(SPORT0_TX_CHANNELS1) = AX0;
	
	AX0 = RX_ENA_CHANNEL1_16;
	DM(SPORT0_RX_CHANNELS0) = AX0;
	
	AX0 = RX_ENA_CHANNEL17_32;
	DM(SPORT0_RX_CHANNELS1) = AX0;
	
    rts;

InitSystemRegs:
    AX0 = b#0001111111111111;
    DM(WAIT_STATE) = AX0;

    /* SPORT1 is now FI, FO, IRQ1, IRQ0, SCLK */
    AX0 = b#0000000000000000;
    DM(SYSTEM_CONTROL) = AX0;
    
    /* clear pending interrupts */
    IFC = 0x00ff;         
    nop;

    /* IRQ2 edge sensitive, no IRQ nesting */
    icntl = b#00100;

    /* ALU saturation, go mode, fractional multiplies */
    ena AR_SAT, ena G_MODE;
   
    rts;

/*
 *  toggle reset line HLH on codec.  it's weird but necessary!
 */

CBRC:
    set FL2;
    cntr = 1000;
    do R1 until CE;
R1:    nop;

    reset FL2;
    cntr = 1000;
    do R2 until CE;
R2:    nop;

    set FL2;
    cntr = 1000;
    do R3 until CE;
R3:    nop;

    rts;
        
.ENDMOD;
