/******************************************************************************
*
*  ADSRPanEnv.dsp
*
*  (c) 2001, 2002, 2003 ethan bordeaux
*  syndevkit@dspmusic.org
*
*
******************************************************************************/

.MODULE                                     modADSRPanEnv;

/*
 *  include files
 */


#include    ".\comsys\hdr\DSPdefs.h"
#include    ".\gen_fx\hdr\GenFX.h"

/*
 *  entry points
 */

.ENTRY      ADSRPanEnv;

/*
 *  1.  determine if ADSR must be updated.  if so, calculate new ADSR scalar.
 *  2.  multiply sample by scalar.
 *  3.  multiply sample by ADSR scalar.
 *  4.  pan sample as required
 *  5.  write sample to L and R channel buffers.
 *
 *  ADSRPanEnv parameters are as follows:
 *
 *  ADSR update rate
 *  current ADSR update rate count
 *  current scalar
 *  current ADSR stage
 *  attack rate
 *  decay rate
 *  decay min
 *  sustain len
 *  current sustain
 *  release rate
 *  scalar
 *  pan (0x0000=L, 0x4000=center, 0x7fff=R)
 *
 *  I2 = ptr into ADSR parameter list
 *  I3 = ptr to current stage
 */

ADSRPanEnv:
    I2 = DM(p_ADSRPanEnv);
    AY0 = DM(I2, M1);                       /* get update rate */
    AR = DM(I2, M0);                        /* get current count */
    AR = AR + 1;                            /* count++ */
    AF=AR-AY0, DM(I2,M1)=AR;                /* write count, check vs rate */
    MY0 = DM(I2, M1);                       /* get ADSR scalar */
    if LE jump ApplyADSR;                   /* if count<=rate, apply ADSR */

UpdateADSR:
    M2 = -2;
    modify(I2, M2);
    DM(I2, M1) = 0;
    modify(I2, M1);
    I5 = ^ADSRJT;
    I3 = I2;                                /* keep stage ptr */
    AX1 = DM(I2, M1);                       /* get ADSR stage */
    M5 = AX1;
    modify(I5, M5);
    jump (I5);
    
ADSRJT:
    jump Attack;
    jump Decay;
    jump Sustain;
    jump Release;

Attack:
    AR = DM(I2, M0);                        /* get attack rate */
    AY0 = MY0;                              /* AY0=ADSR scalar */
    AR = AR + AY0;                          /* inc ADSR scalar, dummy read */
    M2 = -2;
    modify(I2, M2);                         /* I2=^ADSR scalar */
    DM(I2, M1) = AR;                        /* update ADSR scalar */
    MY0 = AR;                               /* MY0=ADSR scalar */
    if NOT AV jump ApplyADSR;               /* no overflow?  scale data */
    
    DM(I3, M0) = 1;                         /* update stage to decay */
    jump ApplyADSR;

Decay:
    AR = MY0;                               /* AR=ADSR scalar */
    modify(I2, M1);                         /* I2=^decay decrement */
    AY1 = DM(I2, M1);                       /* get decay rate */
    AR = AR - AY1;                          /* dec ADSR scalar */
    AY0 = DM(I2, M1);                       /* get dec threshold */
    M2 = -5;
    modify(I2, M2);                         /* I2=^ADSR scalar */
    DM(I2, M0) = AR;                        /* update ADSR scalar */
    NONE = AR - AY0;                        
    if GT jump Decay2;                      /* is decay thresh < curr scalar */
    
    DM(I2, M1) = AY0;                       /* if thresh<scalr, thresh=scalr */
    MY0 = AY0;                              /* ADSR scalar = thresh */
    DM(I3, M0) = 2;                         /* update stage to sustain */
    jump ApplyADSR;

Decay2:
    modify(I2, M1);
    MY0 = AR;
    jump ApplyADSR;
    
Sustain:
    M2 = 3;
    modify(I2, M2);
    AY1 = DM(I2, M1);                       /* get sustain len */
    AR = DM(I2, M0);                        /* get current sustain */
    AR = AR + 1;
    DM(I2, M0) = AR;                        /* sustain++ */
    M2 = -5;
    modify(I2, M2);
    NONE = AR - AY1;                        /* count<thresh? */
    if LE jump ApplyADSR;                   /* if true, just scale */
    
    DM(I3, M0) = 3;                         /* else, update stage to release */
    jump ApplyADSR;

Release:
    M2 = 5;
    modify(I2, M2);
    AR = MY0;                               /* AR = ADSR scalar */
    AY1 = DM(I2, M1);                       /* AY1 = release rate */
    AR = AR - AY1;                          /* scalar = scalar - release rate */
    M2 = -8;
    if LE AR = pass 0;
    modify(I2, M2);
    DM(I2, M1) = AR;
    MY0 = AR;

ApplyADSR:
    I3 = DM(p_GenData);
    AR = DM(I3, M1);                        /* get current sample from gen */
    DM(p_GenData) = I3;
    M2 = 7;
    MR=AR*MY0(SS), AX1=DM(I2,M2);           /* MR=ADSRscalar*samp, I2=^scalar */
    MY0 = DM(I2, M1);
    
ApplyConstScalar:
    MR=MR1*MY0(SS), AY0=DM(I2,M1);          /* scalar*sample, get pan */

ApplyPan:
    AX0 = 0x7fff;
    AR=AX0-AY0, MY0=MR1;                    /* L=0x7fff-pan, MY0=scaled val */
    MR=AR*MY0(SS), MX0=AY0;                 /* MR=L*scaled val, MX0=pan */
    DM(I4,M7)=MR1, MR=MX0*MY0(SS);          /* MR=R*scaled val, write to LCH */
    DM(I7, M7) = MR1;                       /* write to RCH array */

ExitADSRPanEnv:    
    DM(p_ADSRPanEnv) = I2;

    rts;
    
.ENDMOD;
