/******************************************************************************
*
*  FM2Op0Gen.dsp
*
*  (c) 2001, 2002, 2003 ethan bordeaux
*  syndevkit@dspmusic.org
*
*
******************************************************************************/

.MODULE                                     modFM2OpGen;

/*
 *  include files
 */


#include    ".\comsys\hdr\DSPdefs.h"
#include    ".\gen_fx\hdr\GenFX.h"

/*
 *  entry points
 */

.VAR temp;

.ENTRY      FM2Op0Gen;

FM2Op0Gen:
    I2 = DM(p_FM2Op0Gen);
    
    MR1 = DM(I2, M1);                       /* get modfreq/modphase */
    dis AR_SAT, ena M_MODE;
    MY0 = 0x7ae0;
    !MR=MR+MR1*MY0 (UU), AY0=DM(I2, M0);     /* get modoldphase */
    MR=MR+MR1*MY0 (SU), AY0=DM(I2, M0);     /* get modoldphase */
    AR = MR1 + AY0;                         /* inc modphase */
    MY0 = 128;
    DM(I2,M1)=AR, MR=AR*MY0 (UU);           /* write ph, get modLUT offset */

    AY0 = DM(I2, M1);                       /* base of modLUT */
    AR = MR1 + AY0;
    I3 = AR;

    AY0 = DM(I3, M1);                       /* modLUT(n) */
    AX0 = DM(I3, M1);                       /* modLUT(n+1) */
    MY0=MR0, AR=AX0-AY0;                    /* get difference */

    MR = AR * MY0 (SU);                     /* scale to interpolate */
    
    ena AR_SAT, dis M_MODE;
    
    !AR = MR1 + AY0;                         /* AR = interpolated output */

    !AX0 = DM(I2, M1);                       /* get abs check param */
    AR=MR1+AY0, AX0=DM(I2, M1);             /* AR=modout, get abs param */
    
    NONE = pass AX0;
    if NE AR = abs AR;                      /* if not zero, abs modulator out */

    DM(temp) = AR;        

    /* start ADSR processing on modulator */    
    AY0 = DM(I2, M1);                       /* get update rate */
    AR = DM(I2, M0);                        /* get current count */
    AR = AR + 1;                            /* count++ */
    AF=AR-AY0, DM(I2,M1)=AR;                /* write count, check vs rate */
    MY0 = DM(I2, M1);                       /* get ADSR scalar */
    if LE jump ApplyADSR;                   /* if count<=rate, apply ADSR */

UpdateADSR:
    M2 = -2;
    modify(I2, M2);
    DM(I2, M1) = 0;
    modify(I2, M1);
    I5 = ^ADSRJT;
    I3 = I2;                                /* keep stage ptr */
    AX1 = DM(I2, M1);                       /* get ADSR stage */
    M5 = AX1;
    modify(I5, M5);
    jump (I5);
    
ADSRJT:
    jump Attack;
    jump Decay;
    jump Sustain;
    jump Release;

Attack:
    AR = DM(I2, M0);                        /* get attack rate */
    AY0 = MY0;                              /* AY0=ADSR scalar */
    AR = AR + AY0;                          /* inc ADSR scalar, dummy read */
    M2 = -2;
    modify(I2, M2);                         /* I2=^ADSR scalar */
    DM(I2, M1) = AR;                        /* update ADSR scalar */
    MY0 = AR;                               /* MY0=ADSR scalar */
    if NOT AV jump ApplyADSR;               /* no overflow?  scale data */
    
    DM(I3, M0) = 1;                         /* update stage to decay */
    jump ApplyADSR;

Decay:
    AR = MY0;                               /* AR=ADSR scalar */
    modify(I2, M1);                         /* I2=^decay decrement */
    AY1 = DM(I2, M1);                       /* get decay rate */
    AR = AR - AY1;                          /* dec ADSR scalar */
    AY0 = DM(I2, M1);                       /* get dec threshold */
    M2 = -5;
    modify(I2, M2);                         /* I2=^ADSR scalar */
    DM(I2, M0) = AR;                        /* update ADSR scalar */
    NONE = AR - AY0;                        
    if GT jump Decay2;                      /* is decay thresh < curr scalar */
    
    DM(I2, M1) = AY0;                       /* if thresh<scalr, thresh=scalr */
    MY0 = AY0;                              /* ADSR scalar = thresh */
    DM(I3, M0) = 2;                         /* update stage to sustain */
    jump ApplyADSR;

Decay2:
    modify(I2, M1);
    MY0 = AR;
    jump ApplyADSR;
    
Sustain:
    M2 = 3;
    modify(I2, M2);
    AY1 = DM(I2, M1);                       /* get sustain len */
    AR = DM(I2, M0);                        /* get current sustain */
    AR = AR + 1;
    DM(I2, M0) = AR;                        /* sustain++ */
    M2 = -5;
    modify(I2, M2);
    NONE = AR - AY1;                        /* count<thresh? */
    if LE jump ApplyADSR;                   /* if true, just scale */
    
    DM(I3, M0) = 3;                         /* else, update stage to release */
    jump ApplyADSR;

Release:
    M2 = 5;
    modify(I2, M2);
    AR = MY0;                               /* AR = ADSR scalar */
    AY1 = DM(I2, M1);                       /* AY1 = release rate */
    AR = AR - AY1;                          /* scalar = scalar - release rate */
    M2 = -8;
    if LE AR = pass 0;
    modify(I2, M2);
    DM(I2, M1) = AR;
    MY0 = AR;

ApplyADSR:
    AR = DM(temp);
    M2 = 7;
    MR=AR*MY0(SS), AX1=DM(I2,M2);           /* MR=ADSRscalar*samp, I2=^scalar */
    MY0 = DM(I2, M1);
    
ApplyConstScalar:
    MR = MR1 * MY0 (SS);                     /* apply scalar */

    /* MR1 holds output */
    AY0 = DM(I2, M1);                       /* get base freq of carrier */
    AR = MR1 + AY0;

    /* generate carrier with modulator output as freq */
    MR1 = AR;                               /* get freq/phase */
    dis AR_SAT, ena M_MODE;
    MY0 = 0x7ae0;
    !MR=MR+MR1*MY0 (UU), AY0=DM(I2, M0);     /* get oldphase */
    MR=MR+MR1*MY0 (SU), AY0=DM(I2, M0);     /* get oldphase */
    AR = MR1 + AY0;                         /* inc phase */
    MY0 = 128;
    DM(I2,M1)=AR, MR=AR*MY0 (UU);           /* writeback ph, get LUT offset */

    AY0 = DM(I2, M1);                       /* base of LUT */
    AR = MR1 + AY0;
    I3 = AR;

    AY0 = DM(I3, M1);                       /* LUT(n) */
    AX0 = DM(I3, M1);                       /* LUT(n+1) */
    MY0=MR0, AR=AX0-AY0;                    /* get difference */

    MR = AR * MY0 (SU);                    /* scale to interpolate */
    
    ena AR_SAT, dis M_MODE;
    
    AR = MR1 + AY0;                         /* AR = interpolated output */
    
    DM(I7, M6) = AR;
    
    DM(p_FM2Op0Gen) = I2;
    
    rts;

.ENDMOD;