/******************************************************************************
*
*  WaveShape.dsp
*
*  (c) 2001, 2002, 2003 ethan bordeaux
*  syndevkit@dspmusic.org
*
*
******************************************************************************/

.MODULE                                     modWaveShapeFX;


#include    ".\comsys\hdr\DSPdefs.h"
#include    ".\gen_fx\hdr\GenFX.h"

.VAR        temp;

.ENTRY      WaveShapeFX;

/*
 *  AX1:      input sample
 *  MX1/MX0:  waveshape scalars
 *  MY0:      input sample
 */

WaveShapeFX: 
    I2 = DM(p_WaveShapeFX);
    AR = DM(I2, M1);
    AR=pass AR, AX1=DM(I7,M6);              /* pich waveshape type, get input */
    if NE jump ContinuousWS;

DiscontinuousWS:
    /* pick pos or neg saturation check based on sign of input */
    I5 = ^CheckPosSat;
    AR = pass AX1;
    if GE jump GetDisWSParams;
    I5 = ^CheckNegSat;

GetDisWSParams:
    AY0 = DM(I2, M1);                       /* cutoff */
    AR=abs AX1, MX1=DM(I2,M1);              /* abs input, get mswscale0 */
    AF=AR-AY0, MX0=DM(I2,M1);               /* check cutoff, get lswscale0 */
    if GE jump ShapeSlope1;
    
ShapeSlope0:
    MY0 = DM(I7, M6);
    call m32by16;
    call (I5);                              /* pos or neg sat check */
    DM(I7, M6) = MR0;
    modify(I2, M1);
    modify(I2, M1);
    DM(p_WaveShapeFX) = I2;
    rts;

ShapeSlope1:
    MX1 = DM(I2, M1);                       /* mswscale1 */
    MX0 = DM(I2, M1);                       /* lswscale1 */
    MY0 = DM(I7, M6);
    call m32by16;
    call (I5);                              /* pos or neg sat check */
    DM(I7, M6) = MR0;
    DM(p_WaveShapeFX) = I2;
    rts;

ContinuousWS:
    /* pick pos or neg saturation check based on sign of input */
    I5 = ^CheckPosSat;
    AR = pass AX1;
    if GT jump GetConWSParams;
    I5 = ^CheckNegSat;

GetConWSParams:
    AY0 = DM(I2, M1);                       /* cutoff */
    AR=abs AX1, MX1=DM(I2,M1);              /* abs input, get mswscale0 */
    AF=AR-AY0, MX0=DM(I2,M1);               /* check cutoff, get lswscale0 */
    if GE jump CWSShapeSlope1;
    jump ShapeSlope0;                       /* use discontinuous WS */

CWSShapeSlope1:
    AR = pass AX1;
    if GT jump NoInvertCutoff;
    AR = -AY0;
    AY0 = AR;

NoInvertCutoff:
    MY0 = AY0;
    call m32by16;
    call (I5);                              /* pos or neg sat check */
    DM(temp) = MR0;                         /* store partial outpu */
    AR=AX1-AY0, MX1=DM(I2,M1);              /* AR:Slope1 scalr, get mswscale1 */
    MX0 = DM(I2, M1);
    MY0 = AR;
    call m32by16;
    call (I5);                              /* pos or neg sat check */
    
    AY1 = DM(temp);
    AR = MR0 + AY1;
    DM(I7, M6) = AR;
    DM(p_WaveShapeFX) = I2;
    rts;

/*
 *  saturation checking local functions on 32x16 multiply
 */
    
CheckPosSat:
    AR = tstbit 15 of MR0;
    if NE jump PosSat;
    AR = pass MR1;
    if NE jump PosSat;
    rts;

PosSat:
    MR0 = 0x7fff;
    rts;

CheckNegSat:
    AR = tstbit 15 of MR0;
    if EQ jump NegSat1;
    AF = MR1 + 1;
    if NE jump NegSat2;
    rts;

/* all clicks due to improper entering of NegSat1 */
NegSat1:
    rts;
    
NegSat2:
    MR0 = 0x8000;
    rts;

/*
 *  32x16 multiply local function
 */

m32by16:
    ena M_MODE;
    MR = MX0 * MY0 (US);                    /* compute LSW */
    MR0 = MR1;                              /* rightshift by 16 */
    MR1 = MR2;
    MR=MR+MX1*MY0(SS);                      /* mult lower by upper */
    dis M_MODE;

    rts;

.ENDMOD;
