/******************************************************************************
*
*  GenFX.h
*
******************************************************************************/

#include    ".\gen_fx\hdr\extvars.h"


/*
 *  important!  ALLGENFX must change as new GenFX functions are added/removed
 *  from SyndevKit.  it must reflect the total number of functions.  this should
 *  be equal to the number of sets of ptr inits inside SetPtrOpcodes.
 */
 
#define     ALLGENFX                    35

/*
 *  this macro must be used when setting pointer for gen/fx initialization
 *  in GenFXIni - post processing routines check for this opcode and init
 *  buffers based upon find it.
 */

#define SETPTR(ptr) \
    I2 = ^ptr
    
/*----------------------------------------------------------------------------*/
/*---------------------------------Bitmask------------------------------------*/
/*----------------------------------------------------------------------------*/

#define     BITMASKFX_VARS              3
#define     BITMASKFX_CALLS             8

#define     BITMASKFX_TYPE              0
#define     BITMASKFX_MASK              1
#define     BITMASKFX_MIX               2
                                        
#define     BITMASKFX_AND               0
#define     BITMASKFX_OR                1
#define     BITMASKFX_XOR               2

/* use with BM_AND and mix set to max value (0x7fff) to deciamte */
#define     DECIMATE0                   0xffff
#define     DECIMATE1                   0xfffe
#define     DECIMATE2                   0xfffc
#define     DECIMATE3                   0xfff8
#define     DECIMATE4                   0xfff0
#define     DECIMATE5                   0xffe0
#define     DECIMATE6                   0xffc0
#define     DECIMATE7                   0xff80
#define     DECIMATE8                   0xff00
#define     DECIMATE9                   0xfe00
#define     DECIMATE10                  0xfc00
#define     DECIMATE11                  0xf800
#define     DECIMATE12                  0xf000
#define     DECIMATE13                  0xe000
#define     DECIMATE14                  0xc000
#define     DECIMATE15                  0x8000

#define INIT_BITMASKFX(type, mask, mix) \
    DM(I2, M1) = type; \
    DM(I2, M1) = mask; \
    DM(I2, M1) = mix
    
/*----------------------------------------------------------------------------*/
/*---------------------------------ClampFX------------------------------------*/
/*----------------------------------------------------------------------------*/

#define     CLAMPFX_VARS                2
#define     CLAMPFX_CALLS               10

#define     CLAMPFX_CLAMPMAX            0
#define     CLAMPFX_MIX                 1

#define INIT_CLAMPFX(clampmax, mix) \
    DM(I2, M1) = clampmax; \
    DM(I2, M1) = mix

/*----------------------------------------------------------------------------*/
/*--------------------------------CopiedGen-----------------------------------*/
/*----------------------------------------------------------------------------*/

#define     COPIEDGEN_VARS              1
#define     COPIEDGEN_CALLS             32

#define     COPIEDGEN_READGEN           0

#define     INIT_COPIEDGEN(readgen) \
    DM(I2, M1) = readgen

/*----------------------------------------------------------------------------*/
/*--------------------------------ZeroSamps-----------------------------------*/
/*----------------------------------------------------------------------------*/

#define     ZEROSAMPSFX_VARS            2
#define     ZEROSAMPSFX_CALLS           10

#define     ZEROSAMPSFX_PROB            0
#define     ZEROSAMPSFX_MIX             1

#define INIT_ZEROSAMPSFX(prob, mix) \
    DM(I2, M1) = prob; \
    DM(I2, M1) = mix

/*----------------------------------------------------------------------------*/
/*---------------------------------Rectify------------------------------------*/
/*----------------------------------------------------------------------------*/

#define     RECTIFYFX_VARS              2
#define     RECTIFYFX_CALLS             5
                                        
#define     RECTIFYFX_REC               0
#define     RECTIFYFX_MIX               1

#define     POS_RECTIFY                 0
#define     NEG_RECTIFY                 1
                                        
#define INIT_RECTIFYFX(rec, mix) \
    DM(I2, M1) = rec; \
    DM(I2, M1) = mix

/*----------------------------------------------------------------------------*/
/*------------------------------TunedRotsynth---------------------------------*/
/*----------------------------------------------------------------------------*/

#define     TUNEDROTSYNTHGEN_VARS       7
#define     TUNEDROTSYNTHGEN_CALLS      8

#define     TUNEDROTSYNTHGEN_COUNT      0
#define     TUNEDROTSYNTHGEN_CURRCOUNT  1
#define     TUNEDROTSYNTHGEN_ROTAMOUNT  2
#define     TUNEDROTSYNTHGEN_VAL        3
#define     TUNEDROTSYNTHGEN_SAMPS      4
#define     TUNEDROTSYNTHGEN_ROTLEFT    5
#define     TUNEDROTSYNTHGEN_FREQ       6

#define INIT_TUNEDROTSYNTHGEN(rotamount, val, freq) \
    DM(I2, M1) = 0x0000; \
    DM(I2, M1) = 0x7fff; \
    DM(I2, M1) = rotamount; \
    DM(I2, M1) = val; \
    DM(I2, M1) = 0x0000; \
    DM(I2, M1) = 1; \
    DM(I2, M1) = freq

#define NEW_RANDVAL_TUNEDROTSYNTHGEN(num) \
    AR = DM(I6, M7); \
    DM(a_TunedRotSynthGen + ((num*TUNEDROTSYNTHGEN_VARS)+TUNEDROTSYNTHGEN_VAL)) = AR

#define CTRLDATA_TO_TUNEDROTSYNTHGEN_FREQ(ctrlnum0, gennum) \
    AR = DM(a_CTRLData+ctrlnum0); \
    DM(a_TunedRotSynthGen + ((gennum*TUNEDROTSYNTHGEN_VARS)+TUNEDROTSYNTHGEN_FREQ)) = AR; \

/*----------------------------------------------------------------------------*/
/*---------------------------------Rotsynth-----------------------------------*/
/*----------------------------------------------------------------------------*/

#define     ROTSYNTHGEN_VARS            4
#define     ROTSYNTHGEN_CALLS           5
                                        
#define     ROTSYNTHGEN_COUNT           0
#define     ROTSYNTHGEN_INTCOUNT        1
#define     ROTSYNTHGEN_ROTDIST         2        
#define     ROTSYNTHGEN_VAL             3

#define INIT_ROTSYNTHGEN(rotcount, rotdist, rotval) \
    DM(I2, M1) = rotcount; \
    DM(I2, M1) = 0x0000; \
    DM(I2, M1) = rotdist; \
    DM(I2, M1) = rotval
    
#define NEW_RANDVAL_ROTSYNTHGEN(num) \
    AR = DM(I6, M7); \
    DM(a_RotSynthGen + ((num*ROTSYNTHGEN_VARS)+ROTSYNTHGEN_VAL)) = AR

/*----------------------------------------------------------------------------*/
/*----------------------------------ProbSyn-----------------------------------*/
/*----------------------------------------------------------------------------*/

#define     PROBSYNTHGEN_VARS           11
#define     PROBSYNTHGEN_CALLS          5

#define     PROBSYNTHGEN_RATE           0
#define     PROBSYNTHGEN_INTCOUNT       1
#define     PROBSYNTHGEN_VAL            2
#define     PROBSYNTHGEN_PROB15         3
#define     PROBSYNTHGEN_PROB14         4
#define     PROBSYNTHGEN_PROB13         5
#define     PROBSYNTHGEN_PROB12         6
#define     PROBSYNTHGEN_PROB11         7
#define     PROBSYNTHGEN_PROB10         8
#define     PROBSYNTHGEN_PROB9          9
#define     PROBSYNTHGEN_PROB8          10

#define INIT_PROBSYNTHGEN(rate, prob15, prob14, prob13, prob12, prob11, prob10, prob9, prob8) \
    DM(I2, M1) = rate; \
    DM(I2, M1) = 0x0000; \
    DM(I2, M1) = 0x0000; \
    DM(I2, M1) = prob15; \
    DM(I2, M1) = prob14; \
    DM(I2, M1) = prob13; \
    DM(I2, M1) = prob12; \
    DM(I2, M1) = prob11; \
    DM(I2, M1) = prob10; \
    DM(I2, M1) = prob9; \
    DM(I2, M1) = prob8

/*----------------------------------------------------------------------------*/
/*---------------------------------ProbKSGen----------------------------------*/
/*----------------------------------------------------------------------------*/

#define     PROBKSGEN_VARS              6
#define     PROBKSGEN_CALLS             8
                                        
#define     PROBKSGEN_FILTPROB          0
#define     PROBKSGEN_ADDSUBPROB1       1
#define     PROBKSGEN_ADDSUBPROB2       2
#define     PROBKSGEN_AVEFAC            3
#define     PROBKSGEN_FREQ              4
#define     PROBKSGEN_BUFFPTR           5

#define INIT_PROBKSGEN(filtprob, addsubprob1, addsubprob2, avefac, freq, buffptr) \
    DM(I2, M1) = filtprob; \
    DM(I2, M1) = addsubprob1; \
    DM(I2, M1) = addsubprob2; \
    DM(I2, M1) = avefac; \
    DM(I2, M1) = freq; \
    DM(I2, M1) = buffptr

#define INIT_PROBKSGEN_MF(filtprob, addsubprob1, addsubprob2, avefac, note, buffptr) \
    DM(I2, M1) = filtprob; \
    DM(I2, M1) = addsubprob1; \
    DM(I2, M1) = addsubprob2; \
    DM(I2, M1) = avefac; \
    I3 = ^a_MIDIFreqKS; \
    M2 = note; \
    modify(I3, M2); \
    AR = DM(I3, M0); \
    DM(I2, M1) = AR; \
    DM(I2, M1) = buffptr

#define CTRLDATA_TO_PROBKSGEN_FREQ_MF(ctrlnum, gennum) \
    M2 = DM(a_CTRLData+ctrlnum); \
    I3 = ^a_MIDIFreqKS; \
    modify(I3, M2); \
    AR = DM(I3, M0); \
    DM(a_ProbKSGen + ((gennum*PROBKSGEN_VARS)+PROBKSGEN_FREQ)) = AR

/*----------------------------------------------------------------------------*/
/*----------------------------------MemEnv1-----------------------------------*/
/*----------------------------------------------------------------------------*/

#define     MEMENV1_VARS                    10
#define     MEMENV1_CALLS                   8
                                        
#define     MEMENV1_CURRSCALE               0
#define     MEMENV1_STAGE                   1
#define     MEMENV1_ATTACKRATE              2
#define     MEMENV1_DECAYRATE               3
#define     MEMENV1_DECAYMIN                4
#define     MEMENV1_SUSTAINLEN              5
#define     MEMENV1_SUSTAINCNT              6
#define     MEMENV1_RELRATE                 7
#define     MEMENV1_SCALAR                  8
#define     MEMENV1_ADDR                    9

#define INIT_MEMENV1(attrate, decrate, susheight, suscount, relrate, scalar, addr) \
    DM(I2, M1) = 0x0000; \
    DM(I2, M1) = 0x0000; \
    DM(I2, M1) = attrate; \
    DM(I2, M1) = decrate; \
    DM(I2, M1) = susheight; \
    DM(I2, M1) = suscount; \
    DM(I2, M1) = 0x0000; \
    DM(I2, M1) = relrate; \
    DM(I2, M1) = scalar; \
    DM(I2, M1) = addr
    
#define RESET_MEMENV1(num) \
    DM(a_MemEnv1 + ((num*MEMENV1_VARS)+MEMENV1_CURRSCALE)) = M0; \
    DM(a_MemEnv1 + ((num*MEMENV1_VARS)+MEMENV1_STAGE)) = M0; \
    DM(a_MemEnv1 + ((num*MEMENV1_VARS)+MEMENV1_SUSTAINCNT)) = M0

/*----------------------------------------------------------------------------*/
/*----------------------------------MemEnv2-----------------------------------*/
/*----------------------------------------------------------------------------*/

#define     MEMENV2_VARS                    19
#define     MEMENV2_CALLS                   4
                                        
#define     MEMENV2_CURRSCALE               0
#define     MEMENV2_ADDR                    1
#define     MEMENV2_MAXOUT                  2
#define     MEMENV2_MINOUT                  3
#define     MEMENV2_LIMIT0                  4
#define     MEMENV2_LIMIT1                  5
#define     MEMENV2_LIMIT2                  6
#define     MEMENV2_DRATE0                  7
#define     MEMENV2_DRATECNTR0              8
#define     MEMENV2_SCALAR0                 9
#define     MEMENV2_DRATE1                  10
#define     MEMENV2_DRATECNTR1              11
#define     MEMENV2_SCALAR1                 12
#define     MEMENV2_DRATE2                  13
#define     MEMENV2_DRATECNTR2              14
#define     MEMENV2_SCALAR2                 15
#define     MEMENV2_DRATE3                  16
#define     MEMENV2_DRATECNTR3              17
#define     MEMENV2_SCALAR3                 18

#define INIT_MEMENV2(addr, maxout, minout, limit0, limit1, limit2, drate0, sc0, drate1, sc1, drate2, sc2, drate3, sc3) \
    DM(I2, M1) = 0x7fff; \
    DM(I2, M1) = addr; \
    DM(I2, M1) = maxout; \
    DM(I2, M1) = minout; \
    DM(I2, M1) = limit0; \
    DM(I2, M1) = limit1; \
    DM(I2, M1) = limit2; \
    DM(I2, M1) = drate0; \
    DM(I2, M1) = 0; \
    DM(I2, M1) = sc0; \
    DM(I2, M1) = drate1; \
    DM(I2, M1) = 0; \
    DM(I2, M1) = sc1; \
    DM(I2, M1) = drate2; \
    DM(I2, M1) = 0; \
    DM(I2, M1) = sc2; \
    DM(I2, M1) = drate3; \
    DM(I2, M1) = 0; \
    DM(I2, M1) = sc3

#define RESET_MEMENV2(num) \
    AR = 0x7fff; \
    DM(a_MemEnv2 + ((num*MEMENV2_VARS)+MEMENV2_CURRSCALE)) = AR; \
    DM(a_MemEnv2 + ((num*MEMENV2_VARS)+MEMENV2_DRATECNTR0)) = M0; \
    DM(a_MemEnv2 + ((num*MEMENV2_VARS)+MEMENV2_DRATECNTR1)) = M0; \
    DM(a_MemEnv2 + ((num*MEMENV2_VARS)+MEMENV2_DRATECNTR2)) = M0; \
    DM(a_MemEnv2 + ((num*MEMENV2_VARS)+MEMENV2_DRATECNTR3)) = M0

/*----------------------------------------------------------------------------*/
/*----------------------------------MemEnv3-----------------------------------*/
/*----------------------------------------------------------------------------*/

#define     MEMENV3_VARS                    8
#define     MEMENV3_CALLS                   24
                                        
#define     MEMENV3_STAGE                   0
#define     MEMENV3_ADDR                    1
#define     MEMENV3_INTCNTR                 2
#define     MEMENV3_SA                      3
#define     MEMENV3_EA                      4
#define     MEMENV3_ATIME                   5
#define     MEMENV3_ED                      6
#define     MEMENV3_DTIME                   7

#define INIT_MEMENV3(addr, SA, EA, arate, ED, drate) \
    DM(I2, M1) = 0x0000; \
    DM(I2, M1) = addr; \
    DM(I2, M1) = 0x0000; \
    DM(I2, M1) = SA; \
    DM(I2, M1) = EA; \
    DM(I2, M1) = arate; \
    DM(I2, M1) = ED; \
    DM(I2, M1) = drate
    
#define RESET_MEMENV3(num) \
    DM(a_MemEnv3 + ((num*MEMENV3_VARS)+MEMENV3_STAGE)) = M0; \
    DM(a_MemEnv3 + ((num*MEMENV3_VARS)+MEMENV3_INTCNTR)) = M0

/*----------------------------------------------------------------------------*/
/*----------------------------------VolEnv------------------------------------*/
/*----------------------------------------------------------------------------*/

/* VOLENV_CALLS = MAXTRACKS */
#define     VOLENV_VARS                 5

#define     VOLENV_TRACK                0
#define     VOLENV_INTSCALAR            1
#define     VOLENV_START                2
#define     VOLENV_END                  3
#define     VOLENV_DURATION             4

#define SET_VOLENV(offset, start, end, time) \
    ena M_MODE; \
    MX1 = offset; \
    MY1 = VOLENV_VARS; \
    MR = MX1 * MY1 (UU); \
    dis M_MODE; \
    M2 = MR0; \
    I2 = ^a_VolEnv; \
    modify(I2, M2); \
    modify(I2, M1); \
    DM(I2, M1) = 0; \
    DM(I2, M1) = start; \
    DM(I2, M1) = end; \
    DM(I2, M1) = time

#define MUTETRACK(n) \
    ena M_MODE; \
    MX1 = n; \
    MY1 = VOLENV_VARS; \
    MR = MX1 * MY1 (UU); \
    dis M_MODE; \
    M2 = MR0; \
    I2 = ^a_VolEnv; \
    modify(I2, M2); \
    modify(I2, M1); \
    DM(I2, M1) = 0; \
    DM(I2, M1) = 0x0000; \
    DM(I2, M1) = 0x0000; \
    DM(I2, M1) = 1

#define FULLTRACK(n) \
    ena M_MODE; \
    MX1 = n; \
    MY1 = VOLENV_VARS; \
    MR = MX1 * MY1 (UU); \
    dis M_MODE; \
    M2 = MR0; \
    I2 = ^a_VolEnv; \
    modify(I2, M2); \
    modify(I2, M1); \
    DM(I2, M1) = 0; \
    DM(I2, M1) = 0x7fff; \
    DM(I2, M1) = 0x7fff; \
    DM(I2, M1) = 1

#define SET_TRACKVOL(n, vol) \
    ena M_MODE; \
    MX1 = n; \
    MY1 = VOLENV_VARS; \
    MR = MX1 * MY1 (UU); \
    dis M_MODE; \
    M2 = MR0; \
    I2 = ^a_VolEnv; \
    modify(I2, M2); \
    modify(I2, M1); \
    DM(I2, M1) = 0; \
    DM(I2, M1) = vol; \
    DM(I2, M1) = vol; \
    DM(I2, M1) = 1

/*----------------------------------------------------------------------------*/
/*-----------------------------------Exp1-------------------------------------*/
/*----------------------------------------------------------------------------*/

#define     EXP1GEN_VARS                7
#define     EXP1GEN_CALLS               8
                                        
#define     EXP1GEN_RATE                0
#define     EXP1GEN_CURRCOUNT           1
#define     EXP1GEN_OLDOUT              2
#define     EXP1GEN_A                   3
#define     EXP1GEN_B                   4
#define     EXP1GEN_C                   5
#define     EXP1GEN_D                   6

#define INIT_EXP1GEN(rate, a, b, c, d) \
    DM(I2, M1) = rate; \
    DM(I2, M1) = 0x0000; \
    DM(I2, M1) = 0x0000; \
    DM(I2, M1) = a; \
    DM(I2, M1) = b; \
    DM(I2, M1) = c; \
    DM(I2, M1) = d

#define RESET_EXP1GEN(num) \
    DM(a_Exp1Gen + ((num*EXP1GEN_VARS)+EXP1GEN_CURRCOUNT)) = M0; \
    DM(a_Exp1Gen + ((num*EXP1GEN_VARS)+EXP1GEN_OLDOUT)) = M0

/*----------------------------------------------------------------------------*/
/*----------------------------------FM2Op0------------------------------------*/
/*----------------------------------------------------------------------------*/

#define     FM2OP0GEN_VARS                 18
#define     FM2OP0GEN_CALLS                8

#define     FM2OP0GEN_MODFREQ              0
#define     FM2OP0GEN_MODOLDPH             1
#define     FM2OP0GEN_MODLUT               2
#define     FM2OP0GEN_ABSFREQ              3
#define     FM2OP0GEN_MODENV_UR            4
#define     FM2OP0GEN_MODENV_RC            5
#define     FM2OP0GEN_MODENV_INTSCALAR     6
#define     FM2OP0GEN_MODENV_STAGE         7
#define     FM2OP0GEN_MODENV_ATTACKRATE    8
#define     FM2OP0GEN_MODENV_DECAYRATE     9
#define     FM2OP0GEN_MODENV_DECAYMIN      10
#define     FM2OP0GEN_MODENV_SUSTAINLEN    11
#define     FM2OP0GEN_MODENV_SUSTAINCNT    12
#define     FM2OP0GEN_MODENV_RELRATE       13
#define     FM2OP0GEN_MODENV_SCALAR        14
#define     FM2OP0GEN_BASECARR             15
#define     FM2OP0GEN_CARROLDPH            16
#define     FM2OP0GEN_CARRLUT              17

#define INIT_FM2OP0GEN(modfreq, modLUT, absfreq, modenv_ur, modenv_attackrate, modenv_decayrate, modenv_decaymin, modenv_sustainlen, modenv_relrate, modenv_scalar, basecarr, carrLUT) \
    DM(I2, M1) = modfreq; \
    DM(I2, M1) = 0; \
    DM(I2, M1) = modLUT; \
    DM(I2, M1) = absfreq; \
    DM(I2, M1) = modenv_ur; \
    DM(I2, M1) = 0; \
    DM(I2, M1) = 0; \
    DM(I2, M1) = 0; \
    DM(I2, M1) = modenv_attackrate; \
    DM(I2, M1) = modenv_decayrate; \
    DM(I2, M1) = modenv_decaymin; \
    DM(I2, M1) = modenv_sustainlen; \
    DM(I2, M1) = 0; \
    DM(I2, M1) = modenv_relrate; \
    DM(I2, M1) = modenv_scalar; \
    DM(I2, M1) = basecarr; \
    DM(I2, M1) = 0; \
    DM(I2, M1) = carrLUT

#define RESET_FM2OP0GEN(num) \
    DM(a_FM2Op0Gen + ((num*FM2OP0GEN_VARS)+FM2OP0GEN_MODOLDPH)) = M0; \
    DM(a_FM2Op0Gen + ((num*FM2OP0GEN_VARS)+FM2OP0GEN_MODENV_RC)) = M0; \
    DM(a_FM2Op0Gen + ((num*FM2OP0GEN_VARS)+FM2OP0GEN_MODENV_INTSCALAR)) = M0; \
    DM(a_FM2Op0Gen + ((num*FM2OP0GEN_VARS)+FM2OP0GEN_MODENV_STAGE)) = M0; \
    DM(a_FM2Op0Gen + ((num*FM2OP0GEN_VARS)+FM2OP0GEN_MODENV_SUSTAINCNT)) = M0; \
    DM(a_FM2Op0Gen + ((num*FM2OP0GEN_VARS)+FM2OP0GEN_CARROLDPH)) = M0

#define CTRLDATA_TO_FM2OP0GEN_CARRFREQ(ctrlnum, gennum) \
    AR = DM(a_CTRLData+ctrlnum); \
    DM(a_FM2Op0Gen + ((gennum*FM2OP0GEN_VARS)+FM2OP0GEN_BASECARR)) = AR
    
#define CTRLDATA_TO_FM2OP0GEN_MODFREQ(ctrlnum, gennum) \
    AR = DM(a_CTRLData+ctrlnum); \
    DM(a_FM2Op0Gen + ((gennum*FM2OP0GEN_VARS)+FM2OP0GEN_MODFREQ)) = AR

#define CTRLDATA_TO_FM2OP0GEN_CARRFREQ_MF(ctrlnum, gennum) \
    M2 = DM(a_CTRLData+ctrlnum); \
    I3 = ^a_MIDIFreq; \
    modify(I3, M2); \
    AR = DM(I3, M0); \
    DM(a_FM2Op0Gen + ((gennum*FM2OP0GEN_VARS)+FM2OP0GEN_BASECARR)) = AR
    
#define CTRLDATA_TO_FM2OP0GEN_MODFREQ_MF(ctrlnum, gennum) \
    M2 = DM(a_CTRLData+ctrlnum); \
    I3 = ^a_MIDIFreq; \
    modify(I3, M2); \
    AR = DM(I3, M0); \
    DM(a_FM2Op0Gen + ((gennum*FM2OP0GEN_VARS)+FM2OP0GEN_MODFREQ)) = AR
    
/*----------------------------------------------------------------------------*/
/*---------------------------------GenSHFX------------------------------------*/
/*----------------------------------------------------------------------------*/

#define     GENSHFX_VARS                4
#define     GENSHFX_CALLS               16

#define     GENSHFX_HOLDPER             0
#define     GENSHFX_HOLDCNT             1
#define     GENSHFX_HOLDVAL             2
#define     GENSHFX_MIX                 3

#define INIT_GENSHFX(holdper, mix) \
    DM(I2, M1) = holdper; \
    DM(I2, M1) = 0; \
    DM(I2, M1) = 0; \
    DM(I2, M1) = mix

/*----------------------------------------------------------------------------*/
/*--------------------------------ExpImpulse----------------------------------*/
/*----------------------------------------------------------------------------*/

#define     EXPIMPULSEGEN_VARS          2
#define     EXPIMPULSEGEN_CALLS         6

#define     EXPIMPULSEGEN_DECRATE       0
#define     EXPIMPULSEGEN_VOL           1

#define INIT_EXPIMPULSEGEN(decrate) \
    DM(I2, M1) = decrate; \
    DM(I2, M1) = 0

#define RESET_EXPIMPULSEGEN_FS(num) \
    AR = 0x7fff; \
    DM(a_ExpImpulseGen + ((num*EXPIMPULSEGEN_VARS)+EXPIMPULSEGEN_VOL)) = AR
    
#define RESET_EXPIMPULSEGEN_RANDOM(num) \
    AR = DM(I6, M7); \
    DM(a_ExpImpulseGen + ((num*EXPIMPULSEGEN_VARS)+EXPIMPULSEGEN_VOL)) = AR

/*----------------------------------------------------------------------------*/
/*---------------------------------PerNoise-----------------------------------*/
/*----------------------------------------------------------------------------*/

#define     PERNOISEGEN_VARS            3
#define     PERNOISEGEN_CALLS           10
                                        
#define     PERNOISEGEN_MSWSEED         0
#define     PERNOISEGEN_LSWSEED         1
#define     PERNOISEGEN_LSWA            2

#define INIT_PERNOISEGEN(mswseed, lswseed, lswa) \
    DM(I2, M1) = mswseed; \
    DM(I2, M1) = lswseed; \
    DM(I2, M1) = lswa

/*----------------------------------------------------------------------------*/
/*---------------------------------KillTime-----------------------------------*/
/*----------------------------------------------------------------------------*/

#define     KILLTIMEFX_VARS             1
#define     KILLTIMEFX_CALLS            10

#define     KILLTIMEFX_CYCLES           0

#define INIT_KILLTIMEFX(cycles) \
    DM(I2, M1) = cycles

/*----------------------------------------------------------------------------*/
/*---------------------------------ExpDecay-----------------------------------*/
/*----------------------------------------------------------------------------*/

#define     EXPDECAYENV_VARS            6
#define     EXPDECAYENV_CALLS           16

#define     EXPDECAYENV_DECRATE         0
#define     EXPDECAYENV_INTCOUNT        1
#define     EXPDECAYENV_DECCONST        2
#define     EXPDECAYENV_INTSCALAR       3
#define     EXPDECAYENV_SCALAR          4
#define     EXPDECAYENV_PAN             5

#define INIT_EXPDECAYENV(decrate, decscalar, pan) \
    DM(I2, M1) = decrate; \
    DM(I2, M1) = 0x0000; \
    DM(I2, M1) = decscalar; \
    DM(I2, M1) = 0x7fff; \
    DM(I2, M1) = 0x0000; \
    DM(I2, M1) = pan

#define RESET_EXPDECAYENV(num) \
    DM(a_ExpDecayEnv + ((num*EXPDECAYENV_VARS)+EXPDECAYENV_INTCOUNT)) = M0; \
    AR = 0x7fff; \
    DM(a_ExpDecayEnv + ((num*EXPDECAYENV_VARS)+EXPDECAYENV_INTSCALAR)) = AR

/*----------------------------------------------------------------------------*/
/*-------------------------------PrevCurrFilt---------------------------------*/
/*----------------------------------------------------------------------------*/

#define     PREVCURRFILTFX_VARS         3
#define     PREVCURRFILTFX_CALLS        5

#define     PREVCURRFILTFX_PCFTYPE      0
#define     PREVCURRFILTFX_MAXDIFF      1
#define     PREVCURRFILTFX_PREVSAMP     2

#define     PREVCURRFILTFX_0            0
#define     PREVCURRFILTFX_1            1

#define INIT_PREVCURRFILTFX(pcftype, maxdiff) \
    DM(I2, M1) = pcftype; \
    DM(I2, M1) = maxdiff; \
    DM(I2, M1) = 0

/*----------------------------------------------------------------------------*/
/*----------------------------------OscComb-----------------------------------*/
/*----------------------------------------------------------------------------*/

#define     OSCCOMBGEN_VARS             3
#define     OSCCOMBGEN_CALLS            10

#define     OSCCOMBGEN_COMBTYPE         0
#define     OSCCOMBGEN_GEN0             1
#define     OSCCOMBGEN_GEN1             2

#define     OSCCOMBGEN_XOR              0
#define     OSCCOMBGEN_AND              1
#define     OSCCOMBGEN_OR               2
#define     OSCCOMBGEN_MULT             3
#define     OSCCOMBGEN_LOFIMULT         4
#define     OSCCOMBGEN_SUB              5
#define     OSCCOMBGEN_ABSMULT          6
#define     OSCCOMBGEN_BIGGER           7
#define     OSCCOMBGEN_SMALLER          8

#define INIT_OSCCOMBGEN(combtype, gen0, gen1) \
    DM(I2, M1) = combtype; \
    DM(I2, M1) = gen0; \
    DM(I2, M1) = gen1

/*----------------------------------------------------------------------------*/
/*----------------------------------ADSRPanEnv-----------------------------------*/
/*----------------------------------------------------------------------------*/

#define     ADSRPANENV_CALLS            20
#define     ADSRPANENV_VARS             12
                                        
#define     ADSRPANENV_UR               0
#define     ADSRPANENV_RC               1
#define     ADSRPANENV_INTSCALAR        2
#define     ADSRPANENV_STAGE            3
#define     ADSRPANENV_ATTACKRATE       4
#define     ADSRPANENV_DECAYRATE        5
#define     ADSRPANENV_DECAYMIN         6
#define     ADSRPANENV_SUSTAINLEN       7
#define     ADSRPANENV_SUSTAINCNT       8
#define     ADSRPANENV_RELRATE          9
#define     ADSRPANENV_SCALAR           10
#define     ADSRPANENV_PAN              11

#define INIT_ADSRPANENV(udrate, arate, drate, dmin, slen, rrate, pan) \
    DM(I2, M1) = udrate; \
    DM(I2, M1) = 0; \
    DM(I2, M1) = 0; \
    DM(I2, M1) = 0; \
    DM(I2, M1) = arate; \
    DM(I2, M1) = drate; \
    DM(I2, M1) = dmin; \
    DM(I2, M1) = slen; \
    DM(I2, M1) = 0; \
    DM(I2, M1) = rrate; \
    DM(I2, M1) = 0; \
    DM(I2, M1) = pan

#define RESET_ADSRPANENV(num) \
    DM(a_ADSRPanEnv + ((num*ADSRPANENV_VARS)+ADSRPANENV_RC)) = M0; \
    DM(a_ADSRPanEnv + ((num*ADSRPANENV_VARS)+ADSRPANENV_INTSCALAR)) = M0; \
    DM(a_ADSRPanEnv + ((num*ADSRPANENV_VARS)+ADSRPANENV_STAGE)) = M0; \
    DM(a_ADSRPanEnv + ((num*ADSRPANENV_VARS)+ADSRPANENV_SUSTAINCNT)) = M0
    

/*----------------------------------------------------------------------------*/
/*---------------------------------AlgoSine-----------------------------------*/
/*----------------------------------------------------------------------------*/

#define     ALGOSINEGEN_CALLS           10
#define     ALGOSINEGEN_VARS            7

#define     ALGOSINEGEN_PHASEINC        0
#define     ALGOSINEGEN_CURRPHASE       1
#define     ALGOSINEGEN_COEFF0          2
#define     ALGOSINEGEN_COEFF1          3
#define     ALGOSINEGEN_COEFF2          4
#define     ALGOSINEGEN_COEFF3          5
#define     ALGOSINEGEN_COEFF4          6

#define INIT_ALGOSINEGEN(freq, coeff0, coeff1, coeff2, coeff3, coeff4) \
    DM(I2, M1) = freq; \
    DM(I2, M1) = 0x0000; \
    DM(I2, M1) = coeff0; \
    DM(I2, M1) = coeff1; \
    DM(I2, M1) = coeff2; \
    DM(I2, M1) = coeff3; \
    DM(I2, M1) = coeff4
    
#define INIT_ALGOSINEGEN_MF(note, coeff0, coeff1, coeff2, coeff3, coeff4) \
    I3 = ^a_MIDIFreq; \
    M2 = note; \
    modify(I3, M2); \
    AR = DM(I3, M0); \
    DM(I2, M1) = AR; \
    DM(I2, M1) = 0x0000; \
    DM(I2, M1) = coeff0; \
    DM(I2, M1) = coeff1; \
    DM(I2, M1) = coeff2; \
    DM(I2, M1) = coeff3; \
    DM(I2, M1) = coeff4

#define RESET_PHASE_ALGOSINEGEN(num) \
    DM(a_AlgoSineGen + ((num*ALGOSINEGEN_VARS)+ALGOSINEGEN_CURRPHASE)) = M0

#define CTRLDATA_TO_ALGOSINEGEN_FREQ(ctrlnum, gennum) \
    AR = DM(a_CTRLData+ctrlnum); \
    DM(a_AlgoSineGen + ((gennum*ALGOSINEGEN_VARS)+ALGOSINEGEN_PHASEINC)) = AR

#define CTRLDATA_TO_ALGOSINEGEN_FREQ_MF(ctrlnum, gennum) \
    M2 = DM(a_CTRLData+ctrlnum); \
    I3 = ^a_MIDIFreq; \
    modify(I3, M2); \
    AR = DM(I3, M0); \
    DM(a_AlgoSineGen + ((gennum*ALGOSINEGEN_VARS)+ALGOSINEGEN_PHASEINC)) = AR
    
/*----------------------------------------------------------------------------*/
/*--------------------------------AlgoSineSat---------------------------------*/
/*----------------------------------------------------------------------------*/

#define     ALGOSINESATGEN_CALLS        10
#define     ALGOSINESATGEN_VARS         7

#define     ALGOSINESATGEN_PHASEINC     0
#define     ALGOSINESATGEN_CURRPHASE    1
#define     ALGOSINESATGEN_COEFF0       2
#define     ALGOSINESATGEN_COEFF1       3
#define     ALGOSINESATGEN_COEFF2       4
#define     ALGOSINESATGEN_COEFF3       5
#define     ALGOSINESATGEN_COEFF4       6

#define INIT_ALGOSINESATGEN(freq, coeff0, coeff1, coeff2, coeff3, coeff4) \
    DM(I2, M1) = freq; \
    DM(I2, M1) = 0x0000; \
    DM(I2, M1) = coeff0; \
    DM(I2, M1) = coeff1; \
    DM(I2, M1) = coeff2; \
    DM(I2, M1) = coeff3; \
    DM(I2, M1) = coeff4
    
#define INIT_ALGOSINESATGEN_MF(note, coeff0, coeff1, coeff2, coeff3, coeff4) \
    I3 = ^a_MIDIFreq; \
    M2 = note; \
    modify(I3, M2); \
    AR = DM(I3, M0); \
    DM(I2, M1) = AR; \
    DM(I2, M1) = 0x0000; \
    DM(I2, M1) = coeff0; \
    DM(I2, M1) = coeff1; \
    DM(I2, M1) = coeff2; \
    DM(I2, M1) = coeff3; \
    DM(I2, M1) = coeff4

#define RESET_PHASE_ALGOSINESATGEN(num) \
    DM(a_AlgoSineSatGen + ((num*ALGOSINESATGEN_VARS)+ALGOSINESATGEN_CURRPHASE)) = M0

#define CTRLDATA_TO_ALGOSINESATGEN_FREQ_MF(ctrlnum, gennum) \
    M2 = DM(a_CTRLData+ctrlnum); \
    I3 = ^a_MIDIFreq; \
    modify(I3, M2); \
    AR = DM(I3, M0); \
    DM(a_AlgoSineSatGen + ((gennum*ALGOSINESATGEN_VARS)+ALGOSINESATGEN_PHASEINC)) = AR
    
/*----------------------------------------------------------------------------*/
/*-----------------------------------LFO3-------------------------------------*/
/*----------------------------------------------------------------------------*/

#define     LFO_CALLS                  32
#define     LFO_VARS                   8
                                        
#define     LFO_HOLDPER                0
#define     LFO_CURRPER                1
#define     LFO_FREQ                   2
#define     LFO_PHASE                  3
#define     LFO_PTR                    4
#define     LFO_MOD                    5
#define     LFO_BASE                   6
#define     LFO_ADDR                   7

#define INIT_LFO(holdperiod, freq, LFOWTptr, modamount, baseamount, modaddr) \
    DM(I2, M1) = holdperiod; \
    DM(I2, M1) = 1; \
    DM(I2, M1) = freq; \
    DM(I2, M1) = 0x0000; \
    DM(I2, M1) = LFOWTptr; \
    DM(I2, M1) = modamount; \
    DM(I2, M1) = baseamount; \
    DM(I2, M1) = modaddr

#define RESET_PHASE_LFO(num) \
    DM(a_LFO + ((num*LFO_VARS)+LFO_PHASE)) = M0

/*----------------------------------------------------------------------------*/
/*-----------------------------------SVF--------------------------------------*/
/*----------------------------------------------------------------------------*/

#define     SVFFX_CALLS                 12
#define     SVFFX_VARS                  4
                                        
#define     SVFFILTOUTCALLS             SVFFX_CALLS
#define     SVFFILTOUTVARS              4
                                        
#define     SVFFX_K2                    0
#define     SVFFX_K3                    1
#define     SVFFX_K4                    2
#define     SVFFX_TYPE                  3

#define     SVFFX_BANDOUT               0
#define     SVFFX_LOWOUT                1
#define     SVFFX_HIGHOUT               2
#define     SVFFX_NOTCHOUT              3

#define INIT_SVFFX(K2, K3, K4, type) \
    DM(I2, M1) = K2; \
    DM(I2, M1) = K3; \
    DM(I2, M1) = K4; \
    DM(I2, M1) = type

/*----------------------------------------------------------------------------*/
/*----------------------------------KSGen-------------------------------------*/
/*----------------------------------------------------------------------------*/

#define     KSGEN_VARS                  5
#define     KSGEN_CALLS                 8
                                        
#define     KSGEN_ADDSUB                0
#define     KSGEN_BUFFPTR               1
#define     KSGEN_FREQ                  2
#define     KSGEN_AVEFACSIGN            3
#define     KSGEN_AVEFAC                4

#define INIT_KSGEN(addsub, buffptr, freq, avefacsign, avefac) \
    DM(I2, M1) = addsub; \
    DM(I2, M1) = buffptr; \
    DM(I2, M1) = freq; \
    DM(I2, M1) = avefacsign; \
    DM(I2, M1) = avefac
    
#define INIT_KSGEN_MF(addsub, buffptr, note, avefacsign, avefac) \
    DM(I2, M1) = addsub; \
    DM(I2, M1) = buffptr; \
    I3 = ^a_MIDIFreqKS; \
    M2 = note; \
    modify(I3, M2); \
    AR = DM(I3, M0); \
    DM(I2, M1) = AR; \
    DM(I2, M1) = avefacsign; \
    DM(I2, M1) = avefac

#define CTRLDATA_TO_KSGEN_FREQ_MF(ctrlnum, gennum) \
    M2 = DM(a_CTRLData+ctrlnum); \
    I3 = ^a_MIDIFreqKS; \
    modify(I3, M2); \
    AR = DM(I3, M0); \
    DM(a_KSGen + ((gennum*KSGEN_VARS)+KSGEN_FREQ)) = AR
    
#define CTRLDATA_TO_KSGEN_FREQ(ctrlnum, gennum) \
    AR = DM(a_CTRLData+ctrlnum); \
    DM(a_KSGen + ((gennum*KSGEN_VARS)+KSGEN_FREQ)) = AR

#define RESET_KSGEN(num) \
    I5 = ^FillKS_JT; \
    M5 = num; \
    modify(I5, M5); \
    call (I5) \

/*----------------------------------------------------------------------------*/
/*---------------------------------DelaySyn-----------------------------------*/
/*----------------------------------------------------------------------------*/

/* note that because DelaySyn has variable # of parameters, cannot use macro */
#define     DELAYSYNTHGEN_VARS          100
#define     DELAYSYNTHGEN_CALLS         1

/*----------------------------------------------------------------------------*/
/*---------------------------------HPWTGen2-----------------------------------*/
/*----------------------------------------------------------------------------*/

#define     HPWTGEN2_VARS               5
#define     HPWTGEN2_CALLS              8

#define     HPWTGEN2_FREQMSW            0
#define     HPWTGEN2_FREQLSW            1
#define     HPWTGEN2_PHASEMSW           2
#define     HPWTGEN2_PHASELSW           3
#define     HPWTGEN2_WTPTR              4

#define INIT_HPWTGEN2(freqmsw, freqlsw, WTptr) \
    DM(I2, M1) = freqmsw; \
    DM(I2, M1) = freqlsw; \
    DM(I2, M1) = 0x0000; \
    DM(I2, M1) = 0x0000; \
    DM(I2, M1) = WTptr

#define RESET_PHASE_HPWTGEN(num) \
    DM(a_HPWTGen2 + ((num*HPWTGEN2_VARS)+HPWTGEN2_PHASEMSW)) = M0; \
    DM(a_HPWTGen2 + ((num*HPWTGEN2_VARS)+HPWTGEN2_PHASELSW)) = M0
    
#define CTRLDATA_TO_HPWTGEN_MSWFREQ(ctrlnum0, gennum) \
    AR = DM(a_CTRLData+ctrlnum0); \
    DM(a_HPWTGen2 + ((gennum*HPWTGEN2_VARS)+HPWTGEN2_FREQMSW)) = AR; \
    
#define CTRLDATA_TO_HPWTGEN_FREQ(ctrlnum0, ctrlnum1, gennum) \
    AR = DM(a_CTRLData+ctrlnum0); \
    DM(a_HPWTGen2 + ((gennum*HPWTGEN2_VARS)+HPWTGEN2_FREQMSW)) = AR; \
    AR = DM(a_CTRLData+ctrlnum1); \
    DM(a_HPWTGen2 + ((gennum*HPWTGEN2_VARS)+HPWTGEN2_FREQLSW)) = AR

/*----------------------------------------------------------------------------*/
/*----------------------------------WTGen-------------------------------------*/
/*----------------------------------------------------------------------------*/

#define     WTGEN_VARS                  3
#define     WTGEN_CALLS                 8
                                        
#define     WTGEN_PHASE                 0
#define     WTGEN_FREQ                  1
#define     WTGEN_WTPTR                 2
                                        
#define INIT_WTGEN(freq, WTptr) \
    DM(I2, M1) = 0x0000; \
    DM(I2, M1) = freq; \
    DM(I2, M1) = WTptr
    
#define INIT_WTGEN_MF(note, WTptr) \
    DM(I2, M1) = 0x0000; \
    I3 = ^a_MIDIFreq; \
    M2 = note; \
    modify(I3, M2); \
    AR = DM(I3, M0); \
    DM(I2, M1) = AR; \
    DM(I2, M1) = WTptr
    
#define RESET_PHASE_WTGEN(num) \
    DM(a_WTGen + ((num*WTGEN_VARS)+WTGEN_PHASE)) = M0

#define CTRLDATA_TO_WTGEN_FREQ_MF(ctrlnum, gennum) \
    M2 = DM(a_CTRLData+ctrlnum); \
    I3 = ^a_MIDIFreq; \
    modify(I3, M2); \
    AR = DM(I3, M0); \
    DM(a_WTGen + ((gennum*WTGEN_VARS)+WTGEN_FREQ)) = AR
    
#define CTRLDATA_TO_WTGEN_FREQ(ctrlnum, gennum) \
    AR = DM(a_CTRLData+ctrlnum); \
    DM(a_WTGen + ((gennum*WTGEN_VARS)+WTGEN_FREQ)) = AR

/*----------------------------------------------------------------------------*/
/*----------------------------------WTGen2------------------------------------*/
/*----------------------------------------------------------------------------*/

#define     WTGEN2_VARS                 3
#define     WTGEN2_CALLS                24

#define     WTGEN2_FREQ                 0
#define     WTGEN2_PHASE                1
#define     WTGEN2_PTR                  2

#define INIT_WTGEN2(freq, WTptr) \
    DM(I2, M1) = freq; \
    DM(I2, M1) = 0x0000; \
    DM(I2, M1) = WTptr
    
#define INIT_WTGEN2_MF(note, WTptr) \
    I3 = ^a_MIDIFreq; \
    M2 = note; \
    modify(I3, M2); \
    AR = DM(I3, M0); \
    DM(I2, M1) = freq; \
    DM(I2, M1) = 0x0000; \
    DM(I2, M1) = WTptr
    
#define RESET_PHASE_WTGEN2(num) \
    DM(a_WTGen2 + ((num*WTGEN2_VARS)+WTGEN2_PHASE)) = M0
    
#define CTRLDATA_TO_WTGEN2_FREQ_MF(ctrlnum, gennum) \
    M2 = DM(a_CTRLData+ctrlnum); \
    I3 = ^a_MIDIFreq; \
    modify(I3, M2); \
    AR = DM(I3, M0); \
    DM(a_WTGen2 + ((gennum*WTGEN2_VARS)+WTGEN2_FREQ)) = AR
    
#define CTRLDATA_TO_WTGEN2_FREQ(ctrlnum, gennum) \
    AR = DM(a_CTRLData+ctrlnum); \
    DM(a_WTGen2 + ((gennum*WTGEN2_VARS)+WTGEN2_FREQ)) = AR

/*----------------------------------------------------------------------------*/
/*--------------------------------WTGenSyncFX---------------------------------*/
/*----------------------------------------------------------------------------*/

#define     WTGENSYNCFX_VARS            4
#define     WTGENSYNCFX_CALLS           8

#define     WTGENSYNCFX_BASEOSC         0
#define     WTGENSYNCFX_OSCNUM0         1
#define     WTGENSYNCFX_OSCNUM1         2
#define     WTGENSYNCFX_INTVAR          3

#define INIT_WTGENSYNCFX(baseosc0, oscnum0, oscnum1) \
    DM(I2, M1) = baseosc0; \
    DM(I2, M1) = oscnum0; \
    DM(I2, M1) = oscnum1; \
    DM(I2, M1) = 0x0000
    
/*----------------------------------------------------------------------------*/
/*---------------------------------WaveShape----------------------------------*/
/*----------------------------------------------------------------------------*/

#define     WAVESHAPEFX_VARS            12
#define     WAVESHAPEFX_CALLS           8

#define     WAVESHAPEFX_CONTINUITY      0
#define     WAVESHAPEFX_CUTOFF          1
#define     WAVESHAPEFX_MSWSCALE0       2
#define     WAVESHAPEFX_LSWSCALE0       3
#define     WAVESHAPEFX_MSWSCALE1       4
#define     WAVESHAPEFX_LSWSCALE1       5

#define     WAVESHAPEFX_DISCONTINUOUS   0
#define     WAVESHAPEFX_CONTINUOUS      1

#define INIT_WAVESHAPEFX(continuity, cutoff, mswscale0, lswscale0, mswscale1, lswscale1) \
    DM(I2, M1) = continuity; \
    DM(I2, M1) = cutoff; \
    DM(I2, M1) = mswscale0; \
    DM(I2, M1) = lswscale0; \
    DM(I2, M1) = mswscale1; \
    DM(I2, M1) = lswscale1
    
/*----------------------------------------------------------------------------*/
/*---------------------------------MultiGen-----------------------------------*/
/*----------------------------------------------------------------------------*/

#define     MULTIGEN_VARS               50

/*----------------------------------------------------------------------------*/
/*-----------------------------------Seq2-------------------------------------*/
/*----------------------------------------------------------------------------*/

#define     SEQ_VARS                   15
#define     MAXTRACKS                  32
                                        
#define     SEQ_TRIGCNT                0
#define     SEQ_TRIGRATE               1
#define     SEQ_BASETRIGRATE           2
#define     SEQ_SWINGPER               3
#define     SEQ_SWINGCNT               4
#define     SEQ_SWINGAMOUNT            5
#define     SEQ_SEQLEN                 6
#define     SEQ_BASETRIGPTR            7
#define     SEQ_CURRTRIGPTR            8
#define     SEQ_VOLPTR                 9
#define     SEQ_CTRLTRACKPTR           10
#define     SEQ_INITFUNC               11
#define     SEQ_ENVTYPE                12
#define     SEQ_ENVNUM                 13
#define     SEQ_AUXFUNC                14

/*
 *  important!  the order of inits here must mirror the order of inits at label
 *  IEC1_JT.
 */

#define     NOOUTPUT                    0
#define     ADSRENV                     1
#define     EXPDECAYENV                 2
#define     MEMENV3ENV                  3
                                        
#define     NUMENVTYPES                 4

#define INIT_SEQ(ticrate, swingper, swingamount, seqlen, envtype) \
    DM(I2, M1) = 0x7fff; \
    DM(I2, M1) = ticrate; \
    DM(I2, M1) = ticrate; \
    DM(I2, M1) = swingper; \
    DM(I2, M1) = 0x7fff; \
    DM(I2, M1) = swingamount; \
    DM(I2, M1) = seqlen; \
    DM(I2, M1) = NULLPTR; \
    DM(I2, M1) = NULLPTR; \
    DM(I2, M1) = NULLPTR; \
    DM(I2, M1) = NULLPTR; \
    DM(I2, M1) = NULLPTR; \
    DM(I2, M1) = envtype; \
    DM(I2, M1) = 0x0000; \
    DM(I2, M1) = ^DummyRet

#define SEQ_SET_MEASURE(num) \
    MX0 = DM(a_Seq+1+((num*SEQ_VARS)+SEQ_TRIGRATE)); \
    MY0 = DM(a_Seq+1+((num*SEQ_VARS)+SEQ_SEQLEN)); \
    MR = MX0 * MY0 (SS); \
    SR = ashift MR0 by -1 (LO); \
    DM(v_TicsPerMeasure) = SR0
    
#define SET_TICS_PER_MEASURE(num) \
    I2 = ^v_TicsPerMeasure; \
    DM(I2, M0) = num
    
/*----------------------------------------------------------------------------*/
/*-----------------------offsets for a_MIDIFreq array-------------------------*/
/*----------------------------------------------------------------------------*/

#define     OCT                         12

#define     C_N1                        0
#define     CS_N1                       1
#define     DF_N1                       1
#define     D_N1                        2
#define     DS_N1                       3
#define     EF_N1                       3
#define     E_N1                        4
#define     F_N1                        5
#define     FS_N1                       6
#define     GF_N1                       6
#define     G_N1                        7
#define     GS_N1                       8
#define     AF_N1                       8
#define     A_N1                        9
#define     AS_N1                       10
#define     BF_N1                       10
#define     B_N1                        11

#define     C_0                         C_N1+OCT
#define     CS_0                        CS_N1+OCT
#define     DF_0                        DF_N1+OCT
#define     D_0                         D_N1+OCT
#define     DS_0                        DS_N1+OCT
#define     EF_0                        EF_N1+OCT
#define     E_0                         E_N1+OCT
#define     F_0                         F_N1+OCT
#define     FS_0                        FS_N1+OCT
#define     GF_0                        GF_N1+OCT
#define     G_0                         G_N1+OCT
#define     GS_0                        GS_N1+OCT
#define     AF_0                        AF_N1+OCT
#define     A_0                         A_N1+OCT
#define     AS_0                        AS_N1+OCT
#define     BF_0                        BF_N1+OCT
#define     B_0                         B_N1+OCT

#define     C_1                         C_0+OCT
#define     CS_1                        CS_0+OCT
#define     DF_1                        DF_0+OCT
#define     D_1                         D_0+OCT
#define     DS_1                        DS_0+OCT
#define     EF_1                        EF_0+OCT
#define     E_1                         E_0+OCT
#define     F_1                         F_0+OCT
#define     FS_1                        FS_0+OCT
#define     GF_1                        GF_0+OCT
#define     G_1                         G_0+OCT
#define     GS_1                        GS_0+OCT
#define     AF_1                        AF_0+OCT
#define     A_1                         A_0+OCT
#define     AS_1                        AS_0+OCT
#define     BF_1                        BF_0+OCT
#define     B_1                         B_0+OCT

#define     C_2                         C_1+OCT
#define     CS_2                        CS_1+OCT
#define     DF_2                        DF_1+OCT
#define     D_2                         D_1+OCT
#define     DS_2                        DS_1+OCT
#define     EF_2                        EF_1+OCT
#define     E_2                         E_1+OCT
#define     F_2                         F_1+OCT
#define     FS_2                        FS_1+OCT
#define     GF_2                        GF_1+OCT
#define     G_2                         G_1+OCT
#define     GS_2                        GS_1+OCT
#define     AF_2                        AF_1+OCT
#define     A_2                         A_1+OCT
#define     AS_2                        AS_1+OCT
#define     BF_2                        BF_1+OCT
#define     B_2                         B_1+OCT

#define     C_3                         C_2+OCT
#define     CS_3                        CS_2+OCT
#define     DF_3                        DF_2+OCT
#define     D_3                         D_2+OCT
#define     DS_3                        DS_2+OCT
#define     EF_3                        EF_2+OCT
#define     E_3                         E_2+OCT
#define     F_3                         F_2+OCT
#define     FS_3                        FS_2+OCT
#define     GF_3                        GF_2+OCT
#define     G_3                         G_2+OCT
#define     GS_3                        GS_2+OCT
#define     AF_3                        AF_2+OCT
#define     A_3                         A_2+OCT
#define     AS_3                        AS_2+OCT
#define     BF_3                        BF_2+OCT
#define     B_3                         B_2+OCT

#define     C_4                         C_3+OCT
#define     CS_4                        CS_3+OCT
#define     DF_4                        DF_3+OCT
#define     D_4                         D_3+OCT
#define     DS_4                        DS_3+OCT
#define     EF_4                        EF_3+OCT
#define     E_4                         E_3+OCT
#define     F_4                         F_3+OCT
#define     FS_4                        FS_3+OCT
#define     GF_4                        GF_3+OCT
#define     G_4                         G_3+OCT
#define     GS_4                        GS_3+OCT
#define     AF_4                        AF_3+OCT
#define     A_4                         A_3+OCT
#define     AS_4                        AS_3+OCT
#define     BF_4                        BF_3+OCT
#define     B_4                         B_3+OCT

#define     C_5                         C_4+OCT
#define     CS_5                        CS_4+OCT
#define     DF_5                        DF_4+OCT
#define     D_5                         D_4+OCT
#define     DS_5                        DS_4+OCT
#define     EF_5                        EF_4+OCT
#define     E_5                         E_4+OCT
#define     F_5                         F_4+OCT
#define     FS_5                        FS_4+OCT
#define     GF_5                        GF_4+OCT
#define     G_5                         G_4+OCT
#define     GS_5                        GS_4+OCT
#define     AF_5                        AF_4+OCT
#define     A_5                         A_4+OCT
#define     AS_5                        AS_4+OCT
#define     BF_5                        BF_4+OCT
#define     B_5                         B_4+OCT

#define     C_6                         C_5+OCT
#define     CS_6                        CS_5+OCT
#define     DF_6                        DF_5+OCT
#define     D_6                         D_5+OCT
#define     DS_6                        DS_5+OCT
#define     EF_6                        EF_5+OCT
#define     E_6                         E_5+OCT
#define     F_6                         F_5+OCT
#define     FS_6                        FS_5+OCT
#define     GF_6                        GF_5+OCT
#define     G_6                         G_5+OCT
#define     GS_6                        GS_5+OCT
#define     AF_6                        AF_5+OCT
#define     A_6                         A_5+OCT
#define     AS_6                        AS_5+OCT
#define     BF_6                        BF_5+OCT
#define     B_6                         B_5+OCT

#define     C_7                         C_6+OCT
#define     CS_7                        CS_6+OCT
#define     DF_7                        DF_6+OCT
#define     D_7                         D_6+OCT
#define     DS_7                        DS_6+OCT
#define     EF_7                        EF_6+OCT
#define     E_7                         E_6+OCT
#define     F_7                         F_6+OCT
#define     FS_7                        FS_6+OCT
#define     GF_7                        GF_6+OCT
#define     G_7                         G_6+OCT
#define     GS_7                        GS_6+OCT
#define     AF_7                        AF_6+OCT
#define     A_7                         A_6+OCT
#define     AS_7                        AS_6+OCT
#define     BF_7                        BF_6+OCT
#define     B_7                         B_6+OCT

#define     C_8                         C_7+OCT
#define     CS_8                        CS_7+OCT
#define     DF_8                        DF_7+OCT
#define     D_8                         D_7+OCT
#define     DS_8                        DS_7+OCT
#define     EF_8                        EF_7+OCT
#define     E_8                         E_7+OCT
#define     F_8                         F_7+OCT
#define     FS_8                        FS_7+OCT
#define     GF_8                        GF_7+OCT
#define     G_8                         G_7+OCT

/*
 *  macros used in GenFXIni and other miscellaneous operations/constants
 */

#define NUMCTRLTRACKS(numctrltracks) \
    DM(I2, M1) = numctrltracks
    
#define CONTROLTRACKSTEP(steptype) \
    DM(I2, M1) = steptype

#define INIT_AP_CTRLTRACK(len, ptr) \
    DM(I2, M1) = len; \
    DM(I2, M1) = ptr

#define     ALWAYSSTEP                  0
#define     STEPONRETRIG                1

#define     RANDLEN                     511
#define     OUTBUFFLEN                  128

/* not really used for anything, just for clarity in some inits */
#define     NULLPTR                     0x0000

#define BASEPLUSRAND_AR(base, rand) \
    ena m_mode; \
    AR = DM(I6, M7); \
    AR = abs AR; \
    MY0 = rand; \
    MR = AR * MY0 (uu); \
    dis m_mode; \
    AY0 = base; \
    AR = MR1 + AY0
    
#define BASEMINUSRAND_AR(base, rand) \
    ena m_mode; \
    AR = DM(I6, M7); \
    AR = abs AR; \
    AR = neg AR; \
    MY0 = rand; \
    MR = AR * MY0 (ss); \
    dis m_mode; \
    AY0 = base; \
    AR = MR1 + AY0
    