/******************************************************************************
*
*  GenFXIni.dsp
*
*  (c) 2001, 2002, 2003 ethan bordeaux
*  syndevkit@dspmusic.org
*
*  initialization of all generators, fx, envelopes and sequencer.  called at
*  the beginning of the song before audio output.  extensive post processing
*  and analysis occurs on this function, so follow the guidelines on usage as
*  outlined in the documentation.
*
******************************************************************************/

.MODULE                                     modGenFXIni_00;

/*
 *  include files for all functions
 */
 

#include    ".\comsys\hdr\DSPdefs.h"
#include    ".\gen_fx\hdr\GenFX.h"
#include    "ex2.h"

/*
 *  external variables and functions
 */

/*
 *  entry points
 */

.ENTRY      InitGMParams_00;
.ENTRY      EndInitGMParams_00;

InitGMParams_00:
    /* insert generators and fx initializations here */
    SETPTR(a_WTGen2);
    INIT_WTGEN2(100, ^a_WTSine);
    INIT_WTGEN2(0, ^a_WTSine);
    
    SETPTR(a_KSGen);
    INIT_KSGEN(0, ^a_KSBuff0, 0x14f, 0xe000, 0x4c00);
    INIT_KSGEN(0, ^a_KSBuff1, 0x14f, 0xe000, 0x4c00);
    INIT_KSGEN(1, ^a_KSBuff2, 0x1f, 0x1000, 0x5000);
            
    SETPTR(a_FM2Op0Gen);
    INIT_FM2OP0GEN(55, ^a_WTSine, 0, 128, 0x0080, 0x0040, 0x0000, 0, 0x0000, 220, 0, ^a_WTSine);
    INIT_FM2OP0GEN(110, ^a_WTSine, 0, 128, 0x7fff, 0x0200, 0x0000, 0, 0x0000, 330, 0, ^a_WTSine);
    INIT_FM2OP0GEN(220, ^a_WTSine, 0, 128, 0x7fff, 0x0400, 0x0000, 0, 0x0000, 440, 0, ^a_WTSine);
    
    SETPTR(a_ProbSynthGen);
    INIT_PROBSYNTHGEN(128*42, 20, 30, 40, 50, 60, 70, 80, 90);
    INIT_PROBSYNTHGEN(128*42, 20, 30, 40, 50, 60, 70, 80, 90);
    
    SETPTR(a_GenSHFX);
    INIT_GENSHFX(0);
    INIT_GENSHFX(0);
    
    SETPTR(a_ADSRPanEnv);
    INIT_ADSRPANENV(128, 0x7fff, 0x0010, 0x0000, 0, 0x0000, 0x1000);
    INIT_ADSRPANENV(128, 0x7fff, 0x0010, 0x0000, 0, 0x0000, 0x7000);
    INIT_ADSRPANENV(128, 0x7fff, 0x0010, 0x0000, 0, 0x0000, 0x4000);
    INIT_ADSRPANENV(128, 0x0080, 0x0040, 0x0000, 0, 0x0000, 0x4000);
    INIT_ADSRPANENV(512, 0x7fff, 0x0000, 0x0000, 0, 0x0000, 0x4000);
        
    SETPTR(a_ExpDecayEnv);
    INIT_EXPDECAYENV(4, 0x7ff0, 0x4000);
    INIT_EXPDECAYENV(4, 0x7ff0, 0x4000);
    INIT_EXPDECAYENV(2, 0x7ff0, 0x4000);
    
    SETPTR(a_SVFFX);
    INIT_SVFFX(0x0400, 0x4000, 0x4000, SVFFX_BANDOUT);
        
    SETPTR(a_LFO);
    INIT_LFO(14, 39, ^a_WTSine, 0x1000, 0x4000, ^a_SVFFX+(0*SVFFX_VARS)+SVFFX_K3);
    INIT_LFO(14, 39, ^a_WTSine, 0x1000, 0x4000, ^a_SVFFX+(0*SVFFX_VARS)+SVFFX_K4);
    INIT_LFO(28, 39, ^a_WTSine, 0x8, 0x1f, ^a_KSGen+(2*KSGEN_VARS)+KSGEN_FREQ);
    INIT_LFO(1, 128*220, ^a_WTSine, 220, 0, ^a_WTGen2+(1*WTGEN2_VARS)+WTGEN2_FREQ);
    
    SETPTR(a_MemEnv3);
    INIT_MEMENV3(^a_WTGen2+(0*WTGEN2_VARS)+WTGEN2_FREQ, 120, 120, 1, 0, 120);
    INIT_MEMENV3(^a_LFO+(3*LFO_VARS)+LFO_MOD, 220, 220, 1, 0, 60);
           
    /* init TrigTrack and VolTrack arrays as needed */    
    AR = 100;
    AX0 = 50;
    AY0 = 0x2000;
    AY1 = 0x3000;
    
    DM(a_TrigTrack00+0) = AR;
    DM(a_TrigTrack00+16) = AR;
    DM(a_TrigTrack00+20) = AR;
    DM(a_TrigTrack00+28) = AR;
    DM(a_VolTrack00+0) = AY0;
    DM(a_VolTrack00+16) = AY0;
    DM(a_VolTrack00+20) = AY0;
    DM(a_VolTrack00+28) = AY0;
    
    AY0 = 0x3000;
    AX0 = 30;
    AY1 = 0x1000;
    
    DM(a_TrigTrack01+8) = AR;
    DM(a_TrigTrack01+24) = AR;
    DM(a_VolTrack01+8) = AY0;
    DM(a_VolTrack01+24) = AY0;
    
    DM(a_TrigTrack02+8) = AR;
    DM(a_TrigTrack02+24) = AR;
    DM(a_VolTrack02+8) = AY0;
    DM(a_VolTrack02+24) = AY0;
    
    DM(a_TrigTrack03+0) = AR;
    DM(a_TrigTrack03+1) = AX0;
    DM(a_VolTrack03+0) = AY0;
    DM(a_VolTrack03+1) = AY1;
    
    DM(a_TrigTrack04+0) = AR;
    DM(a_VolTrack04+0) = AY1;
    
    AX1 = 70;
    DM(a_TrigTrack05+0) = AR;
    DM(a_TrigTrack05+1) = AX1;
    DM(a_VolTrack05+0) = AY1;
    DM(a_VolTrack05+1) = AY1;
    
    AY1 = 0x0700;
    DM(a_TrigTrack06+0) = AR;
    DM(a_VolTrack06+0) = AY1;
        
    AY0 = 0x0600;
    AY1 = 0x0300;
    AX0 = 15;
    DM(a_TrigTrack07+0) = AR;
    DM(a_TrigTrack07+2) = AR;
    DM(a_TrigTrack07+4) = AR;
    DM(a_TrigTrack07+6) = AR;
    DM(a_TrigTrack07+7) = AX0;
    DM(a_VolTrack07+0) = AY0;
    DM(a_VolTrack07+2) = AY1;
    DM(a_VolTrack07+4) = AY0;
    DM(a_VolTrack07+6) = AY1;
    DM(a_VolTrack07+7) = AY1;
    

    SETPTR(ap_CTRLTrack04);
    NUMCTRLTRACKS(NUMCTRLTRACK04);
    CONTROLTRACKSTEP(ALWAYSSTEP);
    INIT_AP_CTRLTRACK(LENCTRLTRACK04_0, ^a_CTRLTrack04_0);
    
    SETPTR(ap_CTRLTrack05);
    NUMCTRLTRACKS(NUMCTRLTRACK05);
    CONTROLTRACKSTEP(ALWAYSSTEP);
    INIT_AP_CTRLTRACK(LENCTRLTRACK05_0, ^a_CTRLTrack05_0);
    
    SETPTR(ap_CTRLTrack07);
    NUMCTRLTRACKS(NUMCTRLTRACK07);
    CONTROLTRACKSTEP(ALWAYSSTEP);
    INIT_AP_CTRLTRACK(LENCTRLTRACK07_0, ^a_CTRLTrack07_0);
    
    I2 = ^a_CTRLTrack04_0;
    AR = DM(a_MIDIFreq+C_3);
    DM(I2, M1) = AR;
    AR = DM(a_MIDIFreq+E_3);
    DM(I2, M1) = AR;
    AR = DM(a_MIDIFreq+G_3);
    DM(I2, M1) = AR;
    AR = DM(a_MIDIFreq+D_3);
    DM(I2, M1) = AR;
    
    I2 = ^a_CTRLTrack05_0;
    AR = DM(a_MIDIFreq+C_4);
    DM(I2, M1) = AR;
    AR = DM(a_MIDIFreq+E_4);
    DM(I2, M1) = AR;
    AR = DM(a_MIDIFreq+G_4);
    DM(I2, M1) = AR;
    AR = DM(a_MIDIFreq+D_4);
    DM(I2, M1) = AR;
    AR = DM(a_MIDIFreq+G_4);
    DM(I2, M1) = AR;
    AR = DM(a_MIDIFreq+D_4);
    DM(I2, M1) = AR;
    
    I2 = ^a_CTRLTrack07_0;
    AR = DM(a_MIDIFreq+C_5);
    DM(I2, M1) = AR;
    AR = DM(a_MIDIFreq+E_5);
    DM(I2, M1) = AR;
    AR = DM(a_MIDIFreq+G_5);
    DM(I2, M1) = AR;
    AR = DM(a_MIDIFreq+D_5);
    DM(I2, M1) = AR;
    AR = DM(a_MIDIFreq+G_5);
    DM(I2, M1) = AR;
    AR = DM(a_MIDIFreq+D_5);
    DM(I2, M1) = AR;
        
/*
 *  init sequencer array
 *  
 *  param0:  tic period
 *  param1:  swing period (num tics between slow/fast transition)
 *  param2:  swing amount
 *  param3:  sequencer length
 *  param4:  envelope type
 *
 */

    SETPTR(a_Seq);
    modify(I2, M1);                         /* numtrax set by CalcSeq2Tracks */

    INIT_SEQ(28, 0, 0, 32, EXPDECAYENV);
    INIT_SEQ(28, 0, 0, 32, ADSRENV);
    INIT_SEQ(28, 0, 0, 32, ADSRENV);
    INIT_SEQ(28, 4, 3, 2, ADSRENV);
    INIT_SEQ(28*32, 0, 0, 1, ADSRENV);
    INIT_SEQ(28*4, 0, 0, 2, EXPDECAYENV);
    INIT_SEQ(28*64, 0, 0, 1, ADSRENV);
    INIT_SEQ(28, 0, 0, 8, EXPDECAYENV);

    /* must be called after Seq2 init to set env and num tracks */
    call Seq2PostProc;
    
    /* set v_TicsPerMeasure to length of first sequence */
    SEQ_SET_MEASURE(0);

    AR = ^InitGMParams_00;
    AX0 = ^EndInitGMParams_00;
    call FillGenFXPtrInits;
    
    /* init ^Env and set length of array of ptrs used in GenFXPtrInits */
    call GenFXIniPostProc;                  /* do not remove! */
    
    /* mute tracks here */
    

EndInitGMParams_00:
    rts;

.ENDMOD;
