/******************************************************************************
*
*  SongCTRL.dsp
*
*  (c) 2001, 2002, 2003 ethan bordeaux
*  syndevkit@dspmusic.org
*
*  function for modifying parameters on a measure by measure basis.  execution
*  rate determined by v_TicsPerMeasure (set in GenFXIni.dsp by macros).
*
******************************************************************************/

.MODULE                                     modSongCTRL_00;

/*
 *  include files for all functions
 */


#include    ".\comsys\hdr\DSPdefs.h"
#include    ".\gen_fx\hdr\GenFX.h"
#include    "exp3_3.h"

/*
 *  external variables and functions
 */

/*
 *  entry points
 */

.ENTRY      SongCTRL_00;
.ENTRY      Measure_JT;
.ENTRY      EndMeasure_JT;

SongCTRL_00:
    AR = DM(v_TicCNTR);
    AR = AR + 1;
    DM(v_TicCNTR) = AR;
    AY0 = DM(v_TicsPerMeasure);
    AR = AR - AY0;
    if LT rts;
    
    DM(v_TicCNTR) = M0;

/*
 *  note!  this initialization used to be in InitFunc.dsp.  however, to make
 *  this function work with multiple tracks in a single executable, Measure_JT
 *  and EndMeasure_JT would need to be abstracted and inited in SongPtrs.dsp.
 *  to try and simplify and reduce the number of initializations that occur in
 *  that function, these variables were made local and it is continually
 *  re-inited here.  note that this code runs in the krate loop so performance
 *  is not a concern.
 */

SetMeasuresPerSong:
    AX0 = ^EndMeasure_JT;
    AY0 = ^Measure_JT;
    AR = AX0 - AY0;
    DM(v_MeasuresPerSong) = AR;
    
    I5 = ^Measure_JT;
    M5 = DM(v_CurrMeasure);
    modify(I5, M5);
    jump (I5);
    
/*
 *  insert JT of operations to perform at the start of every new measure
 */
    
Measure_JT:
    jump Measure0;                              ! 1st bd, snare, 1st fm
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    
    jump Measure4;                              ! hihat, more bd
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
   
    jump Measure8;                              ! 2nd bd
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    
    jump Measure12;                             ! revnoise and 2nd fm
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    
    jump Measure20;                             ! bass, 8 measure filter mod
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;

    jump Measure28;                             ! pad
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    
    ! breakdown, remove lots of stuff, bring in 2 leads, change lead freq
    jump Measure32;
    jump DoNothing;

    jump Measure34;                             ! 2nd lead, 1st freq
    jump DoNothing;
    
    jump Measure36;                             ! 2nd lead freq
    jump DoNothing;
    jump Measure38;                             ! more pad, mod pad freq
    jump DoNothing;
    
    jump Measure40;                             ! add rhythmsaw
    jump Measure41;                             ! add bd
    jump Measure42;                             ! add hh
    jump Measure43;                             ! new fm patterm
    
    jump Measure44;                             ! kill, fade in bass
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    
    jump Measure48;                             ! nothing, everything in
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;

    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    
    jump Measure56;                             ! new bd pattern
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    
    jump Measure60;                             ! new snare pattern
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    
    jump Measure64;                             ! fade bd
    jump DoNothing;
    jump Measure66;                             ! fade hh, rhythmsaw
    jump DoNothing;
    
    jump Measure68;                             ! bass
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    
    jump Measure76;                             ! fade leads
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    
    jump DoNothing;
    jump DoNothing;
    
    jump Measure80;                             ! bd one more time
    jump DoNothing;
    jump DoNothing;
    
    jump Measure82;                             ! hh one more time
    jump DoNothing;
    jump DoNothing;
    
    jump Measure84;                             ! mute all but noise, pad
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    
    jump Measure88;                             ! kill all but pad
    jump Measure89;                             ! kill pad
    jump DoNothing;
    jump DoNothing;
    
EndMeasure_JT:

/*
 *  insert all multimeasure operations - for parameters that should be modified
 *  over multiple measures
 */

MultiMeasureCheck:

EndSongCheck:
    AR = DM(v_CurrMeasure);
    AR = AR + 1;
    DM(v_CurrMeasure) = AR;
    AY0 = DM(v_MeasuresPerSong);
    AR = AR - AY0;
    if LT rts;

/*
 *  SongCTRL list is complete.  jumping to EndSong will start the next song
 *  in a multisong environment.  resetting CurrMeasure starts the Measure_JT
 *  loop over again, causing the same song to play indefinitely
 */
    
    !jump EndSong;
    
    DM(v_CurrMeasure) = M0;
    rts;

MultiMeasureMod:
    AR = DM(v_CurrMeasure);   
    AY0 = SR0;       
    NONE = AR - AY0;         
    if LT jump EndMMM;
    AY0 = SR1;
    NONE = AR - AY0;
    if GT jump EndMMM;
    call (I5);
EndMMM:
    rts;

Measure0:

/* SETTRACK(CLEAR, 0, 0,  a--- ---- ---- ----  a--- ---- ---- a---   ---- a--- ---- ----  a--- ---- ---- a---,
            CLEAR, 0, 5,  ---- ---- e--- ----  ---- ---- e--- ----   ---- ---- e--- ----  ---- ---- e--- ----,
            CLEAR, 0, 6,  ---- ---- e--- ----  ---- ---- e--- ----   ---- ---- e--- ----  ---- ---- e--- ----,
            CLEAR, 0, 14, m--- ---- ---- ----  ---- ---- ---- ----   m--- ---- ---- ----  ---- ---- ---- ----,
            a, 100, 0x3000,
            b, 50,  0x1000,
            e, 100, 0x2000,
            m, 100, 0x0700,
            END); */
    
    SET_VOLENV(T_BD0,  0x0000, 0x7fff, MEASURE*2);
    SET_VOLENV(T_HH1L, 0x0000, 0x7fff, MEASURE*2);
    SET_VOLENV(T_HH1R, 0x0000, 0x7fff, MEASURE*2);
    SET_VOLENV(T_FM0,  0x0000, 0x7fff, MEASURE*2);

    jump MultiMeasureCheck;

Measure4:

/* SETTRACK(NOCLEAR, 0, 0,  ---- --x- ---- ----  ---- ---- ---- ----   ---- ---- ---- ----  ---- --x- ---- ----,
            CLEAR,   0, 7,  f--- f-a- ---- f-f-  fif- f--- g--- --ff   f--- f--- ---- fif-  f-f- f--- g-aa abcd,
            CLEAR,   0, 8,  f--- f-a- ---- f-f-  fif- f--- g--- --ff   f--- f--- ---- fif-  f-f- f--- g-aa abcd,
            a, 100, 0x0400,
            b, 100, 0x0600,
            c, 100, 0x0800,
            d, 100, 0x0a00,
            f, 100, 0x1000,
            g, 100, 0x2c00,
            i, 100, 0x0100,
            x, 50,  0x1000,
            END); */

    SET_VOLENV(T_HH2L, 0x0000, 0x7fff, MEASURE*1);
    SET_VOLENV(T_HH2R, 0x0000, 0x7fff, MEASURE*1);    

    jump MultiMeasureCheck;

Measure8:

/* SETTRACK(CLEAR, 0, 1,  ---- ---- b--- ----  ---- c--- ---- ----   ---- --c- ---- ----  ---- ---- b--- ----,
            b, 100, 0x2000,
            c, 100, 0x1500,
            END); */

    SET_VOLENV(T_BD1, 0x0000, 0x7fff, MEASURE*2);

    jump MultiMeasureCheck;

Measure12:

/* SETTRACK(CLEAR, 0, 3,  --d- ---- d--- ----  d--- ---- d-d- d---   --d- ---- d--- ----  d--- ---- d-d- d---,
            CLEAR, 0, 4,  --d- ---- d--- ----  d--- ---- d-d- d---   --d- ---- d--- ----  d--- ---- d-d- d---,
            CLEAR, 0, 15, ---- ---- n--- ----  ---- ---- n--- ----   ---- ---- n--- ----  ---- ---- n--- ----,
            d, 100, 0x0100,
            m, 100, 0x0700,
            n, 100, 0x0400,
            o, 100, 0x0600,
            END); */

    SET_VOLENV(T_HH0L, 0x0000, 0x7fff, MEASURE*4);
    SET_VOLENV(T_HH0R, 0x0000, 0x7fff, MEASURE*4);
    SET_VOLENV(T_FM1, 0x0000, 0x7fff, MEASURE*4);

    jump MultiMeasureCheck;

Measure20:

/* SETTRACK(NOCLEAR, 0, 5,  ---- ---- ---- ----  ---- ---- ---- --h-   ---- ---- ---- ----  ---- ---- ---- --f-,
            NOCLEAR, 0, 6,  ---- ---- ---- ----  ---- ---- ---- --h-   ---- ---- ---- ----  ---- ---- ---- --f-,
            CLEAR, 0, 9,    g--- ---- ---- ----  ---- --g- ---- ----   g--- ---- ---- ----  ---- --g- ---- ----,
            f, 100, 0x0800,
            h, 100, 0x0400,
            g, 100, 0x0a00,
            END); */

    SET_VOLENV(T_BASS0, 0x0000, 0x7fff, MEASURE*8);

    jump MultiMeasureCheck;

Measure28:

/* SETTRACK(CLEAR, 0, 12, k--- ---- ---- ----  ---- ---- ---- ----   k--- ---- ---- ----  ---- ---- ---- ----,
            CLEAR, 0, 13, l--- ---- ---- ----  ---- ---- ---- ----   l--- ---- ---- ----  ---- ---- ---- ----,
            k, 100, 0x0e00,
            l, 100, 0x0900,
            END); */

    SET_VOLENV(T_PAD0L, 0x2000, 0x7fff, MEASURE*4);
    SET_VOLENV(T_PAD0R, 0x2000, 0x7fff, MEASURE*4);

    jump MultiMeasureCheck;

Measure32:
    MUTETRACK(T_BD0);
    MUTETRACK(T_BD1);
    MUTETRACK(T_HH0L);
    MUTETRACK(T_HH0R);
    MUTETRACK(T_HH2L);
    MUTETRACK(T_HH2R);
    
/* SETTRACK(CLEAR, 0, 10, h-i- i--- j--- h---  i--- i-j- h-i- --j-   h--- i--- j--- h-i-  i--- i--- h--- j-j-,
            h, 100, 0x1400,
            i, 100, 0x0c00,
            j, 100, 0x0800,
            END); */

    SET_VOLENV(T_BASS0, 0x0000, 0x4000, MEASURE*16);
    SET_VOLENV(T_LEAD0, 0x2000, 0x7fff, MEASURE*8);
    
    jump MultiMeasureCheck;

Measure34:

/* SETTRACK(CLEAR, 0, 11, h-i- i--- j-i- h-i-  i--- i-j- h-i- j-j-   h-i- i--- j-i- h-i-  i--- i-j- h-i- j-j-,
            h, 100, 0x1400,
            i, 100, 0x0c00,
            j, 100, 0x0800,
            END); */

    SET_VOLENV(T_LEAD1, 0x2000, 0x7fff, MEASURE*8);

    AR = 80;
    DM(v_KeepLeadFreq0) = AR;
    AR = 15;
    DM(v_IncLeadFreq0) = AR;
    AR = 5;
    DM(v_DecLeadFreq0) = AR;

    jump MultiMeasureCheck;

Measure36:
    AR = 80;
    DM(v_KeepLeadFreq1) = AR;
    AR = 15;
    DM(v_IncLeadFreq1) = AR;
    AR = 5;
    DM(v_DecLeadFreq1) = AR;
    
    jump MultiMeasureCheck;

Measure38:

/* SETTRACK(CLEAR, 0, 12, k--- ---- ---- ----  ---- --k- ---- ----   k--- ---- ---- ----  ---- --k- ---- ----,
            CLEAR, 0, 13, l--- ---- ---- ----  ---- --l- ---- ----   l--- ---- ---- ----  ---- --l- ---- ----,
            k, 100, 0x0e00,
            l, 100, 0x0900,
            END); */

    AR = 0x0003;
    DM(v_PadMask) = AR;
    
    jump MultiMeasureCheck;

Measure40:

/* SETTRACK(CLEAR, 0, 2,  ---- c--- ---- ----  ---- c--- ---- --c-   ---- ---- ---- c---  ---- c--- ---- ----,
            c, 100, 0x0600,
            END); */

    SET_VOLENV(T_BD0, 0x0000, 0x7fff, MEASURE*4);
    SET_VOLENV(T_BD1, 0x0000, 0x7fff, MEASURE*4);
    
    jump MultiMeasureCheck;

Measure41:
    SET_VOLENV(T_HH2L, 0x0000, 0x7fff, MEASURE*4);
    SET_VOLENV(T_HH2R, 0x0000, 0x7fff, MEASURE*4);
    
    jump MultiMeasureCheck;

Measure42:
/* SETTRACK(CLEAR, 0, 3,  --d- ---- d--- ----  d--- ---- dede dfef   --d- ---- d--- ----  dfef d--- --de d-f-,
            CLEAR, 0, 4,  --d- ---- d--- ----  d--- ---- dede dfef   --d- ---- d--- ----  dfef d--- --de d-f-,
            d, 100, 0x0100,
            e, 100, 0x0040,
            f, 100, 0x0080,
            END); */

    SET_VOLENV(T_HH0L, 0x0000, 0x7fff, MEASURE*2);
    SET_VOLENV(T_HH0R, 0x0000, 0x7fff, MEASURE*2);

    jump MultiMeasureCheck;

Measure43:
/* SETTRACK(CLEAR, 0, 14, o--- ---- --m- ----  m--- ---- m--- ----   o--- ---- --m- ----  m--- ---- ---- m---,
            CLEAR, 0, 15, ---- n--- ---- p---  ---- n--- --n- ----   ---- n--- ---- n---  ---- p--- --n- ----,
            m, 100, 0x0600,
            n, 100, 0x0400,
            o, 100, 0x0a00,
            p, 100, 0x0800,
            END); */

    SET_VOLENV(T_FM0, 0x0000, 0x7fff, MEASURE*4);
    SET_VOLENV(T_FM1, 0x0000, 0x7fff, MEASURE*4);

    /* modify panning */
    RESET_MEMENV3(5);
    RESET_MEMENV3(6);
    
    AY0 = 0x1000;
    DM(a_MemEnv3+(5*MEMENV3_VARS)+MEMENV3_EA) = AY0;
    DM(a_MemEnv3+(5*MEMENV3_VARS)+MEMENV3_EA) = AY0;
    
    DM(a_MemEnv3+(6*MEMENV3_VARS)+MEMENV3_ED) = AY0;
    DM(a_MemEnv3+(6*MEMENV3_VARS)+MEMENV3_ED) = AY0;

    jump MultiMeasureCheck;

Measure44:
    SET_VOLENV(T_BASS0, 0x0000, 0x4000, MEASURE*8);
  
    jump MultiMeasureCheck;

Measure48:
    FULLTRACK(T_BASS0);

    jump MultiMeasureCheck;

Measure56:

/* SETTRACK(CLEAR, 0, 0,  a--- ---- ---- ----  a--- ---- ---- a---   ---- ---- a--- ----  a--- ---- ---- a---,
            CLEAR, 0, 1,  ---- ---- b--- ----  ---- ---- b--- ----   b--- ---- ---- ----  ---- ---- b--- ----,
            a, 100, 0x3000,
            b, 100, 0x2000,
            END); */

    AR = 64;
    DM(a_Seq+1+(T_BD1*SEQ_VARS)+SEQ_TRIGRATE) = AR;

    jump MultiMeasureCheck;

Measure60:

/* SETTRACK(NOCLEAR, 0, 5,  ---- ---- ---- ----  --f- ---- ---- ----   ---- ---- ---- --g-  ---- ---- ---- ----,
            NOCLEAR, 0, 6,  ---- ---- ---- ----  --f- ---- ---- ----   ---- ---- ---- --g-  ---- ---- ---- ----,
            f, 100, 0x0400,
            g, 100, 0x0600,
            END); */

    jump MultiMeasureCheck;

Measure64:
    SET_VOLENV(T_BD0, 0x7fff, 0, MEASURE*4);
    SET_VOLENV(T_BD1, 0x7fff, 0, MEASURE*4);
    
    jump MultiMeasureCheck;

Measure66:
    SET_VOLENV(T_RHYTHMSAW0, 0x7fff, 0x0000, MEASURE*8);
    SET_VOLENV(T_HH2L,       0x7fff, 0x0000, MEASURE*8);
    SET_VOLENV(T_HH2R,       0x7fff, 0x0000, MEASURE*8);

    jump MultiMeasureCheck;

Measure68:
    SET_VOLENV(T_BASS0, 0x7fff, 0x0000, MEASURE*8);
    
    jump MultiMeasureCheck;

Measure76:
    SET_VOLENV(T_LEAD0, 0x7fff, 0x0000, MEASURE*4);
    SET_VOLENV(T_LEAD1, 0x7fff, 0x0000, MEASURE*4);

    jump MultiMeasureCheck;


Measure80:
/* SETTRACK(CLEAR, 0, 0,  a--- --c- ---- --c-  ---- c--- ---- a---   ---- a--- --c- ----  a--- ---- a--- c---,
            CLEAR, 0, 1,  ---- ---- b--- ----  ---- ---- b--- ----   ---- ---- b--- ----  ---- b--- ---- ----,
            a, 100, 0x3000,
            b, 100, 0x2000,
            c, 100, 0x1000,
            END); */

    FULLTRACK(T_BD0);
    FULLTRACK(T_BD1);

    jump MultiMeasureCheck;

Measure82:
/* SETTRACK(CLEAR, 0, 7,  g-fa fa-- bb-b fc-c  g-f- faf- --f- f-ff   g-f- fff- f--- f---  g-ff a-fa fb-- faf-,
            CLEAR, 0, 8,  g-fa fa-- bb-b fc-c  g-f- faf- --f- f-ff   g-f- fff- f--- f---  g-ff a-fa fb-- faf-,
            f, 100, 0x1c00,
            a, 100, 0x0200,
            b, 100, 0x0300,
            c, 100, 0x400,
            g, 100, 0x3000,
            END); */

    FULLTRACK(T_HH2L);
    FULLTRACK(T_HH2R);

    jump MultiMeasureCheck;

Measure84:
    MUTETRACK(T_BD0);
    MUTETRACK(T_BD1);
    MUTETRACK(T_HH1L);
    MUTETRACK(T_HH1R);
    MUTETRACK(T_HH2L);
    MUTETRACK(T_HH2R);
    
    jump MultiMeasureCheck;

Measure88:

/* SETTRACK(CLEAR, 0, 12, k--- ---- ---- ----  ---- ---- ---- ----   ---- ---- ---- ----  ---- ---- ---- ----,
            CLEAR, 0, 13, l--- ---- ---- ----  ---- ---- ---- ----   ---- ---- ---- ----  ---- ---- ---- ----,
            k, 100, 0x0e00,
            l, 100, 0x0900,
            END); */

    MUTETRACK(T_HH0L);
    MUTETRACK(T_HH0R);
    MUTETRACK(T_FM0);
    MUTETRACK(T_FM1);
  
    jump MultiMeasureCheck;

Measure89:
    MUTETRACK(T_PAD0L);
    MUTETRACK(T_PAD0R);
    
    jump MultiMeasureCheck;
    

DoNothing:
    jump MultiMeasureCheck;

.ENDMOD;
