/******************************************************************************
*
*  SongCTRL.dsp
*
*  (c) 2001, 2002, 2003 ethan bordeaux
*  syndevkit@dspmusic.org
*
*  function for modifying parameters on a measure by measure basis.  execution
*  rate determined by v_TicsPerMeasure (set in GenFXIni.dsp by macros).
*
******************************************************************************/

.MODULE                                     modSongCTRL_00;

/*
 *  include files for all functions
 */


#include    ".\comsys\hdr\DSPdefs.h"
#include    ".\gen_fx\hdr\GenFX.h"
#include    "exp3_3.h"

/*
 *  external variables and functions
 */

/*
 *  entry points
 */

.ENTRY      SongCTRL_00;
.ENTRY      Measure_JT;
.ENTRY      EndMeasure_JT;

SongCTRL_00:
    AR = DM(v_TicCNTR);
    AR = AR + 1;
    DM(v_TicCNTR) = AR;
    AY0 = DM(v_TicsPerMeasure);
    AR = AR - AY0;
    if LT rts;
    
    DM(v_TicCNTR) = M0;

/*
 *  note!  this initialization used to be in InitFunc.dsp.  however, to make
 *  this function work with multiple tracks in a single executable, Measure_JT
 *  and EndMeasure_JT would need to be abstracted and inited in SongPtrs.dsp.
 *  to try and simplify and reduce the number of initializations that occur in
 *  that function, these variables were made local and it is continually
 *  re-inited here.  note that this code runs in the krate loop so performance
 *  is not a concern.
 */

SetMeasuresPerSong:
    AX0 = ^EndMeasure_JT;
    AY0 = ^Measure_JT;
    AR = AX0 - AY0;
    DM(v_MeasuresPerSong) = AR;
    
    I5 = ^Measure_JT;
    M5 = DM(v_CurrMeasure);
    modify(I5, M5);
    jump (I5);
    
/*
 *  insert JT of operations to perform at the start of every new measure
 */
    
Measure_JT:
    jump Measure0;                              ! 1st bd, snare, 1st fm
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    
    jump Measure4;                              ! hihat, more bd
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
   
    jump Measure8;                              ! 2nd bd
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    
    jump Measure12;                             ! revnoise and 2nd fm
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    
    jump Measure20;                             ! bass, 8 measure filter mod
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;

    jump Measure28;                             ! pad
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    
    ! breakdown, remove lots of stuff, bring in 2 leads, change lead freq
    jump Measure32;
    jump DoNothing;

    jump Measure34;                             ! 2nd lead, 1st freq
    jump DoNothing;
    
    jump Measure36;                             ! 2nd lead freq
    jump DoNothing;
    jump Measure38;                             ! more pad, mod pad freq
    jump DoNothing;
    
    jump Measure40;                             ! add rhythmsaw
    jump Measure41;                             ! add bd
    jump Measure42;                             ! add hh
    jump Measure43;                             ! new fm patterm
    
    jump Measure44;                             ! kill, fade in bass
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    
    jump Measure48;                             ! nothing, everything in
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;

    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    
    jump Measure56;                             ! new bd pattern
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    
    jump Measure60;                             ! new snare pattern
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    
    jump Measure64;                             ! fade bd
    jump DoNothing;
    jump Measure66;                             ! fade hh, rhythmsaw
    jump DoNothing;
    
    jump Measure68;                             ! bass
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    
    jump Measure76;                             ! fade leads
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    
    jump DoNothing;
    jump DoNothing;
    
    jump Measure80;                             ! bd one more time
    jump DoNothing;
    jump DoNothing;
    
    jump Measure82;                             ! hh one more time
    jump DoNothing;
    jump DoNothing;
    
    jump Measure84;                             ! mute all but noise, pad
    jump DoNothing;
    jump DoNothing;
    jump DoNothing;
    
    jump Measure88;                             ! kill all but pad
    jump Measure89;                             ! kill pad
    jump DoNothing;
    jump DoNothing;
    
EndMeasure_JT:

/*
 *  insert all multimeasure operations - for parameters that should be modified
 *  over multiple measures
 */

MultiMeasureCheck:

EndSongCheck:
    AR = DM(v_CurrMeasure);
    AR = AR + 1;
    DM(v_CurrMeasure) = AR;
    AY0 = DM(v_MeasuresPerSong);
    AR = AR - AY0;
    if LT rts;

/*
 *  SongCTRL list is complete.  jumping to EndSong will start the next song
 *  in a multisong environment.  resetting CurrMeasure starts the Measure_JT
 *  loop over again, causing the same song to play indefinitely
 */
    
    !jump EndSong;
    
    DM(v_CurrMeasure) = M0;
    rts;

MultiMeasureMod:
    AR = DM(v_CurrMeasure);   
    AY0 = SR0;       
    NONE = AR - AY0;         
    if LT jump EndMMM;
    AY0 = SR1;
    NONE = AR - AY0;
    if GT jump EndMMM;
    call (I5);
EndMMM:
    rts;

Measure0:

/*
 *  autogenerated code for SETTRACK macro
 */
    I2 = ^a_TrigTrack00;
    I3 = ^a_VolTrack00;
    CNTR = 128;
    do CL00236 until CE;
        DM(I2, M1) = 0;
CL00236:   DM(I3, M1) = 0;

    I2 = ^a_TrigTrack05;
    I3 = ^a_VolTrack05;
    CNTR = 128;
    do CL05236 until CE;
        DM(I2, M1) = 0;
CL05236:   DM(I3, M1) = 0;

    I2 = ^a_TrigTrack06;
    I3 = ^a_VolTrack06;
    CNTR = 128;
    do CL06236 until CE;
        DM(I2, M1) = 0;
CL06236:   DM(I3, M1) = 0;

    I2 = ^a_TrigTrack14;
    I3 = ^a_VolTrack14;
    CNTR = 128;
    do CL14236 until CE;
        DM(I2, M1) = 0;
CL14236:   DM(I3, M1) = 0;

    /* inits for a */
    AX0 = 100;
    AX1 = 0x3000;
    DM(a_TrigTrack00+0+0) = AX0;
    DM(a_VolTrack00+0+0) = AX1;
    DM(a_TrigTrack00+0+16) = AX0;
    DM(a_VolTrack00+0+16) = AX1;
    DM(a_TrigTrack00+0+28) = AX0;
    DM(a_VolTrack00+0+28) = AX1;
    DM(a_TrigTrack00+0+36) = AX0;
    DM(a_VolTrack00+0+36) = AX1;
    DM(a_TrigTrack00+0+48) = AX0;
    DM(a_VolTrack00+0+48) = AX1;
    DM(a_TrigTrack00+0+60) = AX0;
    DM(a_VolTrack00+0+60) = AX1;

    /* inits for b */
    AX0 = 50;
    AX1 = 0x1000;

    /* inits for e */
    AX0 = 100;
    AX1 = 0x2000;
    DM(a_TrigTrack05+0+8) = AX0;
    DM(a_VolTrack05+0+8) = AX1;
    DM(a_TrigTrack06+0+8) = AX0;
    DM(a_VolTrack06+0+8) = AX1;
    DM(a_TrigTrack05+0+24) = AX0;
    DM(a_VolTrack05+0+24) = AX1;
    DM(a_TrigTrack06+0+24) = AX0;
    DM(a_VolTrack06+0+24) = AX1;
    DM(a_TrigTrack05+0+40) = AX0;
    DM(a_VolTrack05+0+40) = AX1;
    DM(a_TrigTrack06+0+40) = AX0;
    DM(a_VolTrack06+0+40) = AX1;
    DM(a_TrigTrack05+0+56) = AX0;
    DM(a_VolTrack05+0+56) = AX1;
    DM(a_TrigTrack06+0+56) = AX0;
    DM(a_VolTrack06+0+56) = AX1;

    /* inits for m */
    AX0 = 100;
    AX1 = 0x0700;
    DM(a_TrigTrack14+0+0) = AX0;
    DM(a_VolTrack14+0+0) = AX1;
    DM(a_TrigTrack14+0+32) = AX0;
    DM(a_VolTrack14+0+32) = AX1;

/* donotparse(CLEAR, 0, 0,  a--- ---- ---- ----  a--- ---- ---- a---   ---- a--- ---- ----  a--- ---- ---- a---,
            CLEAR, 0, 5,  ---- ---- e--- ----  ---- ---- e--- ----   ---- ---- e--- ----  ---- ---- e--- ----,
            CLEAR, 0, 6,  ---- ---- e--- ----  ---- ---- e--- ----   ---- ---- e--- ----  ---- ---- e--- ----,
            CLEAR, 0, 14, m--- ---- ---- ----  ---- ---- ---- ----   m--- ---- ---- ----  ---- ---- ---- ----,
            a, 100, 0x3000,
            b, 50,  0x1000,
            e, 100, 0x2000,
            m, 100, 0x0700,
            END); */
    
    SET_VOLENV(T_BD0,  0x0000, 0x7fff, MEASURE*2);
    SET_VOLENV(T_HH1L, 0x0000, 0x7fff, MEASURE*2);
    SET_VOLENV(T_HH1R, 0x0000, 0x7fff, MEASURE*2);
    SET_VOLENV(T_FM0,  0x0000, 0x7fff, MEASURE*2);

    jump MultiMeasureCheck;

Measure4:

/*
 *  autogenerated code for SETTRACK macro
 */
    I2 = ^a_TrigTrack07;
    I3 = ^a_VolTrack07;
    CNTR = 128;
    do CL07255 until CE;
        DM(I2, M1) = 0;
CL07255:   DM(I3, M1) = 0;

    I2 = ^a_TrigTrack08;
    I3 = ^a_VolTrack08;
    CNTR = 128;
    do CL08255 until CE;
        DM(I2, M1) = 0;
CL08255:   DM(I3, M1) = 0;

    /* inits for a */
    AX0 = 100;
    AX1 = 0x0400;
    DM(a_TrigTrack07+0+6) = AX0;
    DM(a_VolTrack07+0+6) = AX1;
    DM(a_TrigTrack08+0+6) = AX0;
    DM(a_VolTrack08+0+6) = AX1;
    DM(a_TrigTrack07+0+58) = AX0;
    DM(a_VolTrack07+0+58) = AX1;
    DM(a_TrigTrack08+0+58) = AX0;
    DM(a_VolTrack08+0+58) = AX1;
    DM(a_TrigTrack07+0+59) = AX0;
    DM(a_VolTrack07+0+59) = AX1;
    DM(a_TrigTrack08+0+59) = AX0;
    DM(a_VolTrack08+0+59) = AX1;
    DM(a_TrigTrack07+0+60) = AX0;
    DM(a_VolTrack07+0+60) = AX1;
    DM(a_TrigTrack08+0+60) = AX0;
    DM(a_VolTrack08+0+60) = AX1;

    /* inits for b */
    AX0 = 100;
    AX1 = 0x0600;
    DM(a_TrigTrack07+0+61) = AX0;
    DM(a_VolTrack07+0+61) = AX1;
    DM(a_TrigTrack08+0+61) = AX0;
    DM(a_VolTrack08+0+61) = AX1;

    /* inits for c */
    AX0 = 100;
    AX1 = 0x0800;
    DM(a_TrigTrack07+0+62) = AX0;
    DM(a_VolTrack07+0+62) = AX1;
    DM(a_TrigTrack08+0+62) = AX0;
    DM(a_VolTrack08+0+62) = AX1;

    /* inits for d */
    AX0 = 100;
    AX1 = 0x0a00;
    DM(a_TrigTrack07+0+63) = AX0;
    DM(a_VolTrack07+0+63) = AX1;
    DM(a_TrigTrack08+0+63) = AX0;
    DM(a_VolTrack08+0+63) = AX1;

    /* inits for f */
    AX0 = 100;
    AX1 = 0x1000;
    DM(a_TrigTrack07+0+0) = AX0;
    DM(a_VolTrack07+0+0) = AX1;
    DM(a_TrigTrack08+0+0) = AX0;
    DM(a_VolTrack08+0+0) = AX1;
    DM(a_TrigTrack07+0+4) = AX0;
    DM(a_VolTrack07+0+4) = AX1;
    DM(a_TrigTrack08+0+4) = AX0;
    DM(a_VolTrack08+0+4) = AX1;
    DM(a_TrigTrack07+0+12) = AX0;
    DM(a_VolTrack07+0+12) = AX1;
    DM(a_TrigTrack08+0+12) = AX0;
    DM(a_VolTrack08+0+12) = AX1;
    DM(a_TrigTrack07+0+14) = AX0;
    DM(a_VolTrack07+0+14) = AX1;
    DM(a_TrigTrack08+0+14) = AX0;
    DM(a_VolTrack08+0+14) = AX1;
    DM(a_TrigTrack07+0+16) = AX0;
    DM(a_VolTrack07+0+16) = AX1;
    DM(a_TrigTrack08+0+16) = AX0;
    DM(a_VolTrack08+0+16) = AX1;
    DM(a_TrigTrack07+0+18) = AX0;
    DM(a_VolTrack07+0+18) = AX1;
    DM(a_TrigTrack08+0+18) = AX0;
    DM(a_VolTrack08+0+18) = AX1;
    DM(a_TrigTrack07+0+20) = AX0;
    DM(a_VolTrack07+0+20) = AX1;
    DM(a_TrigTrack08+0+20) = AX0;
    DM(a_VolTrack08+0+20) = AX1;
    DM(a_TrigTrack07+0+30) = AX0;
    DM(a_VolTrack07+0+30) = AX1;
    DM(a_TrigTrack08+0+30) = AX0;
    DM(a_VolTrack08+0+30) = AX1;
    DM(a_TrigTrack07+0+31) = AX0;
    DM(a_VolTrack07+0+31) = AX1;
    DM(a_TrigTrack08+0+31) = AX0;
    DM(a_VolTrack08+0+31) = AX1;
    DM(a_TrigTrack07+0+32) = AX0;
    DM(a_VolTrack07+0+32) = AX1;
    DM(a_TrigTrack08+0+32) = AX0;
    DM(a_VolTrack08+0+32) = AX1;
    DM(a_TrigTrack07+0+36) = AX0;
    DM(a_VolTrack07+0+36) = AX1;
    DM(a_TrigTrack08+0+36) = AX0;
    DM(a_VolTrack08+0+36) = AX1;
    DM(a_TrigTrack07+0+44) = AX0;
    DM(a_VolTrack07+0+44) = AX1;
    DM(a_TrigTrack08+0+44) = AX0;
    DM(a_VolTrack08+0+44) = AX1;
    DM(a_TrigTrack07+0+46) = AX0;
    DM(a_VolTrack07+0+46) = AX1;
    DM(a_TrigTrack08+0+46) = AX0;
    DM(a_VolTrack08+0+46) = AX1;
    DM(a_TrigTrack07+0+48) = AX0;
    DM(a_VolTrack07+0+48) = AX1;
    DM(a_TrigTrack08+0+48) = AX0;
    DM(a_VolTrack08+0+48) = AX1;
    DM(a_TrigTrack07+0+50) = AX0;
    DM(a_VolTrack07+0+50) = AX1;
    DM(a_TrigTrack08+0+50) = AX0;
    DM(a_VolTrack08+0+50) = AX1;
    DM(a_TrigTrack07+0+52) = AX0;
    DM(a_VolTrack07+0+52) = AX1;
    DM(a_TrigTrack08+0+52) = AX0;
    DM(a_VolTrack08+0+52) = AX1;

    /* inits for g */
    AX0 = 100;
    AX1 = 0x2c00;
    DM(a_TrigTrack07+0+24) = AX0;
    DM(a_VolTrack07+0+24) = AX1;
    DM(a_TrigTrack08+0+24) = AX0;
    DM(a_VolTrack08+0+24) = AX1;
    DM(a_TrigTrack07+0+56) = AX0;
    DM(a_VolTrack07+0+56) = AX1;
    DM(a_TrigTrack08+0+56) = AX0;
    DM(a_VolTrack08+0+56) = AX1;

    /* inits for i */
    AX0 = 100;
    AX1 = 0x0100;
    DM(a_TrigTrack07+0+17) = AX0;
    DM(a_VolTrack07+0+17) = AX1;
    DM(a_TrigTrack08+0+17) = AX0;
    DM(a_VolTrack08+0+17) = AX1;
    DM(a_TrigTrack07+0+45) = AX0;
    DM(a_VolTrack07+0+45) = AX1;
    DM(a_TrigTrack08+0+45) = AX0;
    DM(a_VolTrack08+0+45) = AX1;

    /* inits for x */
    AX0 = 50;
    AX1 = 0x1000;
    DM(a_TrigTrack00+0+6) = AX0;
    DM(a_VolTrack00+0+6) = AX1;
    DM(a_TrigTrack00+0+54) = AX0;
    DM(a_VolTrack00+0+54) = AX1;

/* donotparse(NOCLEAR, 0, 0,  ---- --x- ---- ----  ---- ---- ---- ----   ---- ---- ---- ----  ---- --x- ---- ----,
            CLEAR,   0, 7,  f--- f-a- ---- f-f-  fif- f--- g--- --ff   f--- f--- ---- fif-  f-f- f--- g-aa abcd,
            CLEAR,   0, 8,  f--- f-a- ---- f-f-  fif- f--- g--- --ff   f--- f--- ---- fif-  f-f- f--- g-aa abcd,
            a, 100, 0x0400,
            b, 100, 0x0600,
            c, 100, 0x0800,
            d, 100, 0x0a00,
            f, 100, 0x1000,
            g, 100, 0x2c00,
            i, 100, 0x0100,
            x, 50,  0x1000,
            END); */

    SET_VOLENV(T_HH2L, 0x0000, 0x7fff, MEASURE*1);
    SET_VOLENV(T_HH2R, 0x0000, 0x7fff, MEASURE*1);    

    jump MultiMeasureCheck;

Measure8:

/*
 *  autogenerated code for SETTRACK macro
 */
    I2 = ^a_TrigTrack01;
    I3 = ^a_VolTrack01;
    CNTR = 128;
    do CL01275 until CE;
        DM(I2, M1) = 0;
CL01275:   DM(I3, M1) = 0;

    /* inits for b */
    AX0 = 100;
    AX1 = 0x2000;
    DM(a_TrigTrack01+0+8) = AX0;
    DM(a_VolTrack01+0+8) = AX1;
    DM(a_TrigTrack01+0+56) = AX0;
    DM(a_VolTrack01+0+56) = AX1;

    /* inits for c */
    AX0 = 100;
    AX1 = 0x1500;
    DM(a_TrigTrack01+0+20) = AX0;
    DM(a_VolTrack01+0+20) = AX1;
    DM(a_TrigTrack01+0+38) = AX0;
    DM(a_VolTrack01+0+38) = AX1;

/* donotparse(CLEAR, 0, 1,  ---- ---- b--- ----  ---- c--- ---- ----   ---- --c- ---- ----  ---- ---- b--- ----,
            b, 100, 0x2000,
            c, 100, 0x1500,
            END); */

    SET_VOLENV(T_BD1, 0x0000, 0x7fff, MEASURE*2);

    jump MultiMeasureCheck;

Measure12:

/*
 *  autogenerated code for SETTRACK macro
 */
    I2 = ^a_TrigTrack03;
    I3 = ^a_VolTrack03;
    CNTR = 128;
    do CL03286 until CE;
        DM(I2, M1) = 0;
CL03286:   DM(I3, M1) = 0;

    I2 = ^a_TrigTrack04;
    I3 = ^a_VolTrack04;
    CNTR = 128;
    do CL04286 until CE;
        DM(I2, M1) = 0;
CL04286:   DM(I3, M1) = 0;

    I2 = ^a_TrigTrack15;
    I3 = ^a_VolTrack15;
    CNTR = 128;
    do CL15286 until CE;
        DM(I2, M1) = 0;
CL15286:   DM(I3, M1) = 0;

    /* inits for d */
    AX0 = 100;
    AX1 = 0x0100;
    DM(a_TrigTrack03+0+2) = AX0;
    DM(a_VolTrack03+0+2) = AX1;
    DM(a_TrigTrack04+0+2) = AX0;
    DM(a_VolTrack04+0+2) = AX1;
    DM(a_TrigTrack03+0+8) = AX0;
    DM(a_VolTrack03+0+8) = AX1;
    DM(a_TrigTrack04+0+8) = AX0;
    DM(a_VolTrack04+0+8) = AX1;
    DM(a_TrigTrack03+0+16) = AX0;
    DM(a_VolTrack03+0+16) = AX1;
    DM(a_TrigTrack04+0+16) = AX0;
    DM(a_VolTrack04+0+16) = AX1;
    DM(a_TrigTrack03+0+24) = AX0;
    DM(a_VolTrack03+0+24) = AX1;
    DM(a_TrigTrack04+0+24) = AX0;
    DM(a_VolTrack04+0+24) = AX1;
    DM(a_TrigTrack03+0+26) = AX0;
    DM(a_VolTrack03+0+26) = AX1;
    DM(a_TrigTrack04+0+26) = AX0;
    DM(a_VolTrack04+0+26) = AX1;
    DM(a_TrigTrack03+0+28) = AX0;
    DM(a_VolTrack03+0+28) = AX1;
    DM(a_TrigTrack04+0+28) = AX0;
    DM(a_VolTrack04+0+28) = AX1;
    DM(a_TrigTrack03+0+34) = AX0;
    DM(a_VolTrack03+0+34) = AX1;
    DM(a_TrigTrack04+0+34) = AX0;
    DM(a_VolTrack04+0+34) = AX1;
    DM(a_TrigTrack03+0+40) = AX0;
    DM(a_VolTrack03+0+40) = AX1;
    DM(a_TrigTrack04+0+40) = AX0;
    DM(a_VolTrack04+0+40) = AX1;
    DM(a_TrigTrack03+0+48) = AX0;
    DM(a_VolTrack03+0+48) = AX1;
    DM(a_TrigTrack04+0+48) = AX0;
    DM(a_VolTrack04+0+48) = AX1;
    DM(a_TrigTrack03+0+56) = AX0;
    DM(a_VolTrack03+0+56) = AX1;
    DM(a_TrigTrack04+0+56) = AX0;
    DM(a_VolTrack04+0+56) = AX1;
    DM(a_TrigTrack03+0+58) = AX0;
    DM(a_VolTrack03+0+58) = AX1;
    DM(a_TrigTrack04+0+58) = AX0;
    DM(a_VolTrack04+0+58) = AX1;
    DM(a_TrigTrack03+0+60) = AX0;
    DM(a_VolTrack03+0+60) = AX1;
    DM(a_TrigTrack04+0+60) = AX0;
    DM(a_VolTrack04+0+60) = AX1;

    /* inits for m */
    AX0 = 100;
    AX1 = 0x0700;

    /* inits for n */
    AX0 = 100;
    AX1 = 0x0400;
    DM(a_TrigTrack15+0+8) = AX0;
    DM(a_VolTrack15+0+8) = AX1;
    DM(a_TrigTrack15+0+24) = AX0;
    DM(a_VolTrack15+0+24) = AX1;
    DM(a_TrigTrack15+0+40) = AX0;
    DM(a_VolTrack15+0+40) = AX1;
    DM(a_TrigTrack15+0+56) = AX0;
    DM(a_VolTrack15+0+56) = AX1;

    /* inits for o */
    AX0 = 100;
    AX1 = 0x0600;

/* donotparse(CLEAR, 0, 3,  --d- ---- d--- ----  d--- ---- d-d- d---   --d- ---- d--- ----  d--- ---- d-d- d---,
            CLEAR, 0, 4,  --d- ---- d--- ----  d--- ---- d-d- d---   --d- ---- d--- ----  d--- ---- d-d- d---,
            CLEAR, 0, 15, ---- ---- n--- ----  ---- ---- n--- ----   ---- ---- n--- ----  ---- ---- n--- ----,
            d, 100, 0x0100,
            m, 100, 0x0700,
            n, 100, 0x0400,
            o, 100, 0x0600,
            END); */

    SET_VOLENV(T_HH0L, 0x0000, 0x7fff, MEASURE*4);
    SET_VOLENV(T_HH0R, 0x0000, 0x7fff, MEASURE*4);
    SET_VOLENV(T_FM1, 0x0000, 0x7fff, MEASURE*4);

    jump MultiMeasureCheck;

Measure20:

/*
 *  autogenerated code for SETTRACK macro
 */
    I2 = ^a_TrigTrack09;
    I3 = ^a_VolTrack09;
    CNTR = 128;
    do CL09303 until CE;
        DM(I2, M1) = 0;
CL09303:   DM(I3, M1) = 0;

    /* inits for f */
    AX0 = 100;
    AX1 = 0x0800;
    DM(a_TrigTrack05+0+62) = AX0;
    DM(a_VolTrack05+0+62) = AX1;
    DM(a_TrigTrack06+0+62) = AX0;
    DM(a_VolTrack06+0+62) = AX1;

    /* inits for h */
    AX0 = 100;
    AX1 = 0x0400;
    DM(a_TrigTrack05+0+30) = AX0;
    DM(a_VolTrack05+0+30) = AX1;
    DM(a_TrigTrack06+0+30) = AX0;
    DM(a_VolTrack06+0+30) = AX1;

    /* inits for g */
    AX0 = 100;
    AX1 = 0x0a00;
    DM(a_TrigTrack09+0+0) = AX0;
    DM(a_VolTrack09+0+0) = AX1;
    DM(a_TrigTrack09+0+22) = AX0;
    DM(a_VolTrack09+0+22) = AX1;
    DM(a_TrigTrack09+0+32) = AX0;
    DM(a_VolTrack09+0+32) = AX1;
    DM(a_TrigTrack09+0+54) = AX0;
    DM(a_VolTrack09+0+54) = AX1;

/* donotparse(NOCLEAR, 0, 5,  ---- ---- ---- ----  ---- ---- ---- --h-   ---- ---- ---- ----  ---- ---- ---- --f-,
            NOCLEAR, 0, 6,  ---- ---- ---- ----  ---- ---- ---- --h-   ---- ---- ---- ----  ---- ---- ---- --f-,
            CLEAR, 0, 9,    g--- ---- ---- ----  ---- --g- ---- ----   g--- ---- ---- ----  ---- --g- ---- ----,
            f, 100, 0x0800,
            h, 100, 0x0400,
            g, 100, 0x0a00,
            END); */

    SET_VOLENV(T_BASS0, 0x0000, 0x7fff, MEASURE*8);

    jump MultiMeasureCheck;

Measure28:

/*
 *  autogenerated code for SETTRACK macro
 */
    I2 = ^a_TrigTrack12;
    I3 = ^a_VolTrack12;
    CNTR = 128;
    do CL12317 until CE;
        DM(I2, M1) = 0;
CL12317:   DM(I3, M1) = 0;

    I2 = ^a_TrigTrack13;
    I3 = ^a_VolTrack13;
    CNTR = 128;
    do CL13317 until CE;
        DM(I2, M1) = 0;
CL13317:   DM(I3, M1) = 0;

    /* inits for k */
    AX0 = 100;
    AX1 = 0x0e00;
    DM(a_TrigTrack12+0+0) = AX0;
    DM(a_VolTrack12+0+0) = AX1;
    DM(a_TrigTrack12+0+32) = AX0;
    DM(a_VolTrack12+0+32) = AX1;

    /* inits for l */
    AX0 = 100;
    AX1 = 0x0900;
    DM(a_TrigTrack13+0+0) = AX0;
    DM(a_VolTrack13+0+0) = AX1;
    DM(a_TrigTrack13+0+32) = AX0;
    DM(a_VolTrack13+0+32) = AX1;

/* donotparse(CLEAR, 0, 12, k--- ---- ---- ----  ---- ---- ---- ----   k--- ---- ---- ----  ---- ---- ---- ----,
            CLEAR, 0, 13, l--- ---- ---- ----  ---- ---- ---- ----   l--- ---- ---- ----  ---- ---- ---- ----,
            k, 100, 0x0e00,
            l, 100, 0x0900,
            END); */

    SET_VOLENV(T_PAD0L, 0x2000, 0x7fff, MEASURE*4);
    SET_VOLENV(T_PAD0R, 0x2000, 0x7fff, MEASURE*4);

    jump MultiMeasureCheck;

Measure32:
    MUTETRACK(T_BD0);
    MUTETRACK(T_BD1);
    MUTETRACK(T_HH0L);
    MUTETRACK(T_HH0R);
    MUTETRACK(T_HH2L);
    MUTETRACK(T_HH2R);
    
/*
 *  autogenerated code for SETTRACK macro
 */
    I2 = ^a_TrigTrack10;
    I3 = ^a_VolTrack10;
    CNTR = 128;
    do CL10336 until CE;
        DM(I2, M1) = 0;
CL10336:   DM(I3, M1) = 0;

    /* inits for h */
    AX0 = 100;
    AX1 = 0x1400;
    DM(a_TrigTrack10+0+0) = AX0;
    DM(a_VolTrack10+0+0) = AX1;
    DM(a_TrigTrack10+0+12) = AX0;
    DM(a_VolTrack10+0+12) = AX1;
    DM(a_TrigTrack10+0+24) = AX0;
    DM(a_VolTrack10+0+24) = AX1;
    DM(a_TrigTrack10+0+32) = AX0;
    DM(a_VolTrack10+0+32) = AX1;
    DM(a_TrigTrack10+0+44) = AX0;
    DM(a_VolTrack10+0+44) = AX1;
    DM(a_TrigTrack10+0+56) = AX0;
    DM(a_VolTrack10+0+56) = AX1;

    /* inits for i */
    AX0 = 100;
    AX1 = 0x0c00;
    DM(a_TrigTrack10+0+2) = AX0;
    DM(a_VolTrack10+0+2) = AX1;
    DM(a_TrigTrack10+0+4) = AX0;
    DM(a_VolTrack10+0+4) = AX1;
    DM(a_TrigTrack10+0+16) = AX0;
    DM(a_VolTrack10+0+16) = AX1;
    DM(a_TrigTrack10+0+20) = AX0;
    DM(a_VolTrack10+0+20) = AX1;
    DM(a_TrigTrack10+0+26) = AX0;
    DM(a_VolTrack10+0+26) = AX1;
    DM(a_TrigTrack10+0+36) = AX0;
    DM(a_VolTrack10+0+36) = AX1;
    DM(a_TrigTrack10+0+46) = AX0;
    DM(a_VolTrack10+0+46) = AX1;
    DM(a_TrigTrack10+0+48) = AX0;
    DM(a_VolTrack10+0+48) = AX1;
    DM(a_TrigTrack10+0+52) = AX0;
    DM(a_VolTrack10+0+52) = AX1;

    /* inits for j */
    AX0 = 100;
    AX1 = 0x0800;
    DM(a_TrigTrack10+0+8) = AX0;
    DM(a_VolTrack10+0+8) = AX1;
    DM(a_TrigTrack10+0+22) = AX0;
    DM(a_VolTrack10+0+22) = AX1;
    DM(a_TrigTrack10+0+30) = AX0;
    DM(a_VolTrack10+0+30) = AX1;
    DM(a_TrigTrack10+0+40) = AX0;
    DM(a_VolTrack10+0+40) = AX1;
    DM(a_TrigTrack10+0+60) = AX0;
    DM(a_VolTrack10+0+60) = AX1;
    DM(a_TrigTrack10+0+62) = AX0;
    DM(a_VolTrack10+0+62) = AX1;

/* donotparse(CLEAR, 0, 10, h-i- i--- j--- h---  i--- i-j- h-i- --j-   h--- i--- j--- h-i-  i--- i--- h--- j-j-,
            h, 100, 0x1400,
            i, 100, 0x0c00,
            j, 100, 0x0800,
            END); */

    SET_VOLENV(T_BASS0, 0x0000, 0x4000, MEASURE*16);
    SET_VOLENV(T_LEAD0, 0x2000, 0x7fff, MEASURE*8);
    
    jump MultiMeasureCheck;

Measure34:

/*
 *  autogenerated code for SETTRACK macro
 */
    I2 = ^a_TrigTrack11;
    I3 = ^a_VolTrack11;
    CNTR = 128;
    do CL11349 until CE;
        DM(I2, M1) = 0;
CL11349:   DM(I3, M1) = 0;

    /* inits for h */
    AX0 = 100;
    AX1 = 0x1400;
    DM(a_TrigTrack11+0+0) = AX0;
    DM(a_VolTrack11+0+0) = AX1;
    DM(a_TrigTrack11+0+12) = AX0;
    DM(a_VolTrack11+0+12) = AX1;
    DM(a_TrigTrack11+0+24) = AX0;
    DM(a_VolTrack11+0+24) = AX1;
    DM(a_TrigTrack11+0+32) = AX0;
    DM(a_VolTrack11+0+32) = AX1;
    DM(a_TrigTrack11+0+44) = AX0;
    DM(a_VolTrack11+0+44) = AX1;
    DM(a_TrigTrack11+0+56) = AX0;
    DM(a_VolTrack11+0+56) = AX1;

    /* inits for i */
    AX0 = 100;
    AX1 = 0x0c00;
    DM(a_TrigTrack11+0+2) = AX0;
    DM(a_VolTrack11+0+2) = AX1;
    DM(a_TrigTrack11+0+4) = AX0;
    DM(a_VolTrack11+0+4) = AX1;
    DM(a_TrigTrack11+0+10) = AX0;
    DM(a_VolTrack11+0+10) = AX1;
    DM(a_TrigTrack11+0+14) = AX0;
    DM(a_VolTrack11+0+14) = AX1;
    DM(a_TrigTrack11+0+16) = AX0;
    DM(a_VolTrack11+0+16) = AX1;
    DM(a_TrigTrack11+0+20) = AX0;
    DM(a_VolTrack11+0+20) = AX1;
    DM(a_TrigTrack11+0+26) = AX0;
    DM(a_VolTrack11+0+26) = AX1;
    DM(a_TrigTrack11+0+34) = AX0;
    DM(a_VolTrack11+0+34) = AX1;
    DM(a_TrigTrack11+0+36) = AX0;
    DM(a_VolTrack11+0+36) = AX1;
    DM(a_TrigTrack11+0+42) = AX0;
    DM(a_VolTrack11+0+42) = AX1;
    DM(a_TrigTrack11+0+46) = AX0;
    DM(a_VolTrack11+0+46) = AX1;
    DM(a_TrigTrack11+0+48) = AX0;
    DM(a_VolTrack11+0+48) = AX1;
    DM(a_TrigTrack11+0+52) = AX0;
    DM(a_VolTrack11+0+52) = AX1;
    DM(a_TrigTrack11+0+58) = AX0;
    DM(a_VolTrack11+0+58) = AX1;

    /* inits for j */
    AX0 = 100;
    AX1 = 0x0800;
    DM(a_TrigTrack11+0+8) = AX0;
    DM(a_VolTrack11+0+8) = AX1;
    DM(a_TrigTrack11+0+22) = AX0;
    DM(a_VolTrack11+0+22) = AX1;
    DM(a_TrigTrack11+0+28) = AX0;
    DM(a_VolTrack11+0+28) = AX1;
    DM(a_TrigTrack11+0+30) = AX0;
    DM(a_VolTrack11+0+30) = AX1;
    DM(a_TrigTrack11+0+40) = AX0;
    DM(a_VolTrack11+0+40) = AX1;
    DM(a_TrigTrack11+0+54) = AX0;
    DM(a_VolTrack11+0+54) = AX1;
    DM(a_TrigTrack11+0+60) = AX0;
    DM(a_VolTrack11+0+60) = AX1;
    DM(a_TrigTrack11+0+62) = AX0;
    DM(a_VolTrack11+0+62) = AX1;

/* donotparse(CLEAR, 0, 11, h-i- i--- j-i- h-i-  i--- i-j- h-i- j-j-   h-i- i--- j-i- h-i-  i--- i-j- h-i- j-j-,
            h, 100, 0x1400,
            i, 100, 0x0c00,
            j, 100, 0x0800,
            END); */

    SET_VOLENV(T_LEAD1, 0x2000, 0x7fff, MEASURE*8);

    AR = 80;
    DM(v_KeepLeadFreq0) = AR;
    AR = 15;
    DM(v_IncLeadFreq0) = AR;
    AR = 5;
    DM(v_DecLeadFreq0) = AR;

    jump MultiMeasureCheck;

Measure36:
    AR = 80;
    DM(v_KeepLeadFreq1) = AR;
    AR = 15;
    DM(v_IncLeadFreq1) = AR;
    AR = 5;
    DM(v_DecLeadFreq1) = AR;
    
    jump MultiMeasureCheck;

Measure38:

/*
 *  autogenerated code for SETTRACK macro
 */
    I2 = ^a_TrigTrack12;
    I3 = ^a_VolTrack12;
    CNTR = 128;
    do CL12378 until CE;
        DM(I2, M1) = 0;
CL12378:   DM(I3, M1) = 0;

    I2 = ^a_TrigTrack13;
    I3 = ^a_VolTrack13;
    CNTR = 128;
    do CL13378 until CE;
        DM(I2, M1) = 0;
CL13378:   DM(I3, M1) = 0;

    /* inits for k */
    AX0 = 100;
    AX1 = 0x0e00;
    DM(a_TrigTrack12+0+0) = AX0;
    DM(a_VolTrack12+0+0) = AX1;
    DM(a_TrigTrack12+0+22) = AX0;
    DM(a_VolTrack12+0+22) = AX1;
    DM(a_TrigTrack12+0+32) = AX0;
    DM(a_VolTrack12+0+32) = AX1;
    DM(a_TrigTrack12+0+54) = AX0;
    DM(a_VolTrack12+0+54) = AX1;

    /* inits for l */
    AX0 = 100;
    AX1 = 0x0900;
    DM(a_TrigTrack13+0+0) = AX0;
    DM(a_VolTrack13+0+0) = AX1;
    DM(a_TrigTrack13+0+22) = AX0;
    DM(a_VolTrack13+0+22) = AX1;
    DM(a_TrigTrack13+0+32) = AX0;
    DM(a_VolTrack13+0+32) = AX1;
    DM(a_TrigTrack13+0+54) = AX0;
    DM(a_VolTrack13+0+54) = AX1;

/* donotparse(CLEAR, 0, 12, k--- ---- ---- ----  ---- --k- ---- ----   k--- ---- ---- ----  ---- --k- ---- ----,
            CLEAR, 0, 13, l--- ---- ---- ----  ---- --l- ---- ----   l--- ---- ---- ----  ---- --l- ---- ----,
            k, 100, 0x0e00,
            l, 100, 0x0900,
            END); */

    AR = 0x0003;
    DM(v_PadMask) = AR;
    
    jump MultiMeasureCheck;

Measure40:

/*
 *  autogenerated code for SETTRACK macro
 */
    I2 = ^a_TrigTrack02;
    I3 = ^a_VolTrack02;
    CNTR = 128;
    do CL02391 until CE;
        DM(I2, M1) = 0;
CL02391:   DM(I3, M1) = 0;

    /* inits for c */
    AX0 = 100;
    AX1 = 0x0600;
    DM(a_TrigTrack02+0+4) = AX0;
    DM(a_VolTrack02+0+4) = AX1;
    DM(a_TrigTrack02+0+20) = AX0;
    DM(a_VolTrack02+0+20) = AX1;
    DM(a_TrigTrack02+0+30) = AX0;
    DM(a_VolTrack02+0+30) = AX1;
    DM(a_TrigTrack02+0+44) = AX0;
    DM(a_VolTrack02+0+44) = AX1;
    DM(a_TrigTrack02+0+52) = AX0;
    DM(a_VolTrack02+0+52) = AX1;

/* donotparse(CLEAR, 0, 2,  ---- c--- ---- ----  ---- c--- ---- --c-   ---- ---- ---- c---  ---- c--- ---- ----,
            c, 100, 0x0600,
            END); */

    SET_VOLENV(T_BD0, 0x0000, 0x7fff, MEASURE*4);
    SET_VOLENV(T_BD1, 0x0000, 0x7fff, MEASURE*4);
    
    jump MultiMeasureCheck;

Measure41:
    SET_VOLENV(T_HH2L, 0x0000, 0x7fff, MEASURE*4);
    SET_VOLENV(T_HH2R, 0x0000, 0x7fff, MEASURE*4);
    
    jump MultiMeasureCheck;

Measure42:
/*
 *  autogenerated code for SETTRACK macro
 */
    I2 = ^a_TrigTrack03;
    I3 = ^a_VolTrack03;
    CNTR = 128;
    do CL03407 until CE;
        DM(I2, M1) = 0;
CL03407:   DM(I3, M1) = 0;

    I2 = ^a_TrigTrack04;
    I3 = ^a_VolTrack04;
    CNTR = 128;
    do CL04407 until CE;
        DM(I2, M1) = 0;
CL04407:   DM(I3, M1) = 0;

    /* inits for d */
    AX0 = 100;
    AX1 = 0x0100;
    DM(a_TrigTrack03+0+2) = AX0;
    DM(a_VolTrack03+0+2) = AX1;
    DM(a_TrigTrack04+0+2) = AX0;
    DM(a_VolTrack04+0+2) = AX1;
    DM(a_TrigTrack03+0+8) = AX0;
    DM(a_VolTrack03+0+8) = AX1;
    DM(a_TrigTrack04+0+8) = AX0;
    DM(a_VolTrack04+0+8) = AX1;
    DM(a_TrigTrack03+0+16) = AX0;
    DM(a_VolTrack03+0+16) = AX1;
    DM(a_TrigTrack04+0+16) = AX0;
    DM(a_VolTrack04+0+16) = AX1;
    DM(a_TrigTrack03+0+24) = AX0;
    DM(a_VolTrack03+0+24) = AX1;
    DM(a_TrigTrack04+0+24) = AX0;
    DM(a_VolTrack04+0+24) = AX1;
    DM(a_TrigTrack03+0+26) = AX0;
    DM(a_VolTrack03+0+26) = AX1;
    DM(a_TrigTrack04+0+26) = AX0;
    DM(a_VolTrack04+0+26) = AX1;
    DM(a_TrigTrack03+0+28) = AX0;
    DM(a_VolTrack03+0+28) = AX1;
    DM(a_TrigTrack04+0+28) = AX0;
    DM(a_VolTrack04+0+28) = AX1;
    DM(a_TrigTrack03+0+34) = AX0;
    DM(a_VolTrack03+0+34) = AX1;
    DM(a_TrigTrack04+0+34) = AX0;
    DM(a_VolTrack04+0+34) = AX1;
    DM(a_TrigTrack03+0+40) = AX0;
    DM(a_VolTrack03+0+40) = AX1;
    DM(a_TrigTrack04+0+40) = AX0;
    DM(a_VolTrack04+0+40) = AX1;
    DM(a_TrigTrack03+0+48) = AX0;
    DM(a_VolTrack03+0+48) = AX1;
    DM(a_TrigTrack04+0+48) = AX0;
    DM(a_VolTrack04+0+48) = AX1;
    DM(a_TrigTrack03+0+52) = AX0;
    DM(a_VolTrack03+0+52) = AX1;
    DM(a_TrigTrack04+0+52) = AX0;
    DM(a_VolTrack04+0+52) = AX1;
    DM(a_TrigTrack03+0+58) = AX0;
    DM(a_VolTrack03+0+58) = AX1;
    DM(a_TrigTrack04+0+58) = AX0;
    DM(a_VolTrack04+0+58) = AX1;
    DM(a_TrigTrack03+0+60) = AX0;
    DM(a_VolTrack03+0+60) = AX1;
    DM(a_TrigTrack04+0+60) = AX0;
    DM(a_VolTrack04+0+60) = AX1;

    /* inits for e */
    AX0 = 100;
    AX1 = 0x0040;
    DM(a_TrigTrack03+0+25) = AX0;
    DM(a_VolTrack03+0+25) = AX1;
    DM(a_TrigTrack04+0+25) = AX0;
    DM(a_VolTrack04+0+25) = AX1;
    DM(a_TrigTrack03+0+27) = AX0;
    DM(a_VolTrack03+0+27) = AX1;
    DM(a_TrigTrack04+0+27) = AX0;
    DM(a_VolTrack04+0+27) = AX1;
    DM(a_TrigTrack03+0+30) = AX0;
    DM(a_VolTrack03+0+30) = AX1;
    DM(a_TrigTrack04+0+30) = AX0;
    DM(a_VolTrack04+0+30) = AX1;
    DM(a_TrigTrack03+0+50) = AX0;
    DM(a_VolTrack03+0+50) = AX1;
    DM(a_TrigTrack04+0+50) = AX0;
    DM(a_VolTrack04+0+50) = AX1;
    DM(a_TrigTrack03+0+59) = AX0;
    DM(a_VolTrack03+0+59) = AX1;
    DM(a_TrigTrack04+0+59) = AX0;
    DM(a_VolTrack04+0+59) = AX1;

    /* inits for f */
    AX0 = 100;
    AX1 = 0x0080;
    DM(a_TrigTrack03+0+29) = AX0;
    DM(a_VolTrack03+0+29) = AX1;
    DM(a_TrigTrack04+0+29) = AX0;
    DM(a_VolTrack04+0+29) = AX1;
    DM(a_TrigTrack03+0+31) = AX0;
    DM(a_VolTrack03+0+31) = AX1;
    DM(a_TrigTrack04+0+31) = AX0;
    DM(a_VolTrack04+0+31) = AX1;
    DM(a_TrigTrack03+0+49) = AX0;
    DM(a_VolTrack03+0+49) = AX1;
    DM(a_TrigTrack04+0+49) = AX0;
    DM(a_VolTrack04+0+49) = AX1;
    DM(a_TrigTrack03+0+51) = AX0;
    DM(a_VolTrack03+0+51) = AX1;
    DM(a_TrigTrack04+0+51) = AX0;
    DM(a_VolTrack04+0+51) = AX1;
    DM(a_TrigTrack03+0+62) = AX0;
    DM(a_VolTrack03+0+62) = AX1;
    DM(a_TrigTrack04+0+62) = AX0;
    DM(a_VolTrack04+0+62) = AX1;

/* donotparse(CLEAR, 0, 3,  --d- ---- d--- ----  d--- ---- dede dfef   --d- ---- d--- ----  dfef d--- --de d-f-,
            CLEAR, 0, 4,  --d- ---- d--- ----  d--- ---- dede dfef   --d- ---- d--- ----  dfef d--- --de d-f-,
            d, 100, 0x0100,
            e, 100, 0x0040,
            f, 100, 0x0080,
            END); */

    SET_VOLENV(T_HH0L, 0x0000, 0x7fff, MEASURE*2);
    SET_VOLENV(T_HH0R, 0x0000, 0x7fff, MEASURE*2);

    jump MultiMeasureCheck;

Measure43:
/*
 *  autogenerated code for SETTRACK macro
 */
    I2 = ^a_TrigTrack14;
    I3 = ^a_VolTrack14;
    CNTR = 128;
    do CL14420 until CE;
        DM(I2, M1) = 0;
CL14420:   DM(I3, M1) = 0;

    I2 = ^a_TrigTrack15;
    I3 = ^a_VolTrack15;
    CNTR = 128;
    do CL15420 until CE;
        DM(I2, M1) = 0;
CL15420:   DM(I3, M1) = 0;

    /* inits for m */
    AX0 = 100;
    AX1 = 0x0600;
    DM(a_TrigTrack14+0+10) = AX0;
    DM(a_VolTrack14+0+10) = AX1;
    DM(a_TrigTrack14+0+16) = AX0;
    DM(a_VolTrack14+0+16) = AX1;
    DM(a_TrigTrack14+0+24) = AX0;
    DM(a_VolTrack14+0+24) = AX1;
    DM(a_TrigTrack14+0+42) = AX0;
    DM(a_VolTrack14+0+42) = AX1;
    DM(a_TrigTrack14+0+48) = AX0;
    DM(a_VolTrack14+0+48) = AX1;
    DM(a_TrigTrack14+0+60) = AX0;
    DM(a_VolTrack14+0+60) = AX1;

    /* inits for n */
    AX0 = 100;
    AX1 = 0x0400;
    DM(a_TrigTrack15+0+4) = AX0;
    DM(a_VolTrack15+0+4) = AX1;
    DM(a_TrigTrack15+0+20) = AX0;
    DM(a_VolTrack15+0+20) = AX1;
    DM(a_TrigTrack15+0+26) = AX0;
    DM(a_VolTrack15+0+26) = AX1;
    DM(a_TrigTrack15+0+36) = AX0;
    DM(a_VolTrack15+0+36) = AX1;
    DM(a_TrigTrack15+0+44) = AX0;
    DM(a_VolTrack15+0+44) = AX1;
    DM(a_TrigTrack15+0+58) = AX0;
    DM(a_VolTrack15+0+58) = AX1;

    /* inits for o */
    AX0 = 100;
    AX1 = 0x0a00;
    DM(a_TrigTrack14+0+0) = AX0;
    DM(a_VolTrack14+0+0) = AX1;
    DM(a_TrigTrack14+0+32) = AX0;
    DM(a_VolTrack14+0+32) = AX1;

    /* inits for p */
    AX0 = 100;
    AX1 = 0x0800;
    DM(a_TrigTrack15+0+12) = AX0;
    DM(a_VolTrack15+0+12) = AX1;
    DM(a_TrigTrack15+0+52) = AX0;
    DM(a_VolTrack15+0+52) = AX1;

/* donotparse(CLEAR, 0, 14, o--- ---- --m- ----  m--- ---- m--- ----   o--- ---- --m- ----  m--- ---- ---- m---,
            CLEAR, 0, 15, ---- n--- ---- p---  ---- n--- --n- ----   ---- n--- ---- n---  ---- p--- --n- ----,
            m, 100, 0x0600,
            n, 100, 0x0400,
            o, 100, 0x0a00,
            p, 100, 0x0800,
            END); */

    SET_VOLENV(T_FM0, 0x0000, 0x7fff, MEASURE*4);
    SET_VOLENV(T_FM1, 0x0000, 0x7fff, MEASURE*4);

    /* modify panning */
    RESET_MEMENV3(5);
    RESET_MEMENV3(6);
    
    AY0 = 0x1000;
    DM(a_MemEnv3+(5*MEMENV3_VARS)+MEMENV3_EA) = AY0;
    DM(a_MemEnv3+(5*MEMENV3_VARS)+MEMENV3_EA) = AY0;
    
    DM(a_MemEnv3+(6*MEMENV3_VARS)+MEMENV3_ED) = AY0;
    DM(a_MemEnv3+(6*MEMENV3_VARS)+MEMENV3_ED) = AY0;

    jump MultiMeasureCheck;

Measure44:
    SET_VOLENV(T_BASS0, 0x0000, 0x4000, MEASURE*8);
  
    jump MultiMeasureCheck;

Measure48:
    FULLTRACK(T_BASS0);

    jump MultiMeasureCheck;

Measure56:

/*
 *  autogenerated code for SETTRACK macro
 */
    I2 = ^a_TrigTrack00;
    I3 = ^a_VolTrack00;
    CNTR = 128;
    do CL00456 until CE;
        DM(I2, M1) = 0;
CL00456:   DM(I3, M1) = 0;

    I2 = ^a_TrigTrack01;
    I3 = ^a_VolTrack01;
    CNTR = 128;
    do CL01456 until CE;
        DM(I2, M1) = 0;
CL01456:   DM(I3, M1) = 0;

    /* inits for a */
    AX0 = 100;
    AX1 = 0x3000;
    DM(a_TrigTrack00+0+0) = AX0;
    DM(a_VolTrack00+0+0) = AX1;
    DM(a_TrigTrack00+0+16) = AX0;
    DM(a_VolTrack00+0+16) = AX1;
    DM(a_TrigTrack00+0+28) = AX0;
    DM(a_VolTrack00+0+28) = AX1;
    DM(a_TrigTrack00+0+40) = AX0;
    DM(a_VolTrack00+0+40) = AX1;
    DM(a_TrigTrack00+0+48) = AX0;
    DM(a_VolTrack00+0+48) = AX1;
    DM(a_TrigTrack00+0+60) = AX0;
    DM(a_VolTrack00+0+60) = AX1;

    /* inits for b */
    AX0 = 100;
    AX1 = 0x2000;
    DM(a_TrigTrack01+0+8) = AX0;
    DM(a_VolTrack01+0+8) = AX1;
    DM(a_TrigTrack01+0+24) = AX0;
    DM(a_VolTrack01+0+24) = AX1;
    DM(a_TrigTrack01+0+32) = AX0;
    DM(a_VolTrack01+0+32) = AX1;
    DM(a_TrigTrack01+0+56) = AX0;
    DM(a_VolTrack01+0+56) = AX1;

/* donotparse(CLEAR, 0, 0,  a--- ---- ---- ----  a--- ---- ---- a---   ---- ---- a--- ----  a--- ---- ---- a---,
            CLEAR, 0, 1,  ---- ---- b--- ----  ---- ---- b--- ----   b--- ---- ---- ----  ---- ---- b--- ----,
            a, 100, 0x3000,
            b, 100, 0x2000,
            END); */

    AR = 64;
    DM(a_Seq+1+(T_BD1*SEQ_VARS)+SEQ_TRIGRATE) = AR;

    jump MultiMeasureCheck;

Measure60:

/*
 *  autogenerated code for SETTRACK macro
 */
    /* inits for f */
    AX0 = 100;
    AX1 = 0x0400;
    DM(a_TrigTrack05+0+18) = AX0;
    DM(a_VolTrack05+0+18) = AX1;
    DM(a_TrigTrack06+0+18) = AX0;
    DM(a_VolTrack06+0+18) = AX1;

    /* inits for g */
    AX0 = 100;
    AX1 = 0x0600;
    DM(a_TrigTrack05+0+46) = AX0;
    DM(a_VolTrack05+0+46) = AX1;
    DM(a_TrigTrack06+0+46) = AX0;
    DM(a_VolTrack06+0+46) = AX1;

/* donotparse(NOCLEAR, 0, 5,  ---- ---- ---- ----  --f- ---- ---- ----   ---- ---- ---- --g-  ---- ---- ---- ----,
            NOCLEAR, 0, 6,  ---- ---- ---- ----  --f- ---- ---- ----   ---- ---- ---- --g-  ---- ---- ---- ----,
            f, 100, 0x0400,
            g, 100, 0x0600,
            END); */

    jump MultiMeasureCheck;

Measure64:
    SET_VOLENV(T_BD0, 0x7fff, 0, MEASURE*4);
    SET_VOLENV(T_BD1, 0x7fff, 0, MEASURE*4);
    
    jump MultiMeasureCheck;

Measure66:
    SET_VOLENV(T_RHYTHMSAW0, 0x7fff, 0x0000, MEASURE*8);
    SET_VOLENV(T_HH2L,       0x7fff, 0x0000, MEASURE*8);
    SET_VOLENV(T_HH2R,       0x7fff, 0x0000, MEASURE*8);

    jump MultiMeasureCheck;

Measure68:
    SET_VOLENV(T_BASS0, 0x7fff, 0x0000, MEASURE*8);
    
    jump MultiMeasureCheck;

Measure76:
    SET_VOLENV(T_LEAD0, 0x7fff, 0x0000, MEASURE*4);
    SET_VOLENV(T_LEAD1, 0x7fff, 0x0000, MEASURE*4);

    jump MultiMeasureCheck;


Measure80:
/*
 *  autogenerated code for SETTRACK macro
 */
    I2 = ^a_TrigTrack00;
    I3 = ^a_VolTrack00;
    CNTR = 128;
    do CL00503 until CE;
        DM(I2, M1) = 0;
CL00503:   DM(I3, M1) = 0;

    I2 = ^a_TrigTrack01;
    I3 = ^a_VolTrack01;
    CNTR = 128;
    do CL01503 until CE;
        DM(I2, M1) = 0;
CL01503:   DM(I3, M1) = 0;

    /* inits for a */
    AX0 = 100;
    AX1 = 0x3000;
    DM(a_TrigTrack00+0+0) = AX0;
    DM(a_VolTrack00+0+0) = AX1;
    DM(a_TrigTrack00+0+28) = AX0;
    DM(a_VolTrack00+0+28) = AX1;
    DM(a_TrigTrack00+0+36) = AX0;
    DM(a_VolTrack00+0+36) = AX1;
    DM(a_TrigTrack00+0+48) = AX0;
    DM(a_VolTrack00+0+48) = AX1;
    DM(a_TrigTrack00+0+56) = AX0;
    DM(a_VolTrack00+0+56) = AX1;

    /* inits for b */
    AX0 = 100;
    AX1 = 0x2000;
    DM(a_TrigTrack01+0+8) = AX0;
    DM(a_VolTrack01+0+8) = AX1;
    DM(a_TrigTrack01+0+24) = AX0;
    DM(a_VolTrack01+0+24) = AX1;
    DM(a_TrigTrack01+0+40) = AX0;
    DM(a_VolTrack01+0+40) = AX1;
    DM(a_TrigTrack01+0+52) = AX0;
    DM(a_VolTrack01+0+52) = AX1;

    /* inits for c */
    AX0 = 100;
    AX1 = 0x1000;
    DM(a_TrigTrack00+0+6) = AX0;
    DM(a_VolTrack00+0+6) = AX1;
    DM(a_TrigTrack00+0+14) = AX0;
    DM(a_VolTrack00+0+14) = AX1;
    DM(a_TrigTrack00+0+20) = AX0;
    DM(a_VolTrack00+0+20) = AX1;
    DM(a_TrigTrack00+0+42) = AX0;
    DM(a_VolTrack00+0+42) = AX1;
    DM(a_TrigTrack00+0+60) = AX0;
    DM(a_VolTrack00+0+60) = AX1;

/* donotparse(CLEAR, 0, 0,  a--- --c- ---- --c-  ---- c--- ---- a---   ---- a--- --c- ----  a--- ---- a--- c---,
            CLEAR, 0, 1,  ---- ---- b--- ----  ---- ---- b--- ----   ---- ---- b--- ----  ---- b--- ---- ----,
            a, 100, 0x3000,
            b, 100, 0x2000,
            c, 100, 0x1000,
            END); */

    FULLTRACK(T_BD0);
    FULLTRACK(T_BD1);

    jump MultiMeasureCheck;

Measure82:
/*
 *  autogenerated code for SETTRACK macro
 */
    I2 = ^a_TrigTrack07;
    I3 = ^a_VolTrack07;
    CNTR = 128;
    do CL07516 until CE;
        DM(I2, M1) = 0;
CL07516:   DM(I3, M1) = 0;

    I2 = ^a_TrigTrack08;
    I3 = ^a_VolTrack08;
    CNTR = 128;
    do CL08516 until CE;
        DM(I2, M1) = 0;
CL08516:   DM(I3, M1) = 0;

    /* inits for f */
    AX0 = 100;
    AX1 = 0x1c00;
    DM(a_TrigTrack07+0+2) = AX0;
    DM(a_VolTrack07+0+2) = AX1;
    DM(a_TrigTrack08+0+2) = AX0;
    DM(a_VolTrack08+0+2) = AX1;
    DM(a_TrigTrack07+0+4) = AX0;
    DM(a_VolTrack07+0+4) = AX1;
    DM(a_TrigTrack08+0+4) = AX0;
    DM(a_VolTrack08+0+4) = AX1;
    DM(a_TrigTrack07+0+12) = AX0;
    DM(a_VolTrack07+0+12) = AX1;
    DM(a_TrigTrack08+0+12) = AX0;
    DM(a_VolTrack08+0+12) = AX1;
    DM(a_TrigTrack07+0+18) = AX0;
    DM(a_VolTrack07+0+18) = AX1;
    DM(a_TrigTrack08+0+18) = AX0;
    DM(a_VolTrack08+0+18) = AX1;
    DM(a_TrigTrack07+0+20) = AX0;
    DM(a_VolTrack07+0+20) = AX1;
    DM(a_TrigTrack08+0+20) = AX0;
    DM(a_VolTrack08+0+20) = AX1;
    DM(a_TrigTrack07+0+22) = AX0;
    DM(a_VolTrack07+0+22) = AX1;
    DM(a_TrigTrack08+0+22) = AX0;
    DM(a_VolTrack08+0+22) = AX1;
    DM(a_TrigTrack07+0+26) = AX0;
    DM(a_VolTrack07+0+26) = AX1;
    DM(a_TrigTrack08+0+26) = AX0;
    DM(a_VolTrack08+0+26) = AX1;
    DM(a_TrigTrack07+0+28) = AX0;
    DM(a_VolTrack07+0+28) = AX1;
    DM(a_TrigTrack08+0+28) = AX0;
    DM(a_VolTrack08+0+28) = AX1;
    DM(a_TrigTrack07+0+30) = AX0;
    DM(a_VolTrack07+0+30) = AX1;
    DM(a_TrigTrack08+0+30) = AX0;
    DM(a_VolTrack08+0+30) = AX1;
    DM(a_TrigTrack07+0+31) = AX0;
    DM(a_VolTrack07+0+31) = AX1;
    DM(a_TrigTrack08+0+31) = AX0;
    DM(a_VolTrack08+0+31) = AX1;
    DM(a_TrigTrack07+0+34) = AX0;
    DM(a_VolTrack07+0+34) = AX1;
    DM(a_TrigTrack08+0+34) = AX0;
    DM(a_VolTrack08+0+34) = AX1;
    DM(a_TrigTrack07+0+36) = AX0;
    DM(a_VolTrack07+0+36) = AX1;
    DM(a_TrigTrack08+0+36) = AX0;
    DM(a_VolTrack08+0+36) = AX1;
    DM(a_TrigTrack07+0+37) = AX0;
    DM(a_VolTrack07+0+37) = AX1;
    DM(a_TrigTrack08+0+37) = AX0;
    DM(a_VolTrack08+0+37) = AX1;
    DM(a_TrigTrack07+0+38) = AX0;
    DM(a_VolTrack07+0+38) = AX1;
    DM(a_TrigTrack08+0+38) = AX0;
    DM(a_VolTrack08+0+38) = AX1;
    DM(a_TrigTrack07+0+40) = AX0;
    DM(a_VolTrack07+0+40) = AX1;
    DM(a_TrigTrack08+0+40) = AX0;
    DM(a_VolTrack08+0+40) = AX1;
    DM(a_TrigTrack07+0+44) = AX0;
    DM(a_VolTrack07+0+44) = AX1;
    DM(a_TrigTrack08+0+44) = AX0;
    DM(a_VolTrack08+0+44) = AX1;
    DM(a_TrigTrack07+0+50) = AX0;
    DM(a_VolTrack07+0+50) = AX1;
    DM(a_TrigTrack08+0+50) = AX0;
    DM(a_VolTrack08+0+50) = AX1;
    DM(a_TrigTrack07+0+51) = AX0;
    DM(a_VolTrack07+0+51) = AX1;
    DM(a_TrigTrack08+0+51) = AX0;
    DM(a_VolTrack08+0+51) = AX1;
    DM(a_TrigTrack07+0+54) = AX0;
    DM(a_VolTrack07+0+54) = AX1;
    DM(a_TrigTrack08+0+54) = AX0;
    DM(a_VolTrack08+0+54) = AX1;
    DM(a_TrigTrack07+0+56) = AX0;
    DM(a_VolTrack07+0+56) = AX1;
    DM(a_TrigTrack08+0+56) = AX0;
    DM(a_VolTrack08+0+56) = AX1;
    DM(a_TrigTrack07+0+60) = AX0;
    DM(a_VolTrack07+0+60) = AX1;
    DM(a_TrigTrack08+0+60) = AX0;
    DM(a_VolTrack08+0+60) = AX1;
    DM(a_TrigTrack07+0+62) = AX0;
    DM(a_VolTrack07+0+62) = AX1;
    DM(a_TrigTrack08+0+62) = AX0;
    DM(a_VolTrack08+0+62) = AX1;

    /* inits for a */
    AX0 = 100;
    AX1 = 0x0200;
    DM(a_TrigTrack07+0+3) = AX0;
    DM(a_VolTrack07+0+3) = AX1;
    DM(a_TrigTrack08+0+3) = AX0;
    DM(a_VolTrack08+0+3) = AX1;
    DM(a_TrigTrack07+0+5) = AX0;
    DM(a_VolTrack07+0+5) = AX1;
    DM(a_TrigTrack08+0+5) = AX0;
    DM(a_VolTrack08+0+5) = AX1;
    DM(a_TrigTrack07+0+21) = AX0;
    DM(a_VolTrack07+0+21) = AX1;
    DM(a_TrigTrack08+0+21) = AX0;
    DM(a_VolTrack08+0+21) = AX1;
    DM(a_TrigTrack07+0+52) = AX0;
    DM(a_VolTrack07+0+52) = AX1;
    DM(a_TrigTrack08+0+52) = AX0;
    DM(a_VolTrack08+0+52) = AX1;
    DM(a_TrigTrack07+0+55) = AX0;
    DM(a_VolTrack07+0+55) = AX1;
    DM(a_TrigTrack08+0+55) = AX0;
    DM(a_VolTrack08+0+55) = AX1;
    DM(a_TrigTrack07+0+61) = AX0;
    DM(a_VolTrack07+0+61) = AX1;
    DM(a_TrigTrack08+0+61) = AX0;
    DM(a_VolTrack08+0+61) = AX1;

    /* inits for b */
    AX0 = 100;
    AX1 = 0x0300;
    DM(a_TrigTrack07+0+8) = AX0;
    DM(a_VolTrack07+0+8) = AX1;
    DM(a_TrigTrack08+0+8) = AX0;
    DM(a_VolTrack08+0+8) = AX1;
    DM(a_TrigTrack07+0+9) = AX0;
    DM(a_VolTrack07+0+9) = AX1;
    DM(a_TrigTrack08+0+9) = AX0;
    DM(a_VolTrack08+0+9) = AX1;
    DM(a_TrigTrack07+0+11) = AX0;
    DM(a_VolTrack07+0+11) = AX1;
    DM(a_TrigTrack08+0+11) = AX0;
    DM(a_VolTrack08+0+11) = AX1;
    DM(a_TrigTrack07+0+57) = AX0;
    DM(a_VolTrack07+0+57) = AX1;
    DM(a_TrigTrack08+0+57) = AX0;
    DM(a_VolTrack08+0+57) = AX1;

    /* inits for c */
    AX0 = 100;
    AX1 = 0x400;
    DM(a_TrigTrack07+0+13) = AX0;
    DM(a_VolTrack07+0+13) = AX1;
    DM(a_TrigTrack08+0+13) = AX0;
    DM(a_VolTrack08+0+13) = AX1;
    DM(a_TrigTrack07+0+15) = AX0;
    DM(a_VolTrack07+0+15) = AX1;
    DM(a_TrigTrack08+0+15) = AX0;
    DM(a_VolTrack08+0+15) = AX1;

    /* inits for g */
    AX0 = 100;
    AX1 = 0x3000;
    DM(a_TrigTrack07+0+0) = AX0;
    DM(a_VolTrack07+0+0) = AX1;
    DM(a_TrigTrack08+0+0) = AX0;
    DM(a_VolTrack08+0+0) = AX1;
    DM(a_TrigTrack07+0+16) = AX0;
    DM(a_VolTrack07+0+16) = AX1;
    DM(a_TrigTrack08+0+16) = AX0;
    DM(a_VolTrack08+0+16) = AX1;
    DM(a_TrigTrack07+0+32) = AX0;
    DM(a_VolTrack07+0+32) = AX1;
    DM(a_TrigTrack08+0+32) = AX0;
    DM(a_VolTrack08+0+32) = AX1;
    DM(a_TrigTrack07+0+48) = AX0;
    DM(a_VolTrack07+0+48) = AX1;
    DM(a_TrigTrack08+0+48) = AX0;
    DM(a_VolTrack08+0+48) = AX1;

/* donotparse(CLEAR, 0, 7,  g-fa fa-- bb-b fc-c  g-f- faf- --f- f-ff   g-f- fff- f--- f---  g-ff a-fa fb-- faf-,
            CLEAR, 0, 8,  g-fa fa-- bb-b fc-c  g-f- faf- --f- f-ff   g-f- fff- f--- f---  g-ff a-fa fb-- faf-,
            f, 100, 0x1c00,
            a, 100, 0x0200,
            b, 100, 0x0300,
            c, 100, 0x400,
            g, 100, 0x3000,
            END); */

    FULLTRACK(T_HH2L);
    FULLTRACK(T_HH2R);

    jump MultiMeasureCheck;

Measure84:
    MUTETRACK(T_BD0);
    MUTETRACK(T_BD1);
    MUTETRACK(T_HH1L);
    MUTETRACK(T_HH1R);
    MUTETRACK(T_HH2L);
    MUTETRACK(T_HH2R);
    
    jump MultiMeasureCheck;

Measure88:

/*
 *  autogenerated code for SETTRACK macro
 */
    I2 = ^a_TrigTrack12;
    I3 = ^a_VolTrack12;
    CNTR = 128;
    do CL12542 until CE;
        DM(I2, M1) = 0;
CL12542:   DM(I3, M1) = 0;

    I2 = ^a_TrigTrack13;
    I3 = ^a_VolTrack13;
    CNTR = 128;
    do CL13542 until CE;
        DM(I2, M1) = 0;
CL13542:   DM(I3, M1) = 0;

    /* inits for k */
    AX0 = 100;
    AX1 = 0x0e00;
    DM(a_TrigTrack12+0+0) = AX0;
    DM(a_VolTrack12+0+0) = AX1;

    /* inits for l */
    AX0 = 100;
    AX1 = 0x0900;
    DM(a_TrigTrack13+0+0) = AX0;
    DM(a_VolTrack13+0+0) = AX1;

/* donotparse(CLEAR, 0, 12, k--- ---- ---- ----  ---- ---- ---- ----   ---- ---- ---- ----  ---- ---- ---- ----,
            CLEAR, 0, 13, l--- ---- ---- ----  ---- ---- ---- ----   ---- ---- ---- ----  ---- ---- ---- ----,
            k, 100, 0x0e00,
            l, 100, 0x0900,
            END); */

    MUTETRACK(T_HH0L);
    MUTETRACK(T_HH0R);
    MUTETRACK(T_FM0);
    MUTETRACK(T_FM1);
  
    jump MultiMeasureCheck;

Measure89:
    set FL1;
    MUTETRACK(T_PAD0L);
    MUTETRACK(T_PAD0R);
    
    jump MultiMeasureCheck;
    

DoNothing:
    jump MultiMeasureCheck;

.ENDMOD;
