/******************************************************************************
*
*  GenFXIni.dsp
*
*  (c) 2001, 2002, 2003 ethan bordeaux
*  syndevkit@dspmusic.org
*
*  initialization of all generators, fx, envelopes and sequencer.  called at
*  the beginning of the song before audio output.  extensive post processing
*  and analysis occurs on this function, so follow the guidelines on usage as
*  outlined in the documentation.
*
******************************************************************************/

.MODULE                                     modGenFXIni_00;

/*
 *  include files for all functions
 */
 

#include    ".\comsys\hdr\DSPdefs.h"
#include    ".\gen_fx\hdr\GenFX.h"
#include    "seqtest.h"

/*
 *  external variables and functions
 */

/*
 *  entry points
 */

.ENTRY      InitGMParams_00;
.ENTRY      EndInitGMParams_00;

InitGMParams_00:
    /* insert generators and fx initializations here */

    SETPTR(a_WTGen2);
    INIT_WTGEN2(200, ^a_WTSine);
    
    SETPTR(a_ExpDecayEnv);
    INIT_EXPDECAYENV(128, 0x7fff, 0x4000);
    
    SETPTR(a_ClampFX);
    INIT_CLAMPFX(0x4000, 0x0000);
    
    SETPTR(a_ZeroSampsFX);
    INIT_ZEROSAMPSFX(0x0010, 0x0000);
    
    SETPTR(a_RectifyFX);
    INIT_RECTIFYFX(NEG_RECTIFY, 0x0080);
            
    SETPTR(a_MemEnv3);
    INIT_MEMENV3(^a_RectifyFX+(0*RECTIFYFX_VARS)+RECTIFYFX_MIX, 0x0000, 0x4000, 640, 0x0000, 640);
    
    SETPTR(ap_CTRLTrack00);
    NUMCTRLTRACKS(NUMCTRLTRACK00);
    CONTROLTRACKSTEP(STEPONRETRIG);
    INIT_AP_CTRLTRACK(LENCTRLTRACK00_0, ^a_CTRLTrack00_0);
    INIT_AP_CTRLTRACK(LENCTRLTRACK00_1, ^a_CTRLTrack00_1);

    I2 = ^a_CTRLTrack00_0;
    AR = DM(a_MIDIFreq+C_3);
    DM(I2, M0) = AR;
    
    I2 = ^a_CTRLTrack00_0+1;
    AR = DM(a_MIDIFreq+G_3);
    DM(I2, M0) = AR;
    
    I2 = ^a_CTRLTrack00_1;
    AR = 128;
    DM(I2, M0) = AR;
    
    I2 = ^a_CTRLTrack00_1+1;
    AR = 128;
    DM(I2, M0) = AR;
    
/*
 *  init sequencer array
 *  
 *  param0:  tic period
 *  param1:  swing period (num tics between slow/fast transition)
 *  param2:  swing amount
 *  param3:  sequencer length
 *  param4:  ptr to TrigInit function
 *  param5:  envelope type
 *  param6:  aux function ptr
 *
 */

    SETPTR(a_Seq);
    modify(I2, M1);                         /* numtrax set by CalcSeq2Tracks */
    INIT_SEQ(320, 0, 0, 8, EXPDECAYENV);

    /* must be called after Seq2 init to set env and num tracks */
    call Seq2PostProc;
    
    /* set v_TicsPerMeasure to length of first sequence */
    SEQ_SET_MEASURE(0);

    AR = ^InitGMParams_00;
    AX0 = ^EndInitGMParams_00;
    call FillGenFXPtrInits;
    
    /* init ^Env and set length of array of ptrs used in GenFXPtrInits */
    call GenFXIniPostProc;                  /* do not remove! */
    
EndInitGMParams_00:
    rts;

.ENDMOD;
