#!/usr/bin/perl
 
$projname = $ARGV[0];
$projpath = '..\\songs\\'.$projname.'\\';

# @files holds filelist with full path
@files = glob "$projpath\*.dsp";

# @snipfile holds filelist without path
$sniplen = length($projpath);
foreach $fileline (@files)
{
    $snipfile[$i++] = substr($fileline, $sniplen);
}

$parsedcntr = 0;

##
 #  main proc loop here!
 ##

$filecntr = 0;
foreach $fileline (@files)
{

    # open real .dsp file, copy to @dspcode
    open(DSPREAD, $files[$filecntr]);
    @dspcode = <DSPREAD>;

    # output written to parsed_filename.dsp        
    open(DSPWRITE, ">$projpath"."parsed_"."$snipfile[$filecntr]");

    #print "now processing $snipfile[$filecntr], writing to parsed_$snipfile[$filecntr]...\n";
    
    $foundsettrack = 0;
    $line = 0;
        
    foreach $dspline (@dspcode)
    {
        # find SETTRACK in .dsp file
        if($dspline =~ /SETTRACK\(/)
        {
            $dspline =~ s/SETTRACK\(/donotparse\(/;
            $foundsettrack = 1;
            
            # found start of SETTRACK, read in lines of macro into @trackparsecode
            $macroline = ($line-1);
            $tpcline = 0;
            do {
                $trackparsecode[$tpcline++] = $dspcode[$macroline++];
            } while ($dspcode[$macroline] !~ /;/);
                    
            # get last lines of SETTTRACK
            $trackparsecode[$tpcline++] = $dspcode[$macroline++];
            $trackparsecode[$tpcline] = $dspcode[$macroline];
                                
            # trackparsecode now holds macro
            # set variables used in parse to "bad" values.  then if they are not
            # set properly after macro is parsed, return a failure code
                            
            for($i=0; $i<32; $i++) {
                $cleartrack[$i] = -1;           # init all Trig/Vol to zero?
                $offset[$i] = -1;               # offset into Trig/Vol tracks
            }
            
            &cleartrackdata;
            
            $trackvolinit = 0;
                            
            # read through $trackparsecode, call proper parser
            for($j=0;  $j<($tpcline-1); $j++) {
                if($trackparsecode[$j] =~ /NOCLEAR/ ||
                   $trackparsecode[$j] =~ /CLEAR/)
                {
                    # parse tics, CLEAR/NOCLEAR, offset, track number
                    &trackdataparse;
                }
                # make sure not parsing a blank line
                elsif($trackparsecode[$j] =~ /,/ )
                {
                    # parse TrigInit/VolInit
                    &trigvoldataparse($trackvolinit);
                    $trackvolinit++;
                }
            }

##
 #  all necessary data stored in the following:
 #
 #  $cleartrack:      execute init code or not (also used to determine if track
 #                    is init'ed at all, check for -1).
 #  $offset:          offset from start of trig/vol track
 #  $trigvolmarker:   holds info on what to init at which location
 #  $trigvartype:     placeholder name (ex: a, b, c etc)
 #  $trigtrackval:    TrigTrack value associated with $trigvartype
 #  $voltrackval:     VolTrack value associated with $trigvartype
 #
 ##

            print DSPWRITE "/*\n";
            print DSPWRITE " *  autogenerated code for SETTRACK macro\n";
            print DSPWRITE " */\n";
            
            # insert code for clearing tracks
            &cleartrackcheck;
            
            # loop over all TrigTrack/VolTrack init (set at end of macro)
            for($i=0; $i<$trackvolinit; $i++) {
            
                $temptrigtrack = @trigtrackval[$i];
                $tempvoltrack = @voltrackval[$i];
            
                print DSPWRITE "    /* inits for $trigvartype[$i] */\n";
                print DSPWRITE "    AX0 = $temptrigtrack;\n";
                print DSPWRITE "    AX1 = $tempvoltrack;\n";
                
                # search each trigvolmarker for reference to trigvartype
                &writetrackinits;
            }
        }
        # if no SETTRACK found, just write output to file
        print DSPWRITE "$dspline";
        $line++;
    }
    # close current files
    close(DSPREAD);
    close(DSPWRITE);
    
    # if SETTRACK not in file, delete parsed_ version.
    if($foundsettrack == 0) {
        unlink "$projpath"."parsed_"."$snipfile[$filecntr]";
        print "SETTRACK not found in $snipfile[$filecntr]...\n";
    } else {
        print "$snipfile[$filecntr] processed, SETTRACK found.  writing parsed_$snipfile[$filecntr]...\n";
        $parsedlist[$parsedcntr] = $snipfile[$filecntr];
        $parsedcntr++;
    }

    $filecntr++;
}

$oldmak = "$projpath"."$projname".".mak";

open(MAKREAD, "$projpath"."$projname".".mak")
    or die "\n  $oldmak doesn't exist!\n";

$newmak = "$projpath"."$projname".".tmp";
open(MAKWRITE, ">$projpath"."$projname".".tmp");

@makfile = <MAKREAD>;

foreach $makline (@makfile)
{
    for($x=0; $x<$parsedcntr; $x++)
    {
        if($makline =~ /$parsedlist[$x]/)
        {
            if($makline !~ /parsed_/)
            {
                if($makline =~ /\\/)
                {
                    $makline = "\t\tparsed_$parsedlist[$x] \\\n";
                }
                else
                {
                    $makline = "\t\tparsed_$parsedlist[$x]\n";
                }
            }
        }
    }

print MAKWRITE "$makline";
}

close($newmak);
close($oldmak);

print "deleting $oldmak\n";
#unlink "$projpath"."$projname".".mak";

print "$projname";

$test = "$projpath"."ex3"."";
#unlink $test;
#rename $newmak, $oldmak;

print "\n done processing $projname.\n";

################################################################################
#                                                                              #
#                                subroutines!                                  #
#                                                                              #
################################################################################

##
 #  trackdataparse:  parse lines that define how Trig/Vol init should happen,
 #                   write into appropriate arrays
 ##

sub trackdataparse
{
    local(@tvdparams);
    # parse/separate current row of data
    @tvdparams = parse_csv($trackparsecode[$j]);
    $track = $tvdparams[2];
        
    # clear or don't clear Trig/Vol track arrays
    if($trackparsecode[$j] =~ /NOCLEAR/) {
        $cleartrack[$track] = 0;
    } else {
        $cleartrack[$track] = 1;
    }
    
    # offset into track
    $offset[$track] = trimspace($tvdparams[1]);
    
    # parse init array
    $tvdparams[3] = trimspace($tvdparams[3]);
        
    $step = 0;
    for($i=0; $i<length($tvdparams[3]); $i++)
    {
        $tmp = substr($tvdparams[3], $i, 1);
        if ($tmp ne " ")
        {
            $temptrigvolmarker[$step++] = $tmp;
        }
    }
    
    # i wish i was smarter than this.  must enumerate all track possibilities.
    if     ($track == 0)  { @trigvolmarker00 = @temptrigvolmarker;
    } elsif($track == 1)  { @trigvolmarker01 = @temptrigvolmarker;
    } elsif($track == 2)  { @trigvolmarker02 = @temptrigvolmarker;
    } elsif($track == 3)  { @trigvolmarker03 = @temptrigvolmarker;
    } elsif($track == 4)  { @trigvolmarker04 = @temptrigvolmarker;
    } elsif($track == 5)  { @trigvolmarker05 = @temptrigvolmarker;
    } elsif($track == 6)  { @trigvolmarker06 = @temptrigvolmarker;
    } elsif($track == 7)  { @trigvolmarker07 = @temptrigvolmarker;
    } elsif($track == 8)  { @trigvolmarker08 = @temptrigvolmarker;
    } elsif($track == 9)  { @trigvolmarker09 = @temptrigvolmarker;
    } elsif($track == 10) { @trigvolmarker10 = @temptrigvolmarker;
    } elsif($track == 11) { @trigvolmarker11 = @temptrigvolmarker;
    } elsif($track == 12) { @trigvolmarker12 = @temptrigvolmarker;
    } elsif($track == 13) { @trigvolmarker13 = @temptrigvolmarker;
    } elsif($track == 14) { @trigvolmarker14 = @temptrigvolmarker;
    } elsif($track == 15) { @trigvolmarker15 = @temptrigvolmarker;
    } elsif($track == 16) { @trigvolmarker16 = @temptrigvolmarker;
    } elsif($track == 17) { @trigvolmarker17 = @temptrigvolmarker;
    } elsif($track == 18) { @trigvolmarker18 = @temptrigvolmarker;
    } elsif($track == 19) { @trigvolmarker19 = @temptrigvolmarker;
    } elsif($track == 20) { @trigvolmarker20 = @temptrigvolmarker;
    } elsif($track == 21) { @trigvolmarker21 = @temptrigvolmarker;
    } elsif($track == 22) { @trigvolmarker22 = @temptrigvolmarker;
    } elsif($track == 23) { @trigvolmarker23 = @temptrigvolmarker;
    } elsif($track == 24) { @trigvolmarker24 = @temptrigvolmarker;
    } elsif($track == 25) { @trigvolmarker25 = @temptrigvolmarker;
    } elsif($track == 26) { @trigvolmarker26 = @temptrigvolmarker;
    } elsif($track == 27) { @trigvolmarker27 = @temptrigvolmarker;
    } elsif($track == 28) { @trigvolmarker28 = @temptrigvolmarker;
    } elsif($track == 29) { @trigvolmarker29 = @temptrigvolmarker;
    } elsif($track == 30) { @trigvolmarker30 = @temptrigvolmarker;
    } elsif($track == 31) { @trigvolmarker31 = @temptrigvolmarker;
    }
        
}

##
 #  trigvoldataparse:  parse actual Trig/Vol data
 ##

sub trigvoldataparse
{
    
    local(@tvdparams);    
        
    @tvdparams = parse_csv($trackparsecode[$j]);
    
    $trigvartype[$_[0]] = trimspace($tvdparams[0]);
    $trigtrackval[$_[0]] = trimspace($tvdparams[1]);
    $voltrackval[$_[0]] = trimspace($tvdparams[2]);
            
}
    
##
 #  trimspace:  trim preceding/following spaces around array
 ##

sub trimspace
{
    my @out = @_;
    for (@out) {
        s/^\s+//;
        s/\s+$//;
    }
    return wantarray ? @out : $out[0];
}

##
 #  parse_csv:  parse line of data, write to array of arrays
 ##

sub parse_csv
{
    my $text = shift;
    my @new = ();
    push(@new, $+) while $text =~ m{
        # the first part groups the phrase inside the quotes
        "([^\"\\]*(?:\\.[^\"\\]*)*)",?
            |   ([^,]+),?
            |   ,
        }gx;
    push(@new, undef) if substr($text, -1, 1) eq ',';
    return @new;
}

##
 #  writetrackinits:  check the trigvolmarker arrays for inits of a trigvartype
 #                    type.  write code to perform actual init if they exist.
 ##

sub writetrackinits
{
    for($j=0; $j<128; $j++) {
        if($trigvolmarker00[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack00+$offset[0]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack00+$offset[0]+$j) = AX1;\n";
        }
        if($trigvolmarker01[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack01+$offset[1]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack01+$offset[1]+$j) = AX1;\n";
        }
        if($trigvolmarker02[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack02+$offset[2]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack02+$offset[2]+$j) = AX1;\n";
        }
        if($trigvolmarker03[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack03+$offset[3]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack03+$offset[3]+$j) = AX1;\n";
        }
        if($trigvolmarker04[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack04+$offset[4]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack04+$offset[4]+$j) = AX1;\n";
        }
        if($trigvolmarker05[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack05+$offset[5]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack05+$offset[5]+$j) = AX1;\n";
        }
        if($trigvolmarker06[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack06+$offset[6]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack06+$offset[6]+$j) = AX1;\n";
        }
        if($trigvolmarker07[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack07+$offset[7]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack07+$offset[7]+$j) = AX1;\n";
        }
        if($trigvolmarker08[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack08+$offset[8]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack08+$offset[8]+$j) = AX1;\n";
        }
        if($trigvolmarker09[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack09+$offset[9]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack09+$offset[9]+$j) = AX1;\n";
        }
        if($trigvolmarker10[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack10+$offset[10]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack10+$offset[10]+$j) = AX1;\n";
        }
        if($trigvolmarker11[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack11+$offset[11]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack11+$offset[11]+$j) = AX1;\n";
        }
        if($trigvolmarker12[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack12+$offset[12]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack12+$offset[12]+$j) = AX1;\n";
        }
        if($trigvolmarker13[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack13+$offset[13]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack13+$offset[13]+$j) = AX1;\n";
        }
        if($trigvolmarker14[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack14+$offset[14]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack14+$offset[14]+$j) = AX1;\n";
        }
        if($trigvolmarker15[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack15+$offset[15]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack15+$offset[15]+$j) = AX1;\n";
        }
        if($trigvolmarker16[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack16+$offset[16]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack16+$offset[16]+$j) = AX1;\n";
        }
        if($trigvolmarker17[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack17+$offset[17]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack17+$offset[17]+$j) = AX1;\n";
        }
        if($trigvolmarker18[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack18+$offset[18]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack18+$offset[18]+$j) = AX1;\n";
        }
        if($trigvolmarker19[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack19+$offset[19]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack19+$offset[19]+$j) = AX1;\n";
        }
        if($trigvolmarker20[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack20+$offset[20]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack20+$offset[20]+$j) = AX1;\n";
        }
        if($trigvolmarker21[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack21+$offset[21]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack21+$offset[21]+$j) = AX1;\n";
        }
        if($trigvolmarker22[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack22+$offset[22]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack22+$offset[22]+$j) = AX1;\n";
        }
        if($trigvolmarker23[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack23+$offset[23]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack23+$offset[23]+$j) = AX1;\n";
        }
        if($trigvolmarker24[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack24+$offset[24]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack24+$offset[24]+$j) = AX1;\n";
        }
        if($trigvolmarker25[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack25+$offset[25]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack25+$offset[25]+$j) = AX1;\n";
        }
        if($trigvolmarker26[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack26+$offset[26]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack26+$offset[26]+$j) = AX1;\n";
        }
        if($trigvolmarker27[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack27+$offset[27]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack27+$offset[27]+$j) = AX1;\n";
        }
        if($trigvolmarker28[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack28+$offset[28]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack28+$offset[28]+$j) = AX1;\n";
        }
        if($trigvolmarker29[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack29+$offset[29]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack29+$offset[29]+$j) = AX1;\n";
        }
        if($trigvolmarker30[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack30+$offset[30]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack30+$offset[30]+$j) = AX1;\n";
        }
        if($trigvolmarker31[$j] eq $trigvartype[$i]) {
            print DSPWRITE "    DM(a_TrigTrack31+$offset[31]+$j) = AX0;\n";
            print DSPWRITE "    DM(a_VolTrack31+$offset[31]+$j) = AX1;\n";
        }
    }
    print DSPWRITE "\n";
}

sub cleartrackcheck
{
    if($cleartrack[0] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack00;\n";
        print DSPWRITE "    I3 = ^a_VolTrack00;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('00');
    }
    if($cleartrack[1] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack01;\n";
        print DSPWRITE "    I3 = ^a_VolTrack01;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('01');
    }
    if($cleartrack[2] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack02;\n";
        print DSPWRITE "    I3 = ^a_VolTrack02;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('02');
    }
    if($cleartrack[3] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack03;\n";
        print DSPWRITE "    I3 = ^a_VolTrack03;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('03');
    }
    if($cleartrack[4] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack04;\n";
        print DSPWRITE "    I3 = ^a_VolTrack04;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('04');
    }
    if($cleartrack[5] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack05;\n";
        print DSPWRITE "    I3 = ^a_VolTrack05;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('05');
    }
    if($cleartrack[6] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack06;\n";
        print DSPWRITE "    I3 = ^a_VolTrack06;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('06');
    }
    if($cleartrack[7] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack07;\n";
        print DSPWRITE "    I3 = ^a_VolTrack07;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('07');
    }
    if($cleartrack[8] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack08;\n";
        print DSPWRITE "    I3 = ^a_VolTrack08;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('08');
    }
    if($cleartrack[9] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack09;\n";
        print DSPWRITE "    I3 = ^a_VolTrack09;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('09');
    }
    if($cleartrack[10] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack10;\n";
        print DSPWRITE "    I3 = ^a_VolTrack10;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('10');
    }
    if($cleartrack[11] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack11;\n";
        print DSPWRITE "    I3 = ^a_VolTrack11;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('11');
    }
    if($cleartrack[12] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack12;\n";
        print DSPWRITE "    I3 = ^a_VolTrack12;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('12');
    }
    if($cleartrack[13] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack13;\n";
        print DSPWRITE "    I3 = ^a_VolTrack13;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('13');
    }
    if($cleartrack[14] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack14;\n";
        print DSPWRITE "    I3 = ^a_VolTrack14;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('14');
    }
    if($cleartrack[15] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack15;\n";
        print DSPWRITE "    I3 = ^a_VolTrack15;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('15');
    }
    if($cleartrack[16] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack16;\n";
        print DSPWRITE "    I3 = ^a_VolTrack16;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('16');
    }
    if($cleartrack[17] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack17;\n";
        print DSPWRITE "    I3 = ^a_VolTrack17;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('17');
    }
    if($cleartrack[18] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack18;\n";
        print DSPWRITE "    I3 = ^a_VolTrack18;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('18');
    }
    if($cleartrack[19] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack19;\n";
        print DSPWRITE "    I3 = ^a_VolTrack19;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('19');
    }
    if($cleartrack[20] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack20;\n";
        print DSPWRITE "    I3 = ^a_VolTrack20;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('20');
    }
    if($cleartrack[21] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack21;\n";
        print DSPWRITE "    I3 = ^a_VolTrack21;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('21');
    }
    if($cleartrack[22] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack22;\n";
        print DSPWRITE "    I3 = ^a_VolTrack22;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('22');
    }
    if($cleartrack[23] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack23;\n";
        print DSPWRITE "    I3 = ^a_VolTrack23;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('23');
    }
    if($cleartrack[24] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack24;\n";
        print DSPWRITE "    I3 = ^a_VolTrack24;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('24');
    }
    if($cleartrack[25] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack25;\n";
        print DSPWRITE "    I3 = ^a_VolTrack25;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('25');
    }
    if($cleartrack[26] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack26;\n";
        print DSPWRITE "    I3 = ^a_VolTrack26;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('26');
    }
    if($cleartrack[27] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack27;\n";
        print DSPWRITE "    I3 = ^a_VolTrack27;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('27');
    }
    if($cleartrack[28] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack28;\n";
        print DSPWRITE "    I3 = ^a_VolTrack28;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('28');
    }
    if($cleartrack[29] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack29;\n";
        print DSPWRITE "    I3 = ^a_VolTrack29;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('29');
    }
    if($cleartrack[30] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack30;\n";
        print DSPWRITE "    I3 = ^a_VolTrack30;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('30');
    }
    if($cleartrack[31] == 1) {
        print DSPWRITE "    I2 = ^a_TrigTrack31;\n";
        print DSPWRITE "    I3 = ^a_VolTrack31;\n";
        print DSPWRITE "    CNTR = 128;\n";
        &clearloop('31');
    }
}

sub clearloop
{
    print DSPWRITE "    do CL$_[0]$line until CE;\n";
    print DSPWRITE "        DM(I2, M1) = 0;\n";
    print DSPWRITE "CL$_[0]$line:   DM(I3, M1) = 0;\n";
    print DSPWRITE "\n";
}

sub cleartrackdata
{
    for($i=0; $i<128; $i++) {
        $trigvolmarker00[$i] = '-';
        $trigvolmarker01[$i] = '-';
        $trigvolmarker02[$i] = '-';
        $trigvolmarker03[$i] = '-';
        $trigvolmarker04[$i] = '-';
        $trigvolmarker05[$i] = '-';
        $trigvolmarker06[$i] = '-';
        $trigvolmarker07[$i] = '-';
        $trigvolmarker08[$i] = '-';
        $trigvolmarker09[$i] = '-';
        $trigvolmarker10[$i] = '-';
        $trigvolmarker11[$i] = '-';
        $trigvolmarker12[$i] = '-';
        $trigvolmarker13[$i] = '-';
        $trigvolmarker14[$i] = '-';
        $trigvolmarker15[$i] = '-';
        $trigvolmarker16[$i] = '-';
        $trigvolmarker17[$i] = '-';
        $trigvolmarker18[$i] = '-';
        $trigvolmarker19[$i] = '-';
        $trigvolmarker20[$i] = '-';
        $trigvolmarker21[$i] = '-';
        $trigvolmarker22[$i] = '-';
        $trigvolmarker23[$i] = '-';
        $trigvolmarker24[$i] = '-';
        $trigvolmarker25[$i] = '-';
        $trigvolmarker26[$i] = '-';
        $trigvolmarker27[$i] = '-';
        $trigvolmarker28[$i] = '-';
        $trigvolmarker29[$i] = '-';
        $trigvolmarker30[$i] = '-';
        $trigvolmarker31[$i] = '-';
        $temptrigvolmarker[$i] = '-';
    }
}